<?php

namespace PHPMaker2024\cidp24;

use Doctrine\DBAL\ParameterType;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Query\QueryBuilder;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Container\ContainerInterface;
use Slim\Routing\RouteCollectorProxy;
use Slim\App;
use Closure;

/**
 * Page class
 */
class CidpDataList extends CidpData
{
    use MessagesTrait;

    // Page ID
    public $PageID = "list";

    // Project ID
    public $ProjectID = PROJECT_ID;

    // Page object name
    public $PageObjName = "CidpDataList";

    // View file path
    public $View = null;

    // Title
    public $Title = null; // Title for <title> tag

    // Rendering View
    public $RenderingView = false;

    // Grid form hidden field names
    public $FormName = "fcidp_datalist";
    public $FormActionName = "";
    public $FormBlankRowName = "";
    public $FormKeyCountName = "";

    // CSS class/style
    public $CurrentPageName = "CidpDataList";

    // Page URLs
    public $AddUrl;
    public $EditUrl;
    public $DeleteUrl;
    public $ViewUrl;
    public $CopyUrl;
    public $ListUrl;

    // Update URLs
    public $InlineAddUrl;
    public $InlineCopyUrl;
    public $InlineEditUrl;
    public $GridAddUrl;
    public $GridEditUrl;
    public $MultiEditUrl;
    public $MultiDeleteUrl;
    public $MultiUpdateUrl;

    // Page headings
    public $Heading = "";
    public $Subheading = "";
    public $PageHeader;
    public $PageFooter;

    // Page layout
    public $UseLayout = true;

    // Page terminated
    private $terminated = false;

    // Page heading
    public function pageHeading()
    {
        global $Language;
        if ($this->Heading != "") {
            return $this->Heading;
        }
        if (method_exists($this, "tableCaption")) {
            return $this->tableCaption();
        }
        return "";
    }

    // Page subheading
    public function pageSubheading()
    {
        global $Language;
        if ($this->Subheading != "") {
            return $this->Subheading;
        }
        if ($this->TableName) {
            return $Language->phrase($this->PageID);
        }
        return "";
    }

    // Page name
    public function pageName()
    {
        return CurrentPageName();
    }

    // Page URL
    public function pageUrl($withArgs = true)
    {
        $route = GetRoute();
        $args = RemoveXss($route->getArguments());
        if (!$withArgs) {
            foreach ($args as $key => &$val) {
                $val = "";
            }
            unset($val);
        }
        return rtrim(UrlFor($route->getName(), $args), "/") . "?";
    }

    // Show Page Header
    public function showPageHeader()
    {
        $header = $this->PageHeader;
        $this->pageDataRendering($header);
        if ($header != "") { // Header exists, display
            echo '<div id="ew-page-header">' . $header . '</div>';
        }
    }

    // Show Page Footer
    public function showPageFooter()
    {
        $footer = $this->PageFooter;
        $this->pageDataRendered($footer);
        if ($footer != "") { // Footer exists, display
            echo '<div id="ew-page-footer">' . $footer . '</div>';
        }
    }

    // Set field visibility
    public function setVisibility()
    {
        $this->DistrictCode->setVisibility();
        $this->DistrictName->setVisibility();
        $this->GRNCode->setVisibility();
        $this->GRN->setVisibility();
        $this->SupplierCode->setVisibility();
        $this->Supplier->setVisibility();
        $this->CampCode->setVisibility();
        $this->Camp->setVisibility();
        $this->VillageCode->setVisibility();
        $this->Village->setVisibility();
        $this->NRC->setVisibility();
        $this->FarmerName->setVisibility();
        $this->DOB->setVisibility();
        $this->Age->setVisibility();
        $this->Group->setVisibility();
        $this->Gender->setVisibility();
        $this->Seedlings->setVisibility();
        $this->Seeds->setVisibility();
        $this->Type->setVisibility();
        $this->DateAllocated->setVisibility();
        $this->ID->setVisibility();
    }

    // Constructor
    public function __construct()
    {
        parent::__construct();
        global $Language, $DashboardReport, $DebugTimer, $UserTable;
        $this->FormActionName = Config("FORM_ROW_ACTION_NAME");
        $this->FormBlankRowName = Config("FORM_BLANK_ROW_NAME");
        $this->FormKeyCountName = Config("FORM_KEY_COUNT_NAME");
        $this->TableVar = 'cidp_data';
        $this->TableName = 'cidp_data';

        // Table CSS class
        $this->TableClass = "table table-bordered table-hover table-sm ew-table";

        // CSS class name as context
        $this->ContextClass = CheckClassName($this->TableVar);
        AppendClass($this->TableGridClass, $this->ContextClass);

        // Fixed header table
        if (!$this->UseCustomTemplate) {
            $this->setFixedHeaderTable(Config("USE_FIXED_HEADER_TABLE"), Config("FIXED_HEADER_TABLE_HEIGHT"));
        }

        // Initialize
        $GLOBALS["Page"] = &$this;

        // Language object
        $Language = Container("app.language");

        // Table object (cidp_data)
        if (!isset($GLOBALS["cidp_data"]) || $GLOBALS["cidp_data"]::class == PROJECT_NAMESPACE . "cidp_data") {
            $GLOBALS["cidp_data"] = &$this;
        }

        // Page URL
        $pageUrl = $this->pageUrl(false);

        // Initialize URLs
        $this->AddUrl = "CidpDataAdd";
        $this->InlineAddUrl = $pageUrl . "action=add";
        $this->GridAddUrl = $pageUrl . "action=gridadd";
        $this->GridEditUrl = $pageUrl . "action=gridedit";
        $this->MultiEditUrl = $pageUrl . "action=multiedit";
        $this->MultiDeleteUrl = "CidpDataDelete";
        $this->MultiUpdateUrl = "CidpDataUpdate";

        // Table name (for backward compatibility only)
        if (!defined(PROJECT_NAMESPACE . "TABLE_NAME")) {
            define(PROJECT_NAMESPACE . "TABLE_NAME", 'cidp_data');
        }

        // Start timer
        $DebugTimer = Container("debug.timer");

        // Debug message
        LoadDebugMessage();

        // Open connection
        $GLOBALS["Conn"] ??= $this->getConnection();

        // User table object
        $UserTable = Container("usertable");

        // List options
        $this->ListOptions = new ListOptions(Tag: "td", TableVar: $this->TableVar);

        // Export options
        $this->ExportOptions = new ListOptions(TagClassName: "ew-export-option");

        // Import options
        $this->ImportOptions = new ListOptions(TagClassName: "ew-import-option");

        // Other options
        $this->OtherOptions = new ListOptionsArray();

        // Grid-Add/Edit
        $this->OtherOptions["addedit"] = new ListOptions(
            TagClassName: "ew-add-edit-option",
            UseDropDownButton: false,
            DropDownButtonPhrase: $Language->phrase("ButtonAddEdit"),
            UseButtonGroup: true
        );

        // Detail tables
        $this->OtherOptions["detail"] = new ListOptions(TagClassName: "ew-detail-option");
        // Actions
        $this->OtherOptions["action"] = new ListOptions(TagClassName: "ew-action-option");

        // Column visibility
        $this->OtherOptions["column"] = new ListOptions(
            TableVar: $this->TableVar,
            TagClassName: "ew-column-option",
            ButtonGroupClass: "ew-column-dropdown",
            UseDropDownButton: true,
            DropDownButtonPhrase: $Language->phrase("Columns"),
            DropDownAutoClose: "outside",
            UseButtonGroup: false
        );

        // Filter options
        $this->FilterOptions = new ListOptions(TagClassName: "ew-filter-option");

        // List actions
        $this->ListActions = new ListActions();
    }

    // Get content from stream
    public function getContents(): string
    {
        global $Response;
        return $Response?->getBody() ?? ob_get_clean();
    }

    // Is lookup
    public function isLookup()
    {
        return SameText(Route(0), Config("API_LOOKUP_ACTION"));
    }

    // Is AutoFill
    public function isAutoFill()
    {
        return $this->isLookup() && SameText(Post("ajax"), "autofill");
    }

    // Is AutoSuggest
    public function isAutoSuggest()
    {
        return $this->isLookup() && SameText(Post("ajax"), "autosuggest");
    }

    // Is modal lookup
    public function isModalLookup()
    {
        return $this->isLookup() && SameText(Post("ajax"), "modal");
    }

    // Is terminated
    public function isTerminated()
    {
        return $this->terminated;
    }

    /**
     * Terminate page
     *
     * @param string $url URL for direction
     * @return void
     */
    public function terminate($url = "")
    {
        if ($this->terminated) {
            return;
        }
        global $TempImages, $DashboardReport, $Response;

        // Page is terminated
        $this->terminated = true;

        // Page Unload event
        if (method_exists($this, "pageUnload")) {
            $this->pageUnload();
        }
        DispatchEvent(new PageUnloadedEvent($this), PageUnloadedEvent::NAME);
        if (!IsApi() && method_exists($this, "pageRedirecting")) {
            $this->pageRedirecting($url);
        }

        // Close connection
        CloseConnections();

        // Return for API
        if (IsApi()) {
            $res = $url === true;
            if (!$res) { // Show response for API
                $ar = array_merge($this->getMessages(), $url ? ["url" => GetUrl($url)] : []);
                WriteJson($ar);
            }
            $this->clearMessages(); // Clear messages for API request
            return;
        } else { // Check if response is JSON
            if (WithJsonResponse()) { // With JSON response
                $this->clearMessages();
                return;
            }
        }

        // Go to URL if specified
        if ($url != "") {
            if (!Config("DEBUG") && ob_get_length()) {
                ob_end_clean();
            }

            // Handle modal response
            if ($this->IsModal) { // Show as modal
                $pageName = GetPageName($url);
                $result = ["url" => GetUrl($url), "modal" => "1"];  // Assume return to modal for simplicity
                if (!SameString($pageName, GetPageName($this->getListUrl()))) { // Not List page
                    $result["caption"] = $this->getModalCaption($pageName);
                    $result["view"] = SameString($pageName, "CidpDataView"); // If View page, no primary button
                } else { // List page
                    $result["error"] = $this->getFailureMessage(); // List page should not be shown as modal => error
                    $this->clearFailureMessage();
                }
                WriteJson($result);
            } else {
                SaveDebugMessage();
                Redirect(GetUrl($url));
            }
        }
        return; // Return to controller
    }

    // Get records from result set
    protected function getRecordsFromRecordset($rs, $current = false)
    {
        $rows = [];
        if (is_object($rs)) { // Result set
            while ($row = $rs->fetch()) {
                $this->loadRowValues($row); // Set up DbValue/CurrentValue
                $row = $this->getRecordFromArray($row);
                if ($current) {
                    return $row;
                } else {
                    $rows[] = $row;
                }
            }
        } elseif (is_array($rs)) {
            foreach ($rs as $ar) {
                $row = $this->getRecordFromArray($ar);
                if ($current) {
                    return $row;
                } else {
                    $rows[] = $row;
                }
            }
        }
        return $rows;
    }

    // Get record from array
    protected function getRecordFromArray($ar)
    {
        $row = [];
        if (is_array($ar)) {
            foreach ($ar as $fldname => $val) {
                if (array_key_exists($fldname, $this->Fields) && ($this->Fields[$fldname]->Visible || $this->Fields[$fldname]->IsPrimaryKey)) { // Primary key or Visible
                    $fld = &$this->Fields[$fldname];
                    if ($fld->HtmlTag == "FILE") { // Upload field
                        if (EmptyValue($val)) {
                            $row[$fldname] = null;
                        } else {
                            if ($fld->DataType == DataType::BLOB) {
                                $url = FullUrl(GetApiUrl(Config("API_FILE_ACTION") .
                                    "/" . $fld->TableVar . "/" . $fld->Param . "/" . rawurlencode($this->getRecordKeyValue($ar))));
                                $row[$fldname] = ["type" => ContentType($val), "url" => $url, "name" => $fld->Param . ContentExtension($val)];
                            } elseif (!$fld->UploadMultiple || !ContainsString($val, Config("MULTIPLE_UPLOAD_SEPARATOR"))) { // Single file
                                $url = FullUrl(GetApiUrl(Config("API_FILE_ACTION") .
                                    "/" . $fld->TableVar . "/" . Encrypt($fld->physicalUploadPath() . $val)));
                                $row[$fldname] = ["type" => MimeContentType($val), "url" => $url, "name" => $val];
                            } else { // Multiple files
                                $files = explode(Config("MULTIPLE_UPLOAD_SEPARATOR"), $val);
                                $ar = [];
                                foreach ($files as $file) {
                                    $url = FullUrl(GetApiUrl(Config("API_FILE_ACTION") .
                                        "/" . $fld->TableVar . "/" . Encrypt($fld->physicalUploadPath() . $file)));
                                    if (!EmptyValue($file)) {
                                        $ar[] = ["type" => MimeContentType($file), "url" => $url, "name" => $file];
                                    }
                                }
                                $row[$fldname] = $ar;
                            }
                        }
                    } else {
                        if ($fld->DataType == DataType::MEMO && $fld->MemoMaxLength > 0) {
                            $val = TruncateMemo($val, $fld->MemoMaxLength, $fld->TruncateMemoRemoveHtml);
                        }
                        $row[$fldname] = $val;
                    }
                }
            }
        }
        return $row;
    }

    // Get record key value from array
    protected function getRecordKeyValue($ar)
    {
        $key = "";
        if (is_array($ar)) {
            $key .= @$ar['ID'];
        }
        return $key;
    }

    /**
     * Hide fields for add/edit
     *
     * @return void
     */
    protected function hideFieldsForAddEdit()
    {
        if ($this->isAdd() || $this->isCopy() || $this->isGridAdd()) {
            $this->ID->Visible = false;
        }
    }

    // Lookup data
    public function lookup(array $req = [], bool $response = true)
    {
        global $Language, $Security;

        // Get lookup object
        $fieldName = $req["field"] ?? null;
        if (!$fieldName) {
            return [];
        }
        $fld = $this->Fields[$fieldName];
        $lookup = $fld->Lookup;
        $name = $req["name"] ?? "";
        if (ContainsString($name, "query_builder_rule")) {
            $lookup->FilterFields = []; // Skip parent fields if any
        }

        // Get lookup parameters
        $lookupType = $req["ajax"] ?? "unknown";
        $pageSize = -1;
        $offset = -1;
        $searchValue = "";
        if (SameText($lookupType, "modal") || SameText($lookupType, "filter")) {
            $searchValue = $req["q"] ?? $req["sv"] ?? "";
            $pageSize = $req["n"] ?? $req["recperpage"] ?? 10;
        } elseif (SameText($lookupType, "autosuggest")) {
            $searchValue = $req["q"] ?? "";
            $pageSize = $req["n"] ?? -1;
            $pageSize = is_numeric($pageSize) ? (int)$pageSize : -1;
            if ($pageSize <= 0) {
                $pageSize = Config("AUTO_SUGGEST_MAX_ENTRIES");
            }
        }
        $start = $req["start"] ?? -1;
        $start = is_numeric($start) ? (int)$start : -1;
        $page = $req["page"] ?? -1;
        $page = is_numeric($page) ? (int)$page : -1;
        $offset = $start >= 0 ? $start : ($page > 0 && $pageSize > 0 ? ($page - 1) * $pageSize : 0);
        $userSelect = Decrypt($req["s"] ?? "");
        $userFilter = Decrypt($req["f"] ?? "");
        $userOrderBy = Decrypt($req["o"] ?? "");
        $keys = $req["keys"] ?? null;
        $lookup->LookupType = $lookupType; // Lookup type
        $lookup->FilterValues = []; // Clear filter values first
        if ($keys !== null) { // Selected records from modal
            if (is_array($keys)) {
                $keys = implode(Config("MULTIPLE_OPTION_SEPARATOR"), $keys);
            }
            $lookup->FilterFields = []; // Skip parent fields if any
            $lookup->FilterValues[] = $keys; // Lookup values
            $pageSize = -1; // Show all records
        } else { // Lookup values
            $lookup->FilterValues[] = $req["v0"] ?? $req["lookupValue"] ?? "";
        }
        $cnt = is_array($lookup->FilterFields) ? count($lookup->FilterFields) : 0;
        for ($i = 1; $i <= $cnt; $i++) {
            $lookup->FilterValues[] = $req["v" . $i] ?? "";
        }
        $lookup->SearchValue = $searchValue;
        $lookup->PageSize = $pageSize;
        $lookup->Offset = $offset;
        if ($userSelect != "") {
            $lookup->UserSelect = $userSelect;
        }
        if ($userFilter != "") {
            $lookup->UserFilter = $userFilter;
        }
        if ($userOrderBy != "") {
            $lookup->UserOrderBy = $userOrderBy;
        }
        return $lookup->toJson($this, $response); // Use settings from current page
    }

    // Class variables
    public $ListOptions; // List options
    public $ExportOptions; // Export options
    public $SearchOptions; // Search options
    public $OtherOptions; // Other options
    public $HeaderOptions; // Header options
    public $FooterOptions; // Footer options
    public $FilterOptions; // Filter options
    public $ImportOptions; // Import options
    public $ListActions; // List actions
    public $SelectedCount = 0;
    public $SelectedIndex = 0;
    public $DisplayRecords = 20;
    public $StartRecord;
    public $StopRecord;
    public $TotalRecords = 0;
    public $RecordRange = 10;
    public $PageSizes = ""; // Page sizes (comma separated)
    public $DefaultSearchWhere = ""; // Default search WHERE clause
    public $SearchWhere = ""; // Search WHERE clause
    public $SearchPanelClass = "ew-search-panel collapse"; // Search Panel class
    public $SearchColumnCount = 0; // For extended search
    public $SearchFieldsPerRow = 1; // For extended search
    public $RecordCount = 0; // Record count
    public $InlineRowCount = 0;
    public $StartRowCount = 1;
    public $Attrs = []; // Row attributes and cell attributes
    public $RowIndex = 0; // Row index
    public $KeyCount = 0; // Key count
    public $MultiColumnGridClass = "row-cols-md";
    public $MultiColumnEditClass = "col-12 w-100";
    public $MultiColumnCardClass = "card h-100 ew-card";
    public $MultiColumnListOptionsPosition = "bottom-start";
    public $DbMasterFilter = ""; // Master filter
    public $DbDetailFilter = ""; // Detail filter
    public $MasterRecordExists;
    public $MultiSelectKey;
    public $Command;
    public $UserAction; // User action
    public $RestoreSearch = false;
    public $HashValue; // Hash value
    public $DetailPages;
    public $TopContentClass = "ew-top";
    public $MiddleContentClass = "ew-middle";
    public $BottomContentClass = "ew-bottom";
    public $PageAction;
    public $RecKeys = [];
    public $IsModal = false;
    protected $FilterForModalActions = "";
    private $UseInfiniteScroll = false;

    /**
     * Load result set from filter
     *
     * @return void
     */
    public function loadRecordsetFromFilter($filter)
    {
        // Set up list options
        $this->setupListOptions();

        // Search options
        $this->setupSearchOptions();

        // Other options
        $this->setupOtherOptions();

        // Set visibility
        $this->setVisibility();

        // Load result set
        $this->TotalRecords = $this->loadRecordCount($filter);
        $this->StartRecord = 1;
        $this->StopRecord = $this->DisplayRecords;
        $this->CurrentFilter = $filter;
        $this->Recordset = $this->loadRecordset();

        // Set up pager
        $this->Pager = new PrevNextPager($this, $this->StartRecord, $this->DisplayRecords, $this->TotalRecords, $this->PageSizes, $this->RecordRange, $this->AutoHidePager, $this->AutoHidePageSizeSelector);
    }

    /**
     * Page run
     *
     * @return void
     */
    public function run()
    {
        global $ExportType, $Language, $Security, $CurrentForm, $DashboardReport;

        // Multi column button position
        $this->MultiColumnListOptionsPosition = Config("MULTI_COLUMN_LIST_OPTIONS_POSITION");
        $DashboardReport ??= Param(Config("PAGE_DASHBOARD"));

        // Is modal
        $this->IsModal = ConvertToBool(Param("modal"));

        // Use layout
        $this->UseLayout = $this->UseLayout && ConvertToBool(Param(Config("PAGE_LAYOUT"), true));

        // View
        $this->View = Get(Config("VIEW"));

        // Load user profile
        if (IsLoggedIn()) {
            Profile()->setUserName(CurrentUserName())->loadFromStorage();
        }

        // Create form object
        $CurrentForm = new HttpForm();

        // Get export parameters
        $custom = "";
        if (Param("export") !== null) {
            $this->Export = Param("export");
            $custom = Param("custom", "");
        } else {
            $this->setExportReturnUrl(CurrentUrl());
        }
        $ExportType = $this->Export; // Get export parameter, used in header
        if ($ExportType != "") {
            global $SkipHeaderFooter;
            $SkipHeaderFooter = true;
        }
        $this->CurrentAction = Param("action"); // Set up current action

        // Get grid add count
        $gridaddcnt = Get(Config("TABLE_GRID_ADD_ROW_COUNT"), "");
        if (is_numeric($gridaddcnt) && $gridaddcnt > 0) {
            $this->GridAddRowCount = $gridaddcnt;
        }

        // Set up list options
        $this->setupListOptions();

        // Setup export options
        $this->setupExportOptions();
        $this->setVisibility();

        // Set lookup cache
        if (!in_array($this->PageID, Config("LOOKUP_CACHE_PAGE_IDS"))) {
            $this->setUseLookupCache(false);
        }

        // Global Page Loading event (in userfn*.php)
        DispatchEvent(new PageLoadingEvent($this), PageLoadingEvent::NAME);

        // Page Load event
        if (method_exists($this, "pageLoad")) {
            $this->pageLoad();
        }

        // Hide fields for add/edit
        if (!$this->UseAjaxActions) {
            $this->hideFieldsForAddEdit();
        }
        // Use inline delete
        if ($this->UseAjaxActions) {
            $this->InlineDelete = true;
        }

        // Setup other options
        $this->setupOtherOptions();

        // Set up lookup cache
        $this->setupLookupOptions($this->DistrictCode);
        $this->setupLookupOptions($this->GRNCode);
        $this->setupLookupOptions($this->SupplierCode);
        $this->setupLookupOptions($this->CampCode);
        $this->setupLookupOptions($this->VillageCode);
        $this->setupLookupOptions($this->Group);
        $this->setupLookupOptions($this->Gender);
        $this->setupLookupOptions($this->Type);

        // Load default values for add
        $this->loadDefaultValues();

        // Update form name to avoid conflict
        if ($this->IsModal) {
            $this->FormName = "fcidp_datagrid";
        }

        // Set up page action
        $this->PageAction = CurrentPageUrl(false);

        // Set up infinite scroll
        $this->UseInfiniteScroll = ConvertToBool(Param("infinitescroll"));

        // Search filters
        $srchAdvanced = ""; // Advanced search filter
        $srchBasic = ""; // Basic search filter
        $query = ""; // Query builder

        // Set up Dashboard Filter
        if ($DashboardReport) {
            AddFilter($this->Filter, $this->getDashboardFilter($DashboardReport, $this->TableVar));
        }

        // Get command
        $this->Command = strtolower(Get("cmd", ""));

        // Process list action first
        if ($this->processListAction()) { // Ajax request
            $this->terminate();
            return;
        }

        // Set up records per page
        $this->setupDisplayRecords();

        // Handle reset command
        $this->resetCmd();

        // Set up Breadcrumb
        if (!$this->isExport()) {
            $this->setupBreadcrumb();
        }

        // Check QueryString parameters
        if (Get("action") !== null) {
            $this->CurrentAction = Get("action");
        } else {
            if (Post("action") !== null && Post("action") !== $this->UserAction) {
                $this->CurrentAction = Post("action"); // Get action
            } elseif (Session(SESSION_INLINE_MODE) == "gridedit") { // Previously in grid edit mode
                if (Get(Config("TABLE_START_REC")) !== null || Get(Config("TABLE_PAGE_NUMBER")) !== null) { // Stay in grid edit mode if paging
                    $this->gridEditMode();
                } else { // Reset grid edit
                    $this->clearInlineMode();
                }
            }
        }

        // Clear inline mode
        if ($this->isCancel()) {
            $this->clearInlineMode();
        }

        // Switch to grid edit mode
        if ($this->isGridEdit()) {
            $this->gridEditMode();
        }

        // Grid Update
        if (IsPost() && ($this->isGridUpdate() || $this->isMultiUpdate() || $this->isGridOverwrite()) && (Session(SESSION_INLINE_MODE) == "gridedit" || Session(SESSION_INLINE_MODE) == "multiedit")) {
            if ($this->validateGridForm()) {
                $gridUpdate = $this->gridUpdate();
            } else {
                $gridUpdate = false;
            }
            if ($gridUpdate) {
                // Handle modal grid edit and multi edit, redirect to list page directly
                if ($this->IsModal && !$this->UseAjaxActions) {
                    $this->terminate("CidpDataList");
                    return;
                }
            } else {
                $this->EventCancelled = true;
                if ($this->UseAjaxActions) {
                    WriteJson([
                        "success" => false,
                        "validation" => $this->ValidationErrors,
                        "error" => $this->getFailureMessage()
                    ]);
                    $this->clearFailureMessage();
                    $this->terminate();
                    return;
                }
                if ($this->isMultiUpdate()) { // Stay in Multi-Edit mode
                    $this->FilterForModalActions = $this->getFilterFromRecords($this->getGridFormValues());
                    $this->multiEditMode();
                } else { // Stay in grid edit mode
                    $this->gridEditMode();
                }
            }
        }

        // Switch to inline edit mode
        if ($this->isEdit()) {
            $this->inlineEditMode();
        // Inline Update
        } elseif (IsPost() && ($this->isUpdate() || $this->isOverwrite()) && Session(SESSION_INLINE_MODE) == "edit") {
            $this->setKey(Post($this->OldKeyName));
            // Return JSON error message if UseAjaxActions
            if (!$this->inlineUpdate() && $this->UseAjaxActions) {
                WriteJson(["success" => false, "validation" => $this->getValidationErrors(), "error" => $this->getFailureMessage()]);
                $this->clearFailureMessage();
                $this->terminate();
                return;
            }
        }

        // Switch to grid add mode
        if ($this->isGridAdd()) {
            $this->gridAddMode();
            // Grid Insert
        } elseif (IsPost() && $this->isGridInsert() && Session(SESSION_INLINE_MODE) == "gridadd") {
            if ($this->validateGridForm()) {
                $gridInsert = $this->gridInsert();
            } else {
                $gridInsert = false;
            }
            if ($gridInsert) {
                // Handle modal grid add, redirect to list page directly
                if ($this->IsModal && !$this->UseAjaxActions) {
                    $this->terminate("CidpDataList");
                    return;
                }
            } else {
                $this->EventCancelled = true;
                if ($this->UseAjaxActions) {
                    WriteJson([
                        "success" => false,
                        "validation" => $this->ValidationErrors,
                        "error" => $this->getFailureMessage()
                    ]);
                    $this->clearFailureMessage();
                    $this->terminate();
                    return;
                }
                $this->gridAddMode(); // Stay in grid add mode
            }
        }

        // Hide list options
        if ($this->isExport()) {
            $this->ListOptions->hideAllOptions(["sequence"]);
            $this->ListOptions->UseDropDownButton = false; // Disable drop down button
            $this->ListOptions->UseButtonGroup = false; // Disable button group
        } elseif ($this->isGridAdd() || $this->isGridEdit() || $this->isMultiEdit() || $this->isConfirm()) {
            $this->ListOptions->hideAllOptions();
            $this->ListOptions->UseDropDownButton = false; // Disable drop down button
            $this->ListOptions->UseButtonGroup = false; // Disable button group
        }

        // Hide options
        if ($this->isExport() || !(EmptyValue($this->CurrentAction) || $this->isSearch())) {
            $this->ExportOptions->hideAllOptions();
            $this->FilterOptions->hideAllOptions();
            $this->ImportOptions->hideAllOptions();
        }

        // Hide other options
        if ($this->isExport()) {
            $this->OtherOptions->hideAllOptions();
        }

        // Show grid delete link for grid add / grid edit
        if ($this->AllowAddDeleteRow) {
            if ($this->isGridAdd() || $this->isGridEdit()) {
                $item = $this->ListOptions["griddelete"];
                if ($item) {
                    $item->Visible = $Security->allowDelete(CurrentProjectID() . $this->TableName);
                }
            }
        }

        // Get default search criteria
        AddFilter($this->DefaultSearchWhere, $this->basicSearchWhere(true));
        AddFilter($this->DefaultSearchWhere, $this->advancedSearchWhere(true));

        // Get basic search values
        $this->loadBasicSearchValues();

        // Get and validate search values for advanced search
        if (EmptyValue($this->UserAction)) { // Skip if user action
            $this->loadSearchValues();
        }

        // Process filter list
        if ($this->processFilterList()) {
            $this->terminate();
            return;
        }
        if (!$this->validateSearch()) {
            // Nothing to do
        }

        // Restore search parms from Session if not searching / reset / export
        if (($this->isExport() || $this->Command != "search" && $this->Command != "reset" && $this->Command != "resetall") && $this->Command != "json" && $this->checkSearchParms()) {
            $this->restoreSearchParms();
        }

        // Call Recordset SearchValidated event
        $this->recordsetSearchValidated();

        // Set up sorting order
        $this->setupSortOrder();

        // Get basic search criteria
        if (!$this->hasInvalidFields()) {
            $srchBasic = $this->basicSearchWhere();
        }

        // Get advanced search criteria
        if (!$this->hasInvalidFields()) {
            $srchAdvanced = $this->advancedSearchWhere();
        }

        // Get query builder criteria
        $query = $DashboardReport ? "" : $this->queryBuilderWhere();

        // Restore display records
        if ($this->Command != "json" && $this->getRecordsPerPage() != "") {
            $this->DisplayRecords = $this->getRecordsPerPage(); // Restore from Session
        } else {
            $this->DisplayRecords = 20; // Load default
            $this->setRecordsPerPage($this->DisplayRecords); // Save default to Session
        }

        // Load search default if no existing search criteria
        if (!$this->checkSearchParms() && !$query) {
            // Load basic search from default
            $this->BasicSearch->loadDefault();
            if ($this->BasicSearch->Keyword != "") {
                $srchBasic = $this->basicSearchWhere(); // Save to session
            }

            // Load advanced search from default
            if ($this->loadAdvancedSearchDefault()) {
                $srchAdvanced = $this->advancedSearchWhere(); // Save to session
            }
        }

        // Restore search settings from Session
        if (!$this->hasInvalidFields()) {
            $this->loadAdvancedSearch();
        }

        // Build search criteria
        if ($query) {
            AddFilter($this->SearchWhere, $query);
        } else {
            AddFilter($this->SearchWhere, $srchAdvanced);
            AddFilter($this->SearchWhere, $srchBasic);
        }

        // Call Recordset_Searching event
        $this->recordsetSearching($this->SearchWhere);

        // Save search criteria
        if ($this->Command == "search" && !$this->RestoreSearch) {
            $this->setSearchWhere($this->SearchWhere); // Save to Session
            $this->StartRecord = 1; // Reset start record counter
            $this->setStartRecordNumber($this->StartRecord);
        } elseif ($this->Command != "json" && !$query) {
            $this->SearchWhere = $this->getSearchWhere();
        }

        // Build filter
        if (!$Security->canList()) {
            $this->Filter = "(0=1)"; // Filter all records
        }
        AddFilter($this->Filter, $this->DbDetailFilter);
        AddFilter($this->Filter, $this->SearchWhere);

        // Set up filter
        if ($this->Command == "json") {
            $this->UseSessionForListSql = false; // Do not use session for ListSQL
            $this->CurrentFilter = $this->Filter;
        } else {
            $this->setSessionWhere($this->Filter);
            $this->CurrentFilter = "";
        }
        $this->Filter = $this->applyUserIDFilters($this->Filter);
        if ($this->isGridAdd()) {
            $this->CurrentFilter = "0=1";
            $this->StartRecord = 1;
            $this->DisplayRecords = $this->GridAddRowCount;
            $this->TotalRecords = $this->DisplayRecords;
            $this->StopRecord = $this->DisplayRecords;
        } elseif (($this->isEdit() || $this->isCopy() || $this->isInlineInserted() || $this->isInlineUpdated()) && $this->UseInfiniteScroll) { // Get current record only
            $this->CurrentFilter = $this->isInlineUpdated() ? $this->getRecordFilter() : $this->getFilterFromRecordKeys();
            $this->TotalRecords = $this->listRecordCount();
            $this->StartRecord = 1;
            $this->StopRecord = $this->DisplayRecords;
            $this->Recordset = $this->loadRecordset();
        } elseif (
            $this->UseInfiniteScroll && $this->isGridInserted() ||
            $this->UseInfiniteScroll && ($this->isGridEdit() || $this->isGridUpdated()) ||
            $this->isMultiEdit() ||
            $this->UseInfiniteScroll && $this->isMultiUpdated()
        ) { // Get current records only
            $this->CurrentFilter = $this->FilterForModalActions; // Restore filter
            $this->TotalRecords = $this->listRecordCount();
            $this->StartRecord = 1;
            $this->StopRecord = $this->DisplayRecords;
            $this->Recordset = $this->loadRecordset();
        } else {
            $this->TotalRecords = $this->listRecordCount();
            $this->StartRecord = 1;
            if ($this->DisplayRecords <= 0 || ($this->isExport() && $this->ExportAll)) { // Display all records
                $this->DisplayRecords = $this->TotalRecords;
            }
            if (!($this->isExport() && $this->ExportAll)) { // Set up start record position
                $this->setupStartRecord();
            }
            $this->Recordset = $this->loadRecordset($this->StartRecord - 1, $this->DisplayRecords);

            // Set no record found message
            if ((EmptyValue($this->CurrentAction) || $this->isSearch()) && $this->TotalRecords == 0) {
                if (!$Security->canList()) {
                    $this->setWarningMessage(DeniedMessage());
                }
                if ($this->SearchWhere == "0=101") {
                    $this->setWarningMessage($Language->phrase("EnterSearchCriteria"));
                } else {
                    $this->setWarningMessage($Language->phrase("NoRecord"));
                }
            }
        }

        // Set up list action columns
        foreach ($this->ListActions as $listAction) {
            if ($listAction->Allowed) {
                if ($listAction->Select == ACTION_MULTIPLE) { // Show checkbox column if multiple action
                    $this->ListOptions["checkbox"]->Visible = true;
                } elseif ($listAction->Select == ACTION_SINGLE) { // Show list action column
                        $this->ListOptions["listactions"]->Visible = true; // Set visible if any list action is allowed
                }
            }
        }

        // Search options
        $this->setupSearchOptions();

        // Set up search panel class
        if ($this->SearchWhere != "") {
            if ($query) { // Hide search panel if using QueryBuilder
                RemoveClass($this->SearchPanelClass, "show");
            } else {
                AppendClass($this->SearchPanelClass, "show");
            }
        }

        // API list action
        if (IsApi()) {
            if (Route(0) == Config("API_LIST_ACTION")) {
                if (!$this->isExport()) {
                    $rows = $this->getRecordsFromRecordset($this->Recordset);
                    $this->Recordset?->free();
                    WriteJson([
                        "success" => true,
                        "action" => Config("API_LIST_ACTION"),
                        $this->TableVar => $rows,
                        "totalRecordCount" => $this->TotalRecords
                    ]);
                    $this->terminate(true);
                }
                return;
            } elseif ($this->getFailureMessage() != "") {
                WriteJson(["error" => $this->getFailureMessage()]);
                $this->clearFailureMessage();
                $this->terminate(true);
                return;
            }
        }

        // Render other options
        $this->renderOtherOptions();

        // Set up pager
        $this->Pager = new PrevNextPager($this, $this->StartRecord, $this->DisplayRecords, $this->TotalRecords, $this->PageSizes, $this->RecordRange, $this->AutoHidePager, $this->AutoHidePageSizeSelector);

        // Set ReturnUrl in header if necessary
        if ($returnUrl = Container("app.flash")->getFirstMessage("Return-Url")) {
            AddHeader("Return-Url", GetUrl($returnUrl));
        }

        // Set LoginStatus / Page_Rendering / Page_Render
        if (!IsApi() && !$this->isTerminated()) {
            // Setup login status
            SetupLoginStatus();

            // Pass login status to client side
            SetClientVar("login", LoginStatus());

            // Global Page Rendering event (in userfn*.php)
            DispatchEvent(new PageRenderingEvent($this), PageRenderingEvent::NAME);

            // Page Render event
            if (method_exists($this, "pageRender")) {
                $this->pageRender();
            }

            // Render search option
            if (method_exists($this, "renderSearchOptions")) {
                $this->renderSearchOptions();
            }
        }
    }

    // Get page number
    public function getPageNumber()
    {
        return ($this->DisplayRecords > 0 && $this->StartRecord > 0) ? ceil($this->StartRecord / $this->DisplayRecords) : 1;
    }

    // Set up number of records displayed per page
    protected function setupDisplayRecords()
    {
        $wrk = Get(Config("TABLE_REC_PER_PAGE"), "");
        if ($wrk != "") {
            if (is_numeric($wrk)) {
                $this->DisplayRecords = (int)$wrk;
            } else {
                if (SameText($wrk, "all")) { // Display all records
                    $this->DisplayRecords = -1;
                } else {
                    $this->DisplayRecords = 20; // Non-numeric, load default
                }
            }
            $this->setRecordsPerPage($this->DisplayRecords); // Save to Session
            // Reset start position
            $this->StartRecord = 1;
            $this->setStartRecordNumber($this->StartRecord);
        }
    }

    // Exit inline mode
    protected function clearInlineMode()
    {
        $this->Age->FormValue = ""; // Clear form value
        $this->Seedlings->FormValue = ""; // Clear form value
        $this->Seeds->FormValue = ""; // Clear form value
        $this->LastAction = $this->CurrentAction; // Save last action
        $this->CurrentAction = ""; // Clear action
        $_SESSION[SESSION_INLINE_MODE] = ""; // Clear inline mode
    }

    // Switch to grid add mode
    protected function gridAddMode()
    {
        $this->CurrentAction = "gridadd";
        $_SESSION[SESSION_INLINE_MODE] = "gridadd";
        $this->hideFieldsForAddEdit();
    }

    // Switch to grid edit mode
    protected function gridEditMode()
    {
        $this->CurrentAction = "gridedit";
        $_SESSION[SESSION_INLINE_MODE] = "gridedit";
        $this->hideFieldsForAddEdit();
    }

    // Switch to Inline Edit mode
    protected function inlineEditMode()
    {
        global $Security, $Language;
        if (!$Security->canEdit()) {
            return false; // Edit not allowed
        }
        $inlineEdit = true;
        if (($keyValue = Get("ID") ?? Route("ID")) !== null) {
            $this->ID->setQueryStringValue($keyValue);
        } elseif (IsApi() && ($keyValue = Route(2)) !== null) {
            $this->ID->setQueryStringValue($keyValue);
        } else {
            $inlineEdit = false;
        }
        if ($inlineEdit) {
            if ($this->loadRow()) {
                $this->OldKey = $this->getKey(true); // Get from CurrentValue
                $this->setKey($this->OldKey); // Set to OldValue
                $_SESSION[SESSION_INLINE_MODE] = "edit"; // Enable inline edit
            }
        }
        return true;
    }

    // Perform update to Inline Edit record
    protected function inlineUpdate()
    {
        global $Language, $CurrentForm;
        $CurrentForm->Index = 1;
        $this->loadFormValues(); // Get form values

        // Validate form
        $inlineUpdate = true;
        if (!$this->validateForm()) {
            $inlineUpdate = false; // Form error, reset action
        } else {
            $inlineUpdate = false;
            $this->SendEmail = true; // Send email on update success
            $inlineUpdate = $this->editRow(); // Update record
        }
        if ($inlineUpdate) { // Update success
            if ($this->getSuccessMessage() == "") {
                $this->setSuccessMessage($Language->phrase("UpdateSuccess")); // Set up success message
            }
            $this->clearInlineMode(); // Clear inline edit mode
        } else {
            if ($this->getFailureMessage() == "") {
                $this->setFailureMessage($Language->phrase("UpdateFailed")); // Set update failed message
            }
            $this->EventCancelled = true; // Cancel event
            $this->CurrentAction = "edit"; // Stay in edit mode
        }
        return $inlineUpdate;
    }

    // Check Inline Edit key
    public function checkInlineEditKey()
    {
        if (!SameString($this->ID->OldValue, $this->ID->CurrentValue)) {
            return false;
        }
        return true;
    }

    // Perform update to grid
    public function gridUpdate()
    {
        global $Language, $CurrentForm;
        $gridUpdate = true;

        // Get old result set
        $this->CurrentFilter = $this->buildKeyFilter();
        if ($this->CurrentFilter == "") {
            $this->CurrentFilter = "0=1";
        }
        $sql = $this->getCurrentSql();
        $conn = $this->getConnection();
        if ($rs = $conn->executeQuery($sql)) {
            $rsold = $rs->fetchAllAssociative();
        }

        // Call Grid Updating event
        if (!$this->gridUpdating($rsold)) {
            if ($this->getFailureMessage() == "") {
                $this->setFailureMessage($Language->phrase("GridEditCancelled")); // Set grid edit cancelled message
            }
            $this->EventCancelled = true;
            return false;
        }

        // Begin transaction
        if ($this->UseTransaction) {
            $conn->beginTransaction();
        }
        $wrkfilter = "";
        $key = "";

        // Update row index and get row key
        $CurrentForm->resetIndex();
        $rowcnt = strval($CurrentForm->getValue($this->FormKeyCountName));
        if ($rowcnt == "" || !is_numeric($rowcnt)) {
            $rowcnt = 0;
        }

        // Update all rows based on key
        for ($rowindex = 1; $rowindex <= $rowcnt; $rowindex++) {
            $CurrentForm->Index = $rowindex;
            $this->setKey($CurrentForm->getValue($this->OldKeyName));
            $rowaction = strval($CurrentForm->getValue($this->FormActionName));

            // Load all values and keys
            if ($rowaction != "insertdelete" && $rowaction != "hide") { // Skip insert then deleted rows / hidden rows for grid edit
                $this->loadFormValues(); // Get form values
                if ($rowaction == "" || $rowaction == "edit" || $rowaction == "delete") {
                    $gridUpdate = $this->OldKey != ""; // Key must not be empty
                } else {
                    $gridUpdate = true;
                }

                // Skip empty row
                if ($rowaction == "insert" && $this->emptyRow()) {
                // Validate form and insert/update/delete record
                } elseif ($gridUpdate) {
                    if ($rowaction == "delete") {
                        $this->CurrentFilter = $this->getRecordFilter();
                        $gridUpdate = $this->deleteRows(); // Delete this row
                    } else {
                        if ($rowaction == "insert") {
                            $gridUpdate = $this->addRow(); // Insert this row
                        } else {
                            if ($this->OldKey != "") {
                                $this->SendEmail = false; // Do not send email on update success
                                $gridUpdate = $this->editRow(); // Update this row
                            }
                        } // End update
                        if ($gridUpdate) { // Get inserted or updated filter
                            AddFilter($wrkfilter, $this->getRecordFilter(), "OR");
                        }
                    }
                }
                if ($gridUpdate) {
                    if ($key != "") {
                        $key .= ", ";
                    }
                    $key .= $this->OldKey;
                } else {
                    $this->EventCancelled = true;
                    break;
                }
            }
        }
        if ($gridUpdate) {
            if ($this->UseTransaction) { // Commit transaction
                if ($conn->isTransactionActive()) {
                    $conn->commit();
                }
            }
            $this->FilterForModalActions = $wrkfilter;

            // Get new records
            $rsnew = $conn->fetchAllAssociative($sql);

            // Call Grid_Updated event
            $this->gridUpdated($rsold, $rsnew);
            if ($this->getSuccessMessage() == "") {
                $this->setSuccessMessage($Language->phrase("UpdateSuccess")); // Set up update success message
            }
            $this->clearInlineMode(); // Clear inline edit mode
        } else {
            if ($this->UseTransaction) { // Rollback transaction
                if ($conn->isTransactionActive()) {
                    $conn->rollback();
                }
            }
            if ($this->getFailureMessage() == "") {
                $this->setFailureMessage($Language->phrase("UpdateFailed")); // Set update failed message
            }
        }
        return $gridUpdate;
    }

    // Build filter for all keys
    protected function buildKeyFilter()
    {
        global $CurrentForm;
        $wrkFilter = "";

        // Update row index and get row key
        $rowindex = 1;
        $CurrentForm->Index = $rowindex;
        $thisKey = strval($CurrentForm->getValue($this->OldKeyName));
        while ($thisKey != "") {
            $this->setKey($thisKey);
            if ($this->OldKey != "") {
                $filter = $this->getRecordFilter();
                if ($wrkFilter != "") {
                    $wrkFilter .= " OR ";
                }
                $wrkFilter .= $filter;
            } else {
                $wrkFilter = "0=1";
                break;
            }

            // Update row index and get row key
            $rowindex++; // Next row
            $CurrentForm->Index = $rowindex;
            $thisKey = strval($CurrentForm->getValue($this->OldKeyName));
        }
        return $wrkFilter;
    }

    // Perform grid add
    public function gridInsert()
    {
        global $Language, $CurrentForm;
        $rowindex = 1;
        $gridInsert = false;
        $conn = $this->getConnection();

        // Call Grid Inserting event
        if (!$this->gridInserting()) {
            if ($this->getFailureMessage() == "") {
                $this->setFailureMessage($Language->phrase("GridAddCancelled")); // Set grid add cancelled message
            }
            $this->EventCancelled = true;
            return false;
        }

        // Begin transaction
        if ($this->UseTransaction) {
            $conn->beginTransaction();
        }

        // Init key filter
        $wrkfilter = "";
        $addcnt = 0;
        $key = "";

        // Get row count
        $CurrentForm->resetIndex();
        $rowcnt = strval($CurrentForm->getValue($this->FormKeyCountName));
        if ($rowcnt == "" || !is_numeric($rowcnt)) {
            $rowcnt = 0;
        }

        // Insert all rows
        for ($rowindex = 1; $rowindex <= $rowcnt; $rowindex++) {
            // Load current row values
            $CurrentForm->Index = $rowindex;
            $rowaction = strval($CurrentForm->getValue($this->FormActionName));
            if ($rowaction != "" && $rowaction != "insert") {
                continue; // Skip
            }
            $rsold = null;
            if ($rowaction == "insert") {
                $this->OldKey = strval($CurrentForm->getValue($this->OldKeyName));
                $rsold = $this->loadOldRecord(); // Load old record
            }
            $this->loadFormValues(); // Get form values
            if (!$this->emptyRow()) {
                $addcnt++;
                $this->SendEmail = false; // Do not send email on insert success
                $gridInsert = $this->addRow($rsold); // Insert row (already validated by validateGridForm())
                if ($gridInsert) {
                    if ($key != "") {
                        $key .= Config("COMPOSITE_KEY_SEPARATOR");
                    }
                    $key .= $this->ID->CurrentValue;

                    // Add filter for this record
                    AddFilter($wrkfilter, $this->getRecordFilter(), "OR");
                } else {
                    $this->EventCancelled = true;
                    break;
                }
            }
        }
        if ($addcnt == 0) { // No record inserted
            $this->setFailureMessage($Language->phrase("NoAddRecord"));
            $gridInsert = false;
        }
        if ($gridInsert) {
            if ($this->UseTransaction) { // Commit transaction
                if ($conn->isTransactionActive()) {
                    $conn->commit();
                }
            }

            // Get new records
            $this->CurrentFilter = $wrkfilter;
            $this->FilterForModalActions = $wrkfilter;
            $sql = $this->getCurrentSql();
            $rsnew = $conn->fetchAllAssociative($sql);

            // Call Grid_Inserted event
            $this->gridInserted($rsnew);
            if ($this->getSuccessMessage() == "") {
                $this->setSuccessMessage($Language->phrase("InsertSuccess")); // Set up insert success message
            }
            $this->clearInlineMode(); // Clear grid add mode
        } else {
            if ($this->UseTransaction) { // Rollback transaction
                if ($conn->isTransactionActive()) {
                    $conn->rollback();
                }
            }
            if ($this->getFailureMessage() == "") {
                $this->setFailureMessage($Language->phrase("InsertFailed")); // Set insert failed message
            }
        }
        return $gridInsert;
    }

    // Check if empty row
    public function emptyRow()
    {
        global $CurrentForm;
        if (
            $CurrentForm->hasValue("x_DistrictCode") &&
            $CurrentForm->hasValue("o_DistrictCode") &&
            $this->DistrictCode->CurrentValue != $this->DistrictCode->DefaultValue &&
            !($this->DistrictCode->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->DistrictCode->CurrentValue == $this->DistrictCode->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_DistrictName") &&
            $CurrentForm->hasValue("o_DistrictName") &&
            $this->DistrictName->CurrentValue != $this->DistrictName->DefaultValue &&
            !($this->DistrictName->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->DistrictName->CurrentValue == $this->DistrictName->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_GRNCode") &&
            $CurrentForm->hasValue("o_GRNCode") &&
            $this->GRNCode->CurrentValue != $this->GRNCode->DefaultValue &&
            !($this->GRNCode->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->GRNCode->CurrentValue == $this->GRNCode->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_GRN") &&
            $CurrentForm->hasValue("o_GRN") &&
            $this->GRN->CurrentValue != $this->GRN->DefaultValue &&
            !($this->GRN->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->GRN->CurrentValue == $this->GRN->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_SupplierCode") &&
            $CurrentForm->hasValue("o_SupplierCode") &&
            $this->SupplierCode->CurrentValue != $this->SupplierCode->DefaultValue &&
            !($this->SupplierCode->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->SupplierCode->CurrentValue == $this->SupplierCode->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Supplier") &&
            $CurrentForm->hasValue("o_Supplier") &&
            $this->Supplier->CurrentValue != $this->Supplier->DefaultValue &&
            !($this->Supplier->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Supplier->CurrentValue == $this->Supplier->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_CampCode") &&
            $CurrentForm->hasValue("o_CampCode") &&
            $this->CampCode->CurrentValue != $this->CampCode->DefaultValue &&
            !($this->CampCode->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->CampCode->CurrentValue == $this->CampCode->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Camp") &&
            $CurrentForm->hasValue("o_Camp") &&
            $this->Camp->CurrentValue != $this->Camp->DefaultValue &&
            !($this->Camp->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Camp->CurrentValue == $this->Camp->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_VillageCode") &&
            $CurrentForm->hasValue("o_VillageCode") &&
            $this->VillageCode->CurrentValue != $this->VillageCode->DefaultValue &&
            !($this->VillageCode->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->VillageCode->CurrentValue == $this->VillageCode->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Village") &&
            $CurrentForm->hasValue("o_Village") &&
            $this->Village->CurrentValue != $this->Village->DefaultValue &&
            !($this->Village->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Village->CurrentValue == $this->Village->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_NRC") &&
            $CurrentForm->hasValue("o_NRC") &&
            $this->NRC->CurrentValue != $this->NRC->DefaultValue &&
            !($this->NRC->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->NRC->CurrentValue == $this->NRC->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_FarmerName") &&
            $CurrentForm->hasValue("o_FarmerName") &&
            $this->FarmerName->CurrentValue != $this->FarmerName->DefaultValue &&
            !($this->FarmerName->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->FarmerName->CurrentValue == $this->FarmerName->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_DOB") &&
            $CurrentForm->hasValue("o_DOB") &&
            $this->DOB->CurrentValue != $this->DOB->DefaultValue &&
            !($this->DOB->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->DOB->CurrentValue == $this->DOB->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Age") &&
            $CurrentForm->hasValue("o_Age") &&
            $this->Age->CurrentValue != $this->Age->DefaultValue &&
            !($this->Age->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Age->CurrentValue == $this->Age->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Group") &&
            $CurrentForm->hasValue("o_Group") &&
            $this->Group->CurrentValue != $this->Group->DefaultValue &&
            !($this->Group->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Group->CurrentValue == $this->Group->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Gender") &&
            $CurrentForm->hasValue("o_Gender") &&
            $this->Gender->CurrentValue != $this->Gender->DefaultValue &&
            !($this->Gender->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Gender->CurrentValue == $this->Gender->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Seedlings") &&
            $CurrentForm->hasValue("o_Seedlings") &&
            $this->Seedlings->CurrentValue != $this->Seedlings->DefaultValue &&
            !($this->Seedlings->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Seedlings->CurrentValue == $this->Seedlings->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Seeds") &&
            $CurrentForm->hasValue("o_Seeds") &&
            $this->Seeds->CurrentValue != $this->Seeds->DefaultValue &&
            !($this->Seeds->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Seeds->CurrentValue == $this->Seeds->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_Type") &&
            $CurrentForm->hasValue("o_Type") &&
            $this->Type->CurrentValue != $this->Type->DefaultValue &&
            !($this->Type->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->Type->CurrentValue == $this->Type->getSessionValue())
        ) {
            return false;
        }
        if (
            $CurrentForm->hasValue("x_DateAllocated") &&
            $CurrentForm->hasValue("o_DateAllocated") &&
            $this->DateAllocated->CurrentValue != $this->DateAllocated->DefaultValue &&
            !($this->DateAllocated->IsForeignKey && $this->getCurrentMasterTable() != "" && $this->DateAllocated->CurrentValue == $this->DateAllocated->getSessionValue())
        ) {
            return false;
        }
        return true;
    }

    // Validate grid form
    public function validateGridForm()
    {
        global $CurrentForm;

        // Get row count
        $CurrentForm->resetIndex();
        $rowcnt = strval($CurrentForm->getValue($this->FormKeyCountName));
        if ($rowcnt == "" || !is_numeric($rowcnt)) {
            $rowcnt = 0;
        }

        // Load default values for emptyRow checking
        $this->loadDefaultValues();

        // Validate all records
        for ($rowindex = 1; $rowindex <= $rowcnt; $rowindex++) {
            // Load current row values
            $CurrentForm->Index = $rowindex;
            $rowaction = strval($CurrentForm->getValue($this->FormActionName));
            if ($rowaction != "delete" && $rowaction != "insertdelete" && $rowaction != "hide") {
                $this->loadFormValues(); // Get form values
                if ($rowaction == "insert" && $this->emptyRow()) {
                    // Ignore
                } elseif (!$this->validateForm()) {
                    $this->ValidationErrors[$rowindex] = $this->getValidationErrors();
                    $this->EventCancelled = true;
                    return false;
                }
            }
        }
        return true;
    }

    // Get all form values of the grid
    public function getGridFormValues()
    {
        global $CurrentForm;
        // Get row count
        $CurrentForm->resetIndex();
        $rowcnt = strval($CurrentForm->getValue($this->FormKeyCountName));
        if ($rowcnt == "" || !is_numeric($rowcnt)) {
            $rowcnt = 0;
        }
        $rows = [];

        // Loop through all records
        for ($rowindex = 1; $rowindex <= $rowcnt; $rowindex++) {
            // Load current row values
            $CurrentForm->Index = $rowindex;
            $rowaction = strval($CurrentForm->getValue($this->FormActionName));
            if ($rowaction != "delete" && $rowaction != "insertdelete") {
                $this->loadFormValues(); // Get form values
                if ($rowaction == "insert" && $this->emptyRow()) {
                    // Ignore
                } else {
                    $rows[] = $this->getFieldValues("FormValue"); // Return row as array
                }
            }
        }
        return $rows; // Return as array of array
    }

    // Restore form values for current row
    public function restoreCurrentRowFormValues($idx)
    {
        global $CurrentForm;

        // Get row based on current index
        $CurrentForm->Index = $idx;
        $rowaction = strval($CurrentForm->getValue($this->FormActionName));
        $this->loadFormValues(); // Load form values
        // Set up invalid status correctly
        $this->resetFormError();
        if ($rowaction == "insert" && $this->emptyRow()) {
            // Ignore
        } else {
            $this->validateForm();
        }
    }

    // Reset form status
    public function resetFormError()
    {
        foreach ($this->Fields as $field) {
            $field->clearErrorMessage();
        }
    }

    // Get list of filters
    public function getFilterList()
    {
        // Initialize
        $filterList = "";
        $savedFilterList = "";

        // Load server side filters
        if (Config("SEARCH_FILTER_OPTION") == "Server") {
            $savedFilterList = Profile()->getSearchFilters("fcidp_datasrch");
        }
        $filterList = Concat($filterList, $this->DistrictCode->AdvancedSearch->toJson(), ","); // Field DistrictCode
        $filterList = Concat($filterList, $this->DistrictName->AdvancedSearch->toJson(), ","); // Field DistrictName
        $filterList = Concat($filterList, $this->GRNCode->AdvancedSearch->toJson(), ","); // Field GRNCode
        $filterList = Concat($filterList, $this->GRN->AdvancedSearch->toJson(), ","); // Field GRN
        $filterList = Concat($filterList, $this->SupplierCode->AdvancedSearch->toJson(), ","); // Field SupplierCode
        $filterList = Concat($filterList, $this->Supplier->AdvancedSearch->toJson(), ","); // Field Supplier
        $filterList = Concat($filterList, $this->CampCode->AdvancedSearch->toJson(), ","); // Field CampCode
        $filterList = Concat($filterList, $this->Camp->AdvancedSearch->toJson(), ","); // Field Camp
        $filterList = Concat($filterList, $this->VillageCode->AdvancedSearch->toJson(), ","); // Field VillageCode
        $filterList = Concat($filterList, $this->Village->AdvancedSearch->toJson(), ","); // Field Village
        $filterList = Concat($filterList, $this->NRC->AdvancedSearch->toJson(), ","); // Field NRC
        $filterList = Concat($filterList, $this->FarmerName->AdvancedSearch->toJson(), ","); // Field FarmerName
        $filterList = Concat($filterList, $this->DOB->AdvancedSearch->toJson(), ","); // Field DOB
        $filterList = Concat($filterList, $this->Age->AdvancedSearch->toJson(), ","); // Field Age
        $filterList = Concat($filterList, $this->Group->AdvancedSearch->toJson(), ","); // Field Group
        $filterList = Concat($filterList, $this->Gender->AdvancedSearch->toJson(), ","); // Field Gender
        $filterList = Concat($filterList, $this->Seedlings->AdvancedSearch->toJson(), ","); // Field Seedlings
        $filterList = Concat($filterList, $this->Seeds->AdvancedSearch->toJson(), ","); // Field Seeds
        $filterList = Concat($filterList, $this->Type->AdvancedSearch->toJson(), ","); // Field Type
        $filterList = Concat($filterList, $this->DateAllocated->AdvancedSearch->toJson(), ","); // Field DateAllocated
        $filterList = Concat($filterList, $this->ID->AdvancedSearch->toJson(), ","); // Field ID
        if ($this->BasicSearch->Keyword != "") {
            $wrk = "\"" . Config("TABLE_BASIC_SEARCH") . "\":\"" . JsEncode($this->BasicSearch->Keyword) . "\",\"" . Config("TABLE_BASIC_SEARCH_TYPE") . "\":\"" . JsEncode($this->BasicSearch->Type) . "\"";
            $filterList = Concat($filterList, $wrk, ",");
        }

        // Return filter list in JSON
        if ($filterList != "") {
            $filterList = "\"data\":{" . $filterList . "}";
        }
        if ($savedFilterList != "") {
            $filterList = Concat($filterList, "\"filters\":" . $savedFilterList, ",");
        }
        return ($filterList != "") ? "{" . $filterList . "}" : "null";
    }

    // Process filter list
    protected function processFilterList()
    {
        if (Post("ajax") == "savefilters") { // Save filter request (Ajax)
            $filters = Post("filters");
            Profile()->setSearchFilters("fcidp_datasrch", $filters);
            WriteJson([["success" => true]]); // Success
            return true;
        } elseif (Post("cmd") == "resetfilter") {
            $this->restoreFilterList();
        }
        return false;
    }

    // Restore list of filters
    protected function restoreFilterList()
    {
        // Return if not reset filter
        if (Post("cmd") !== "resetfilter") {
            return false;
        }
        $filter = json_decode(Post("filter"), true);
        $this->Command = "search";

        // Field DistrictCode
        $this->DistrictCode->AdvancedSearch->SearchValue = @$filter["x_DistrictCode"];
        $this->DistrictCode->AdvancedSearch->SearchOperator = @$filter["z_DistrictCode"];
        $this->DistrictCode->AdvancedSearch->SearchCondition = @$filter["v_DistrictCode"];
        $this->DistrictCode->AdvancedSearch->SearchValue2 = @$filter["y_DistrictCode"];
        $this->DistrictCode->AdvancedSearch->SearchOperator2 = @$filter["w_DistrictCode"];
        $this->DistrictCode->AdvancedSearch->save();

        // Field DistrictName
        $this->DistrictName->AdvancedSearch->SearchValue = @$filter["x_DistrictName"];
        $this->DistrictName->AdvancedSearch->SearchOperator = @$filter["z_DistrictName"];
        $this->DistrictName->AdvancedSearch->SearchCondition = @$filter["v_DistrictName"];
        $this->DistrictName->AdvancedSearch->SearchValue2 = @$filter["y_DistrictName"];
        $this->DistrictName->AdvancedSearch->SearchOperator2 = @$filter["w_DistrictName"];
        $this->DistrictName->AdvancedSearch->save();

        // Field GRNCode
        $this->GRNCode->AdvancedSearch->SearchValue = @$filter["x_GRNCode"];
        $this->GRNCode->AdvancedSearch->SearchOperator = @$filter["z_GRNCode"];
        $this->GRNCode->AdvancedSearch->SearchCondition = @$filter["v_GRNCode"];
        $this->GRNCode->AdvancedSearch->SearchValue2 = @$filter["y_GRNCode"];
        $this->GRNCode->AdvancedSearch->SearchOperator2 = @$filter["w_GRNCode"];
        $this->GRNCode->AdvancedSearch->save();

        // Field GRN
        $this->GRN->AdvancedSearch->SearchValue = @$filter["x_GRN"];
        $this->GRN->AdvancedSearch->SearchOperator = @$filter["z_GRN"];
        $this->GRN->AdvancedSearch->SearchCondition = @$filter["v_GRN"];
        $this->GRN->AdvancedSearch->SearchValue2 = @$filter["y_GRN"];
        $this->GRN->AdvancedSearch->SearchOperator2 = @$filter["w_GRN"];
        $this->GRN->AdvancedSearch->save();

        // Field SupplierCode
        $this->SupplierCode->AdvancedSearch->SearchValue = @$filter["x_SupplierCode"];
        $this->SupplierCode->AdvancedSearch->SearchOperator = @$filter["z_SupplierCode"];
        $this->SupplierCode->AdvancedSearch->SearchCondition = @$filter["v_SupplierCode"];
        $this->SupplierCode->AdvancedSearch->SearchValue2 = @$filter["y_SupplierCode"];
        $this->SupplierCode->AdvancedSearch->SearchOperator2 = @$filter["w_SupplierCode"];
        $this->SupplierCode->AdvancedSearch->save();

        // Field Supplier
        $this->Supplier->AdvancedSearch->SearchValue = @$filter["x_Supplier"];
        $this->Supplier->AdvancedSearch->SearchOperator = @$filter["z_Supplier"];
        $this->Supplier->AdvancedSearch->SearchCondition = @$filter["v_Supplier"];
        $this->Supplier->AdvancedSearch->SearchValue2 = @$filter["y_Supplier"];
        $this->Supplier->AdvancedSearch->SearchOperator2 = @$filter["w_Supplier"];
        $this->Supplier->AdvancedSearch->save();

        // Field CampCode
        $this->CampCode->AdvancedSearch->SearchValue = @$filter["x_CampCode"];
        $this->CampCode->AdvancedSearch->SearchOperator = @$filter["z_CampCode"];
        $this->CampCode->AdvancedSearch->SearchCondition = @$filter["v_CampCode"];
        $this->CampCode->AdvancedSearch->SearchValue2 = @$filter["y_CampCode"];
        $this->CampCode->AdvancedSearch->SearchOperator2 = @$filter["w_CampCode"];
        $this->CampCode->AdvancedSearch->save();

        // Field Camp
        $this->Camp->AdvancedSearch->SearchValue = @$filter["x_Camp"];
        $this->Camp->AdvancedSearch->SearchOperator = @$filter["z_Camp"];
        $this->Camp->AdvancedSearch->SearchCondition = @$filter["v_Camp"];
        $this->Camp->AdvancedSearch->SearchValue2 = @$filter["y_Camp"];
        $this->Camp->AdvancedSearch->SearchOperator2 = @$filter["w_Camp"];
        $this->Camp->AdvancedSearch->save();

        // Field VillageCode
        $this->VillageCode->AdvancedSearch->SearchValue = @$filter["x_VillageCode"];
        $this->VillageCode->AdvancedSearch->SearchOperator = @$filter["z_VillageCode"];
        $this->VillageCode->AdvancedSearch->SearchCondition = @$filter["v_VillageCode"];
        $this->VillageCode->AdvancedSearch->SearchValue2 = @$filter["y_VillageCode"];
        $this->VillageCode->AdvancedSearch->SearchOperator2 = @$filter["w_VillageCode"];
        $this->VillageCode->AdvancedSearch->save();

        // Field Village
        $this->Village->AdvancedSearch->SearchValue = @$filter["x_Village"];
        $this->Village->AdvancedSearch->SearchOperator = @$filter["z_Village"];
        $this->Village->AdvancedSearch->SearchCondition = @$filter["v_Village"];
        $this->Village->AdvancedSearch->SearchValue2 = @$filter["y_Village"];
        $this->Village->AdvancedSearch->SearchOperator2 = @$filter["w_Village"];
        $this->Village->AdvancedSearch->save();

        // Field NRC
        $this->NRC->AdvancedSearch->SearchValue = @$filter["x_NRC"];
        $this->NRC->AdvancedSearch->SearchOperator = @$filter["z_NRC"];
        $this->NRC->AdvancedSearch->SearchCondition = @$filter["v_NRC"];
        $this->NRC->AdvancedSearch->SearchValue2 = @$filter["y_NRC"];
        $this->NRC->AdvancedSearch->SearchOperator2 = @$filter["w_NRC"];
        $this->NRC->AdvancedSearch->save();

        // Field FarmerName
        $this->FarmerName->AdvancedSearch->SearchValue = @$filter["x_FarmerName"];
        $this->FarmerName->AdvancedSearch->SearchOperator = @$filter["z_FarmerName"];
        $this->FarmerName->AdvancedSearch->SearchCondition = @$filter["v_FarmerName"];
        $this->FarmerName->AdvancedSearch->SearchValue2 = @$filter["y_FarmerName"];
        $this->FarmerName->AdvancedSearch->SearchOperator2 = @$filter["w_FarmerName"];
        $this->FarmerName->AdvancedSearch->save();

        // Field DOB
        $this->DOB->AdvancedSearch->SearchValue = @$filter["x_DOB"];
        $this->DOB->AdvancedSearch->SearchOperator = @$filter["z_DOB"];
        $this->DOB->AdvancedSearch->SearchCondition = @$filter["v_DOB"];
        $this->DOB->AdvancedSearch->SearchValue2 = @$filter["y_DOB"];
        $this->DOB->AdvancedSearch->SearchOperator2 = @$filter["w_DOB"];
        $this->DOB->AdvancedSearch->save();

        // Field Age
        $this->Age->AdvancedSearch->SearchValue = @$filter["x_Age"];
        $this->Age->AdvancedSearch->SearchOperator = @$filter["z_Age"];
        $this->Age->AdvancedSearch->SearchCondition = @$filter["v_Age"];
        $this->Age->AdvancedSearch->SearchValue2 = @$filter["y_Age"];
        $this->Age->AdvancedSearch->SearchOperator2 = @$filter["w_Age"];
        $this->Age->AdvancedSearch->save();

        // Field Group
        $this->Group->AdvancedSearch->SearchValue = @$filter["x_Group"];
        $this->Group->AdvancedSearch->SearchOperator = @$filter["z_Group"];
        $this->Group->AdvancedSearch->SearchCondition = @$filter["v_Group"];
        $this->Group->AdvancedSearch->SearchValue2 = @$filter["y_Group"];
        $this->Group->AdvancedSearch->SearchOperator2 = @$filter["w_Group"];
        $this->Group->AdvancedSearch->save();

        // Field Gender
        $this->Gender->AdvancedSearch->SearchValue = @$filter["x_Gender"];
        $this->Gender->AdvancedSearch->SearchOperator = @$filter["z_Gender"];
        $this->Gender->AdvancedSearch->SearchCondition = @$filter["v_Gender"];
        $this->Gender->AdvancedSearch->SearchValue2 = @$filter["y_Gender"];
        $this->Gender->AdvancedSearch->SearchOperator2 = @$filter["w_Gender"];
        $this->Gender->AdvancedSearch->save();

        // Field Seedlings
        $this->Seedlings->AdvancedSearch->SearchValue = @$filter["x_Seedlings"];
        $this->Seedlings->AdvancedSearch->SearchOperator = @$filter["z_Seedlings"];
        $this->Seedlings->AdvancedSearch->SearchCondition = @$filter["v_Seedlings"];
        $this->Seedlings->AdvancedSearch->SearchValue2 = @$filter["y_Seedlings"];
        $this->Seedlings->AdvancedSearch->SearchOperator2 = @$filter["w_Seedlings"];
        $this->Seedlings->AdvancedSearch->save();

        // Field Seeds
        $this->Seeds->AdvancedSearch->SearchValue = @$filter["x_Seeds"];
        $this->Seeds->AdvancedSearch->SearchOperator = @$filter["z_Seeds"];
        $this->Seeds->AdvancedSearch->SearchCondition = @$filter["v_Seeds"];
        $this->Seeds->AdvancedSearch->SearchValue2 = @$filter["y_Seeds"];
        $this->Seeds->AdvancedSearch->SearchOperator2 = @$filter["w_Seeds"];
        $this->Seeds->AdvancedSearch->save();

        // Field Type
        $this->Type->AdvancedSearch->SearchValue = @$filter["x_Type"];
        $this->Type->AdvancedSearch->SearchOperator = @$filter["z_Type"];
        $this->Type->AdvancedSearch->SearchCondition = @$filter["v_Type"];
        $this->Type->AdvancedSearch->SearchValue2 = @$filter["y_Type"];
        $this->Type->AdvancedSearch->SearchOperator2 = @$filter["w_Type"];
        $this->Type->AdvancedSearch->save();

        // Field DateAllocated
        $this->DateAllocated->AdvancedSearch->SearchValue = @$filter["x_DateAllocated"];
        $this->DateAllocated->AdvancedSearch->SearchOperator = @$filter["z_DateAllocated"];
        $this->DateAllocated->AdvancedSearch->SearchCondition = @$filter["v_DateAllocated"];
        $this->DateAllocated->AdvancedSearch->SearchValue2 = @$filter["y_DateAllocated"];
        $this->DateAllocated->AdvancedSearch->SearchOperator2 = @$filter["w_DateAllocated"];
        $this->DateAllocated->AdvancedSearch->save();

        // Field ID
        $this->ID->AdvancedSearch->SearchValue = @$filter["x_ID"];
        $this->ID->AdvancedSearch->SearchOperator = @$filter["z_ID"];
        $this->ID->AdvancedSearch->SearchCondition = @$filter["v_ID"];
        $this->ID->AdvancedSearch->SearchValue2 = @$filter["y_ID"];
        $this->ID->AdvancedSearch->SearchOperator2 = @$filter["w_ID"];
        $this->ID->AdvancedSearch->save();
        $this->BasicSearch->setKeyword(@$filter[Config("TABLE_BASIC_SEARCH")]);
        $this->BasicSearch->setType(@$filter[Config("TABLE_BASIC_SEARCH_TYPE")]);
    }

    // Advanced search WHERE clause based on QueryString
    public function advancedSearchWhere($default = false)
    {
        global $Security;
        $where = "";
        if (!$Security->canSearch()) {
            return "";
        }
        $this->buildSearchSql($where, $this->DistrictCode, $default, false); // DistrictCode
        $this->buildSearchSql($where, $this->DistrictName, $default, false); // DistrictName
        $this->buildSearchSql($where, $this->GRNCode, $default, false); // GRNCode
        $this->buildSearchSql($where, $this->GRN, $default, false); // GRN
        $this->buildSearchSql($where, $this->SupplierCode, $default, false); // SupplierCode
        $this->buildSearchSql($where, $this->Supplier, $default, false); // Supplier
        $this->buildSearchSql($where, $this->CampCode, $default, false); // CampCode
        $this->buildSearchSql($where, $this->Camp, $default, false); // Camp
        $this->buildSearchSql($where, $this->VillageCode, $default, false); // VillageCode
        $this->buildSearchSql($where, $this->Village, $default, false); // Village
        $this->buildSearchSql($where, $this->NRC, $default, false); // NRC
        $this->buildSearchSql($where, $this->FarmerName, $default, false); // FarmerName
        $this->buildSearchSql($where, $this->DOB, $default, false); // DOB
        $this->buildSearchSql($where, $this->Age, $default, false); // Age
        $this->buildSearchSql($where, $this->Group, $default, true); // Group
        $this->buildSearchSql($where, $this->Gender, $default, false); // Gender
        $this->buildSearchSql($where, $this->Seedlings, $default, false); // Seedlings
        $this->buildSearchSql($where, $this->Seeds, $default, false); // Seeds
        $this->buildSearchSql($where, $this->Type, $default, false); // Type
        $this->buildSearchSql($where, $this->DateAllocated, $default, false); // DateAllocated
        $this->buildSearchSql($where, $this->ID, $default, false); // ID

        // Set up search command
        if (!$default && $where != "" && in_array($this->Command, ["", "reset", "resetall"])) {
            $this->Command = "search";
        }
        if (!$default && $this->Command == "search") {
            $this->DistrictCode->AdvancedSearch->save(); // DistrictCode
            $this->DistrictName->AdvancedSearch->save(); // DistrictName
            $this->GRNCode->AdvancedSearch->save(); // GRNCode
            $this->GRN->AdvancedSearch->save(); // GRN
            $this->SupplierCode->AdvancedSearch->save(); // SupplierCode
            $this->Supplier->AdvancedSearch->save(); // Supplier
            $this->CampCode->AdvancedSearch->save(); // CampCode
            $this->Camp->AdvancedSearch->save(); // Camp
            $this->VillageCode->AdvancedSearch->save(); // VillageCode
            $this->Village->AdvancedSearch->save(); // Village
            $this->NRC->AdvancedSearch->save(); // NRC
            $this->FarmerName->AdvancedSearch->save(); // FarmerName
            $this->DOB->AdvancedSearch->save(); // DOB
            $this->Age->AdvancedSearch->save(); // Age
            $this->Group->AdvancedSearch->save(); // Group
            $this->Gender->AdvancedSearch->save(); // Gender
            $this->Seedlings->AdvancedSearch->save(); // Seedlings
            $this->Seeds->AdvancedSearch->save(); // Seeds
            $this->Type->AdvancedSearch->save(); // Type
            $this->DateAllocated->AdvancedSearch->save(); // DateAllocated
            $this->ID->AdvancedSearch->save(); // ID

            // Clear rules for QueryBuilder
            $this->setSessionRules("");
        }
        return $where;
    }

    // Query builder rules
    public function queryBuilderRules()
    {
        return Post("rules") ?? $this->getSessionRules();
    }

    // Quey builder WHERE clause
    public function queryBuilderWhere($fieldName = "")
    {
        global $Security;
        if (!$Security->canSearch()) {
            return "";
        }

        // Get rules by query builder
        $rules = $this->queryBuilderRules();

        // Decode and parse rules
        $where = $rules ? $this->parseRules(json_decode($rules, true), $fieldName) : "";

        // Clear other search and save rules to session
        if ($where && $fieldName == "") { // Skip if get query for specific field
            $this->resetSearchParms();
            $this->DistrictCode->AdvancedSearch->save(); // DistrictCode
            $this->DistrictName->AdvancedSearch->save(); // DistrictName
            $this->GRNCode->AdvancedSearch->save(); // GRNCode
            $this->GRN->AdvancedSearch->save(); // GRN
            $this->SupplierCode->AdvancedSearch->save(); // SupplierCode
            $this->Supplier->AdvancedSearch->save(); // Supplier
            $this->CampCode->AdvancedSearch->save(); // CampCode
            $this->Camp->AdvancedSearch->save(); // Camp
            $this->VillageCode->AdvancedSearch->save(); // VillageCode
            $this->Village->AdvancedSearch->save(); // Village
            $this->NRC->AdvancedSearch->save(); // NRC
            $this->FarmerName->AdvancedSearch->save(); // FarmerName
            $this->DOB->AdvancedSearch->save(); // DOB
            $this->Age->AdvancedSearch->save(); // Age
            $this->Group->AdvancedSearch->save(); // Group
            $this->Gender->AdvancedSearch->save(); // Gender
            $this->Seedlings->AdvancedSearch->save(); // Seedlings
            $this->Seeds->AdvancedSearch->save(); // Seeds
            $this->Type->AdvancedSearch->save(); // Type
            $this->DateAllocated->AdvancedSearch->save(); // DateAllocated
            $this->ID->AdvancedSearch->save(); // ID
            $this->setSessionRules($rules);
        }

        // Return query
        return $where;
    }

    // Build search SQL
    protected function buildSearchSql(&$where, $fld, $default, $multiValue)
    {
        $fldParm = $fld->Param;
        $fldVal = $default ? $fld->AdvancedSearch->SearchValueDefault : $fld->AdvancedSearch->SearchValue;
        $fldOpr = $default ? $fld->AdvancedSearch->SearchOperatorDefault : $fld->AdvancedSearch->SearchOperator;
        $fldCond = $default ? $fld->AdvancedSearch->SearchConditionDefault : $fld->AdvancedSearch->SearchCondition;
        $fldVal2 = $default ? $fld->AdvancedSearch->SearchValue2Default : $fld->AdvancedSearch->SearchValue2;
        $fldOpr2 = $default ? $fld->AdvancedSearch->SearchOperator2Default : $fld->AdvancedSearch->SearchOperator2;
        $fldVal = ConvertSearchValue($fldVal, $fldOpr, $fld);
        $fldVal2 = ConvertSearchValue($fldVal2, $fldOpr2, $fld);
        $fldOpr = ConvertSearchOperator($fldOpr, $fld, $fldVal);
        $fldOpr2 = ConvertSearchOperator($fldOpr2, $fld, $fldVal2);
        $wrk = "";
        $sep = $fld->UseFilter ? Config("FILTER_OPTION_SEPARATOR") : Config("MULTIPLE_OPTION_SEPARATOR");
        if (is_array($fldVal)) {
            $fldVal = implode($sep, $fldVal);
        }
        if (is_array($fldVal2)) {
            $fldVal2 = implode($sep, $fldVal2);
        }
        if (Config("SEARCH_MULTI_VALUE_OPTION") == 1 && !$fld->UseFilter || !IsMultiSearchOperator($fldOpr)) {
            $multiValue = false;
        }
        if ($multiValue) {
            $wrk = $fldVal != "" ? GetMultiSearchSql($fld, $fldOpr, $fldVal, $this->Dbid) : ""; // Field value 1
            $wrk2 = $fldVal2 != "" ? GetMultiSearchSql($fld, $fldOpr2, $fldVal2, $this->Dbid) : ""; // Field value 2
            AddFilter($wrk, $wrk2, $fldCond);
        } else {
            $wrk = GetSearchSql($fld, $fldVal, $fldOpr, $fldCond, $fldVal2, $fldOpr2, $this->Dbid);
        }
        if ($this->SearchOption == "AUTO" && in_array($this->BasicSearch->getType(), ["AND", "OR"])) {
            $cond = $this->BasicSearch->getType();
        } else {
            $cond = SameText($this->SearchOption, "OR") ? "OR" : "AND";
        }
        AddFilter($where, $wrk, $cond);
    }

    // Show list of filters
    public function showFilterList()
    {
        global $Language;

        // Initialize
        $filterList = "";
        $captionClass = $this->isExport("email") ? "ew-filter-caption-email" : "ew-filter-caption";
        $captionSuffix = $this->isExport("email") ? ": " : "";

        // Field DistrictCode
        $filter = $this->queryBuilderWhere("DistrictCode");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->DistrictCode, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->DistrictCode->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field DistrictName
        $filter = $this->queryBuilderWhere("DistrictName");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->DistrictName, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->DistrictName->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field GRNCode
        $filter = $this->queryBuilderWhere("GRNCode");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->GRNCode, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->GRNCode->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field GRN
        $filter = $this->queryBuilderWhere("GRN");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->GRN, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->GRN->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field SupplierCode
        $filter = $this->queryBuilderWhere("SupplierCode");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->SupplierCode, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->SupplierCode->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Supplier
        $filter = $this->queryBuilderWhere("Supplier");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Supplier, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Supplier->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field CampCode
        $filter = $this->queryBuilderWhere("CampCode");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->CampCode, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->CampCode->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Camp
        $filter = $this->queryBuilderWhere("Camp");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Camp, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Camp->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field VillageCode
        $filter = $this->queryBuilderWhere("VillageCode");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->VillageCode, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->VillageCode->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Village
        $filter = $this->queryBuilderWhere("Village");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Village, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Village->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field NRC
        $filter = $this->queryBuilderWhere("NRC");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->NRC, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->NRC->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field FarmerName
        $filter = $this->queryBuilderWhere("FarmerName");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->FarmerName, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->FarmerName->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field DOB
        $filter = $this->queryBuilderWhere("DOB");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->DOB, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->DOB->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Age
        $filter = $this->queryBuilderWhere("Age");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Age, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Age->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Group
        $filter = $this->queryBuilderWhere("Group");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Group, false, true);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Group->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Gender
        $filter = $this->queryBuilderWhere("Gender");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Gender, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Gender->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Seedlings
        $filter = $this->queryBuilderWhere("Seedlings");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Seedlings, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Seedlings->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Seeds
        $filter = $this->queryBuilderWhere("Seeds");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Seeds, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Seeds->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field Type
        $filter = $this->queryBuilderWhere("Type");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->Type, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->Type->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field DateAllocated
        $filter = $this->queryBuilderWhere("DateAllocated");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->DateAllocated, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->DateAllocated->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }

        // Field ID
        $filter = $this->queryBuilderWhere("ID");
        if (!$filter) {
            $this->buildSearchSql($filter, $this->ID, false, false);
        }
        if ($filter != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $this->ID->caption() . "</span>" . $captionSuffix . $filter . "</div>";
        }
        if ($this->BasicSearch->Keyword != "") {
            $filterList .= "<div><span class=\"" . $captionClass . "\">" . $Language->phrase("BasicSearchKeyword") . "</span>" . $captionSuffix . $this->BasicSearch->Keyword . "</div>";
        }

        // Show Filters
        if ($filterList != "") {
            $message = "<div id=\"ew-filter-list\" class=\"callout callout-info d-table\"><div id=\"ew-current-filters\">" .
                $Language->phrase("CurrentFilters") . "</div>" . $filterList . "</div>";
            $this->messageShowing($message, "");
            Write($message);
        } else { // Output empty tag
            Write("<div id=\"ew-filter-list\"></div>");
        }
    }

    // Return basic search WHERE clause based on search keyword and type
    public function basicSearchWhere($default = false)
    {
        global $Security;
        $searchStr = "";
        if (!$Security->canSearch()) {
            return "";
        }

        // Fields to search
        $searchFlds = [];
        $searchFlds[] = &$this->DistrictName;
        $searchFlds[] = &$this->GRNCode;
        $searchFlds[] = &$this->GRN;
        $searchFlds[] = &$this->Supplier;
        $searchFlds[] = &$this->Camp;
        $searchFlds[] = &$this->Village;
        $searchFlds[] = &$this->NRC;
        $searchFlds[] = &$this->FarmerName;
        $searchFlds[] = &$this->DOB;
        $searchFlds[] = &$this->Group;
        $searchFlds[] = &$this->Gender;
        $searchFlds[] = &$this->Seeds;
        $searchFlds[] = &$this->Type;
        $searchFlds[] = &$this->DateAllocated;
        $searchKeyword = $default ? $this->BasicSearch->KeywordDefault : $this->BasicSearch->Keyword;
        $searchType = $default ? $this->BasicSearch->TypeDefault : $this->BasicSearch->Type;

        // Get search SQL
        if ($searchKeyword != "") {
            $ar = $this->BasicSearch->keywordList($default);
            $searchStr = GetQuickSearchFilter($searchFlds, $ar, $searchType, Config("BASIC_SEARCH_ANY_FIELDS"), $this->Dbid);
            if (!$default && in_array($this->Command, ["", "reset", "resetall"])) {
                $this->Command = "search";
            }
        }
        if (!$default && $this->Command == "search") {
            $this->BasicSearch->setKeyword($searchKeyword);
            $this->BasicSearch->setType($searchType);

            // Clear rules for QueryBuilder
            $this->setSessionRules("");
        }
        return $searchStr;
    }

    // Check if search parm exists
    protected function checkSearchParms()
    {
        // Check basic search
        if ($this->BasicSearch->issetSession()) {
            return true;
        }
        if ($this->DistrictCode->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->DistrictName->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->GRNCode->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->GRN->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->SupplierCode->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Supplier->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->CampCode->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Camp->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->VillageCode->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Village->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->NRC->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->FarmerName->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->DOB->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Age->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Group->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Gender->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Seedlings->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Seeds->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->Type->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->DateAllocated->AdvancedSearch->issetSession()) {
            return true;
        }
        if ($this->ID->AdvancedSearch->issetSession()) {
            return true;
        }
        return false;
    }

    // Clear all search parameters
    protected function resetSearchParms()
    {
        // Clear search WHERE clause
        $this->SearchWhere = "";
        $this->setSearchWhere($this->SearchWhere);

        // Clear basic search parameters
        $this->resetBasicSearchParms();

        // Clear advanced search parameters
        $this->resetAdvancedSearchParms();

        // Clear queryBuilder
        $this->setSessionRules("");
    }

    // Load advanced search default values
    protected function loadAdvancedSearchDefault()
    {
        return false;
    }

    // Clear all basic search parameters
    protected function resetBasicSearchParms()
    {
        $this->BasicSearch->unsetSession();
    }

    // Clear all advanced search parameters
    protected function resetAdvancedSearchParms()
    {
        $this->DistrictCode->AdvancedSearch->unsetSession();
        $this->DistrictName->AdvancedSearch->unsetSession();
        $this->GRNCode->AdvancedSearch->unsetSession();
        $this->GRN->AdvancedSearch->unsetSession();
        $this->SupplierCode->AdvancedSearch->unsetSession();
        $this->Supplier->AdvancedSearch->unsetSession();
        $this->CampCode->AdvancedSearch->unsetSession();
        $this->Camp->AdvancedSearch->unsetSession();
        $this->VillageCode->AdvancedSearch->unsetSession();
        $this->Village->AdvancedSearch->unsetSession();
        $this->NRC->AdvancedSearch->unsetSession();
        $this->FarmerName->AdvancedSearch->unsetSession();
        $this->DOB->AdvancedSearch->unsetSession();
        $this->Age->AdvancedSearch->unsetSession();
        $this->Group->AdvancedSearch->unsetSession();
        $this->Gender->AdvancedSearch->unsetSession();
        $this->Seedlings->AdvancedSearch->unsetSession();
        $this->Seeds->AdvancedSearch->unsetSession();
        $this->Type->AdvancedSearch->unsetSession();
        $this->DateAllocated->AdvancedSearch->unsetSession();
        $this->ID->AdvancedSearch->unsetSession();
    }

    // Restore all search parameters
    protected function restoreSearchParms()
    {
        $this->RestoreSearch = true;

        // Restore basic search values
        $this->BasicSearch->load();

        // Restore advanced search values
        $this->DistrictCode->AdvancedSearch->load();
        $this->DistrictName->AdvancedSearch->load();
        $this->GRNCode->AdvancedSearch->load();
        $this->GRN->AdvancedSearch->load();
        $this->SupplierCode->AdvancedSearch->load();
        $this->Supplier->AdvancedSearch->load();
        $this->CampCode->AdvancedSearch->load();
        $this->Camp->AdvancedSearch->load();
        $this->VillageCode->AdvancedSearch->load();
        $this->Village->AdvancedSearch->load();
        $this->NRC->AdvancedSearch->load();
        $this->FarmerName->AdvancedSearch->load();
        $this->DOB->AdvancedSearch->load();
        $this->Age->AdvancedSearch->load();
        $this->Group->AdvancedSearch->load();
        $this->Gender->AdvancedSearch->load();
        $this->Seedlings->AdvancedSearch->load();
        $this->Seeds->AdvancedSearch->load();
        $this->Type->AdvancedSearch->load();
        $this->DateAllocated->AdvancedSearch->load();
        $this->ID->AdvancedSearch->load();
    }

    // Set up sort parameters
    protected function setupSortOrder()
    {
        // Load default Sorting Order
        if ($this->Command != "json") {
            $defaultSort = ""; // Set up default sort
            if ($this->getSessionOrderBy() == "" && $defaultSort != "") {
                $this->setSessionOrderBy($defaultSort);
            }
        }

        // Check for "order" parameter
        if (Get("order") !== null) {
            $this->CurrentOrder = Get("order");
            $this->CurrentOrderType = Get("ordertype", "");
            $this->updateSort($this->DistrictCode); // DistrictCode
            $this->updateSort($this->DistrictName); // DistrictName
            $this->updateSort($this->GRNCode); // GRNCode
            $this->updateSort($this->GRN); // GRN
            $this->updateSort($this->SupplierCode); // SupplierCode
            $this->updateSort($this->Supplier); // Supplier
            $this->updateSort($this->CampCode); // CampCode
            $this->updateSort($this->Camp); // Camp
            $this->updateSort($this->VillageCode); // VillageCode
            $this->updateSort($this->Village); // Village
            $this->updateSort($this->NRC); // NRC
            $this->updateSort($this->FarmerName); // FarmerName
            $this->updateSort($this->DOB); // DOB
            $this->updateSort($this->Age); // Age
            $this->updateSort($this->Group); // Group
            $this->updateSort($this->Gender); // Gender
            $this->updateSort($this->Seedlings); // Seedlings
            $this->updateSort($this->Seeds); // Seeds
            $this->updateSort($this->Type); // Type
            $this->updateSort($this->DateAllocated); // DateAllocated
            $this->updateSort($this->ID); // ID
            $this->setStartRecordNumber(1); // Reset start position
        }

        // Update field sort
        $this->updateFieldSort();
    }

    // Reset command
    // - cmd=reset (Reset search parameters)
    // - cmd=resetall (Reset search and master/detail parameters)
    // - cmd=resetsort (Reset sort parameters)
    protected function resetCmd()
    {
        // Check if reset command
        if (StartsString("reset", $this->Command)) {
            // Reset search criteria
            if ($this->Command == "reset" || $this->Command == "resetall") {
                $this->resetSearchParms();
            }

            // Reset (clear) sorting order
            if ($this->Command == "resetsort") {
                $orderBy = "";
                $this->setSessionOrderBy($orderBy);
                $this->DistrictCode->setSort("");
                $this->DistrictName->setSort("");
                $this->GRNCode->setSort("");
                $this->GRN->setSort("");
                $this->SupplierCode->setSort("");
                $this->Supplier->setSort("");
                $this->CampCode->setSort("");
                $this->Camp->setSort("");
                $this->VillageCode->setSort("");
                $this->Village->setSort("");
                $this->NRC->setSort("");
                $this->FarmerName->setSort("");
                $this->DOB->setSort("");
                $this->Age->setSort("");
                $this->Group->setSort("");
                $this->Gender->setSort("");
                $this->Seedlings->setSort("");
                $this->Seeds->setSort("");
                $this->Type->setSort("");
                $this->DateAllocated->setSort("");
                $this->ID->setSort("");
            }

            // Reset start position
            $this->StartRecord = 1;
            $this->setStartRecordNumber($this->StartRecord);
        }
    }

    // Set up list options
    protected function setupListOptions()
    {
        global $Security, $Language;

        // "griddelete"
        if ($this->AllowAddDeleteRow) {
            $item = &$this->ListOptions->add("griddelete");
            $item->CssClass = "text-nowrap";
            $item->OnLeft = true;
            $item->Visible = false; // Default hidden
        }

        // Add group option item ("button")
        $item = &$this->ListOptions->addGroupOption();
        $item->Body = "";
        $item->OnLeft = true;
        $item->Visible = false;

        // "view"
        $item = &$this->ListOptions->add("view");
        $item->CssClass = "text-nowrap";
        $item->Visible = $Security->canView();
        $item->OnLeft = true;

        // "edit"
        $item = &$this->ListOptions->add("edit");
        $item->CssClass = "text-nowrap";
        $item->Visible = $Security->canEdit();
        $item->OnLeft = true;

        // "copy"
        $item = &$this->ListOptions->add("copy");
        $item->CssClass = "text-nowrap";
        $item->Visible = $Security->canAdd();
        $item->OnLeft = true;

        // "delete"
        $item = &$this->ListOptions->add("delete");
        $item->CssClass = "text-nowrap";
        $item->Visible = $Security->canDelete();
        $item->OnLeft = true;

        // List actions
        $item = &$this->ListOptions->add("listactions");
        $item->CssClass = "text-nowrap";
        $item->OnLeft = true;
        $item->Visible = false;
        $item->ShowInButtonGroup = false;
        $item->ShowInDropDown = false;

        // "checkbox"
        $item = &$this->ListOptions->add("checkbox");
        $item->Visible = false;
        $item->OnLeft = true;
        $item->Header = "<div class=\"form-check\"><input type=\"checkbox\" name=\"key\" id=\"key\" class=\"form-check-input\" data-ew-action=\"select-all-keys\"></div>";
        if ($item->OnLeft) {
            $item->moveTo(0);
        }
        $item->ShowInDropDown = false;
        $item->ShowInButtonGroup = false;

        // Drop down button for ListOptions
        $this->ListOptions->UseDropDownButton = true;
        $this->ListOptions->DropDownButtonPhrase = $Language->phrase("ButtonListOptions");
        $this->ListOptions->UseButtonGroup = true;
        if ($this->ListOptions->UseButtonGroup && IsMobile()) {
            $this->ListOptions->UseDropDownButton = true;
        }

        //$this->ListOptions->ButtonClass = ""; // Class for button group

        // Call ListOptions_Load event
        $this->listOptionsLoad();
        $this->setupListOptionsExt();
        $item = $this->ListOptions[$this->ListOptions->GroupOptionName];
        $item->Visible = $this->ListOptions->groupOptionVisible();
    }

    // Set up list options (extensions)
    protected function setupListOptionsExt()
    {
        // Preview extension
        $this->ListOptions->hideDetailItemsForDropDown(); // Hide detail items for dropdown if necessary
    }

    // Add "hash" parameter to URL
    public function urlAddHash($url, $hash)
    {
        return $this->UseAjaxActions ? $url : UrlAddQuery($url, "hash=" . $hash);
    }

    // Render list options
    public function renderListOptions()
    {
        global $Security, $Language, $CurrentForm;
        $this->ListOptions->loadDefault();

        // Call ListOptions_Rendering event
        $this->listOptionsRendering();

        // Set up row action and key
        if ($CurrentForm && is_numeric($this->RowIndex) && $this->RowType != "view") {
            $CurrentForm->Index = $this->RowIndex;
            $actionName = str_replace("k_", "k" . $this->RowIndex . "_", $this->FormActionName);
            $oldKeyName = str_replace("k_", "k" . $this->RowIndex . "_", $this->OldKeyName);
            $blankRowName = str_replace("k_", "k" . $this->RowIndex . "_", $this->FormBlankRowName);
            if ($this->RowAction != "") {
                $this->MultiSelectKey .= "<input type=\"hidden\" name=\"" . $actionName . "\" id=\"" . $actionName . "\" value=\"" . $this->RowAction . "\">";
            }
            $oldKey = $this->getKey(false); // Get from OldValue
            if ($oldKeyName != "" && $oldKey != "") {
                $this->MultiSelectKey .= "<input type=\"hidden\" name=\"" . $oldKeyName . "\" id=\"" . $oldKeyName . "\" value=\"" . HtmlEncode($oldKey) . "\">";
            }
            if ($this->RowAction == "insert" && $this->isConfirm() && $this->emptyRow()) {
                $this->MultiSelectKey .= "<input type=\"hidden\" name=\"" . $blankRowName . "\" id=\"" . $blankRowName . "\" value=\"1\">";
            }
        }

        // "delete"
        if ($this->AllowAddDeleteRow) {
            if ($this->isGridAdd() || $this->isGridEdit()) {
                $options = &$this->ListOptions;
                $options->UseButtonGroup = true; // Use button group for grid delete button
                $opt = $options["griddelete"];
                if (!$Security->allowDelete(CurrentProjectID() . $this->TableName) && is_numeric($this->RowIndex) && ($this->RowAction == "" || $this->RowAction == "edit")) { // Do not allow delete existing record
                    $opt->Body = "&nbsp;";
                } else {
                    $opt->Body = "<a class=\"ew-grid-link ew-grid-delete\" title=\"" . HtmlTitle($Language->phrase("DeleteLink")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("DeleteLink")) . "\" data-ew-action=\"delete-grid-row\" data-rowindex=\"" . $this->RowIndex . "\">" . $Language->phrase("DeleteLink") . "</a>";
                }
            }
        }
        $pageUrl = $this->pageUrl(false);

        // "edit"
        $opt = $this->ListOptions["edit"];
        if ($this->isInlineEditRow()) { // Inline-Edit
            $this->ListOptions->CustomItem = "edit"; // Show edit column only
            $cancelurl = $this->addMasterUrl($pageUrl . "action=cancel");
                $divClass = $opt->OnLeft ? " class=\"text-end\"" : "";
                $updateCaption = $Language->phrase("UpdateLink");
                $updateTitle = HtmlTitle($updateCaption);
                $cancelCaption = $Language->phrase("CancelLink");
                $cancelTitle = HtmlTitle($cancelCaption);
                $oldKey = HtmlEncode($this->getKey(true));
                $inlineUpdateUrl = HtmlEncode(GetUrl($this->urlAddHash($this->pageName(), "r" . $this->RowCount . "_" . $this->TableVar)));
                if ($this->UseAjaxActions) {
                    $inlineCancelUrl = $this->InlineEditUrl . "?action=cancel";
                    $opt->Body = <<<INLINEEDITAJAX
                    <div{$divClass}>
                        <button type="button" class="ew-grid-link ew-inline-update" title="{$updateTitle}" data-caption="{$updateTitle}" data-ew-action="inline" data-action="update" data-key="{$oldKey}" data-url="{$inlineUpdateUrl}">{$updateCaption}</button>
                        <button type="button" class="ew-grid-link ew-inline-cancel" title="{$cancelTitle}" data-caption="{$cancelTitle}" data-ew-action="inline" data-action="cancel" data-key="{$oldKey}" data-url="{$inlineCancelUrl}">{$cancelCaption}</button>
                    </div>
                    INLINEEDITAJAX;
                } else {
                    $opt->Body = <<<INLINEEDIT
                    <div{$divClass}>
                        <button class="ew-grid-link ew-inline-update" title="{$updateTitle}" data-caption="{$updateTitle}" form="fcidp_datalist" formaction="{$inlineUpdateUrl}">{$updateCaption}</button>
                        <a class="ew-grid-link ew-inline-cancel" title="{$cancelTitle}" data-caption="{$updateTitle}" href="{$cancelurl}">{$cancelCaption}</a>
                        <input type="hidden" name="action" id="action" value="update">
                    </div>
                    INLINEEDIT;
                }
            $opt->Body .= "<input type=\"hidden\" name=\"" . $this->OldKeyName . "\" id=\"" . $this->OldKeyName . "\" value=\"" . HtmlEncode($this->ID->CurrentValue) . "\">";
            return;
        }
        if ($this->CurrentMode == "view") {
            // "view"
            $opt = $this->ListOptions["view"];
            $viewcaption = HtmlTitle($Language->phrase("ViewLink"));
            if ($Security->canView()) {
                if ($this->ModalView && !IsMobile()) {
                    $opt->Body = "<a class=\"ew-row-link ew-view\" title=\"" . $viewcaption . "\" data-table=\"cidp_data\" data-caption=\"" . $viewcaption . "\" data-ew-action=\"modal\" data-action=\"view\" data-ajax=\"" . ($this->UseAjaxActions ? "true" : "false") . "\" data-url=\"" . HtmlEncode(GetUrl($this->ViewUrl)) . "\" data-btn=\"null\">" . $Language->phrase("ViewLink") . "</a>";
                } else {
                    $opt->Body = "<a class=\"ew-row-link ew-view\" title=\"" . $viewcaption . "\" data-caption=\"" . $viewcaption . "\" href=\"" . HtmlEncode(GetUrl($this->ViewUrl)) . "\">" . $Language->phrase("ViewLink") . "</a>";
                }
            } else {
                $opt->Body = "";
            }

            // "edit"
            $opt = $this->ListOptions["edit"];
            $editcaption = HtmlTitle($Language->phrase("EditLink"));
            if ($Security->canEdit()) {
                if ($this->ModalEdit && !IsMobile()) {
                    $opt->Body = "<a class=\"ew-row-link ew-edit\" title=\"" . $editcaption . "\" data-table=\"cidp_data\" data-caption=\"" . $editcaption . "\" data-ew-action=\"modal\" data-action=\"edit\" data-ajax=\"" . ($this->UseAjaxActions ? "true" : "false") . "\" data-url=\"" . HtmlEncode(GetUrl($this->EditUrl)) . "\" data-btn=\"SaveBtn\">" . $Language->phrase("EditLink") . "</a>";
                } else {
                    $opt->Body = "<a class=\"ew-row-link ew-edit\" title=\"" . $editcaption . "\" data-caption=\"" . $editcaption . "\" href=\"" . HtmlEncode(GetUrl($this->EditUrl)) . "\">" . $Language->phrase("EditLink") . "</a>";
                }
                $inlineEditCaption = $Language->phrase("InlineEditLink");
                $inlineEditTitle = HtmlTitle($inlineEditCaption);
                if ($this->UseAjaxActions) {
                    $opt->Body .= "<a class=\"ew-row-link ew-inline-edit\" title=\"" . $inlineEditTitle . "\" data-caption=\"" . $inlineEditTitle . "\" data-ew-action=\"inline\" data-action=\"edit\" data-key=\"" . HtmlEncode($this->getKey(true)) . "\" data-url=\"" . HtmlEncode($this->InlineEditUrl) . "\">" . $inlineEditCaption . "</a>";
                } else {
                    $opt->Body .= "<a class=\"ew-row-link ew-inline-edit\" title=\"" . $inlineEditTitle . "\" data-caption=\"" . $inlineEditTitle . "\" href=\"" . HtmlEncode($this->urlAddHash(GetUrl($this->InlineEditUrl), "r" . $this->RowCount . "_" . $this->TableVar)) . "\">" . $inlineEditCaption . "</a>";
                }
            } else {
                $opt->Body = "";
            }

            // "copy"
            $opt = $this->ListOptions["copy"];
            $copycaption = HtmlTitle($Language->phrase("CopyLink"));
            if ($Security->canAdd()) {
                if ($this->ModalAdd && !IsMobile()) {
                    $opt->Body = "<a class=\"ew-row-link ew-copy\" title=\"" . $copycaption . "\" data-table=\"cidp_data\" data-caption=\"" . $copycaption . "\" data-ew-action=\"modal\" data-action=\"add\" data-ajax=\"" . ($this->UseAjaxActions ? "true" : "false") . "\" data-url=\"" . HtmlEncode(GetUrl($this->CopyUrl)) . "\" data-btn=\"AddBtn\">" . $Language->phrase("CopyLink") . "</a>";
                } else {
                    $opt->Body = "<a class=\"ew-row-link ew-copy\" title=\"" . $copycaption . "\" data-caption=\"" . $copycaption . "\" href=\"" . HtmlEncode(GetUrl($this->CopyUrl)) . "\">" . $Language->phrase("CopyLink") . "</a>";
                }
            } else {
                $opt->Body = "";
            }

            // "delete"
            $opt = $this->ListOptions["delete"];
            if ($Security->canDelete()) {
                $deleteCaption = $Language->phrase("DeleteLink");
                $deleteTitle = HtmlTitle($deleteCaption);
                if ($this->UseAjaxActions) {
                    $opt->Body = "<a class=\"ew-row-link ew-delete\" data-ew-action=\"inline\" data-action=\"delete\" title=\"" . $deleteTitle . "\" data-caption=\"" . $deleteTitle . "\" data-key= \"" . HtmlEncode($this->getKey(true)) . "\" data-url=\"" . HtmlEncode(GetUrl($this->DeleteUrl)) . "\">" . $deleteCaption . "</a>";
                } else {
                    $opt->Body = "<a class=\"ew-row-link ew-delete\"" .
                        ($this->InlineDelete ? " data-ew-action=\"inline-delete\"" : "") .
                        " title=\"" . $deleteTitle . "\" data-caption=\"" . $deleteTitle . "\" href=\"" . HtmlEncode(GetUrl($this->DeleteUrl)) . "\">" . $deleteCaption . "</a>";
                }
            } else {
                $opt->Body = "";
            }
        } // End View mode

        // Set up list action buttons
        $opt = $this->ListOptions["listactions"];
        if ($opt && !$this->isExport() && !$this->CurrentAction) {
            $body = "";
            $links = [];
            foreach ($this->ListActions as $listAction) {
                $action = $listAction->Action;
                $allowed = $listAction->Allowed;
                $disabled = false;
                if ($listAction->Select == ACTION_SINGLE && $allowed) {
                    $caption = $listAction->Caption;
                    $title = HtmlTitle($caption);
                    if ($action != "") {
                        $icon = ($listAction->Icon != "") ? "<i class=\"" . HtmlEncode(str_replace(" ew-icon", "", $listAction->Icon)) . "\" data-caption=\"" . $title . "\"></i> " : "";
                        $link = $disabled
                            ? "<li><div class=\"alert alert-light\">" . $icon . " " . $caption . "</div></li>"
                            : "<li><button type=\"button\" class=\"dropdown-item ew-action ew-list-action\" data-caption=\"" . $title . "\" data-ew-action=\"submit\" form=\"fcidp_datalist\" data-key=\"" . $this->keyToJson(true) . "\"" . $listAction->toDataAttributes() . ">" . $icon . " " . $caption . "</button></li>";
                        $links[] = $link;
                        if ($body == "") { // Setup first button
                            $body = $disabled
                            ? "<div class=\"alert alert-light\">" . $icon . " " . $caption . "</div>"
                            : "<button type=\"button\" class=\"btn btn-default ew-action ew-list-action\" title=\"" . $title . "\" data-caption=\"" . $title . "\" data-ew-action=\"submit\" form=\"fcidp_datalist\" data-key=\"" . $this->keyToJson(true) . "\"" . $listAction->toDataAttributes() . ">" . $icon . " " . $caption . "</button>";
                        }
                    }
                }
            }
            if (count($links) > 1) { // More than one buttons, use dropdown
                $body = "<button type=\"button\" class=\"dropdown-toggle btn btn-default ew-actions\" title=\"" . HtmlTitle($Language->phrase("ListActionButton")) . "\" data-bs-toggle=\"dropdown\">" . $Language->phrase("ListActionButton") . "</button>";
                $content = implode(array_map(fn($link) => "<li>" . $link . "</li>", $links));
                $body .= "<ul class=\"dropdown-menu" . ($opt->OnLeft ? "" : " dropdown-menu-right") . "\">" . $content . "</ul>";
                $body = "<div class=\"btn-group btn-group-sm\">" . $body . "</div>";
            }
            if (count($links) > 0) {
                $opt->Body = $body;
            }
        }

        // "checkbox"
        $opt = $this->ListOptions["checkbox"];
        $opt->Body = "<div class=\"form-check\"><input type=\"checkbox\" id=\"key_m_" . $this->RowCount . "\" name=\"key_m[]\" class=\"form-check-input ew-multi-select\" value=\"" . HtmlEncode($this->ID->CurrentValue) . "\" data-ew-action=\"select-key\"></div>";
        $this->renderListOptionsExt();

        // Call ListOptions_Rendered event
        $this->listOptionsRendered();
    }

    // Render list options (extensions)
    protected function renderListOptionsExt()
    {
        // Render list options (to be implemented by extensions)
        global $Security, $Language;
    }

    // Set up other options
    protected function setupOtherOptions()
    {
        global $Language, $Security;
        $options = &$this->OtherOptions;
        $option = $options["addedit"];

        // Add
        $item = &$option->add("add");
        $addcaption = HtmlTitle($Language->phrase("AddLink"));
        if ($this->ModalAdd && !IsMobile()) {
            $item->Body = "<a class=\"ew-add-edit ew-add\" title=\"" . $addcaption . "\" data-table=\"cidp_data\" data-caption=\"" . $addcaption . "\" data-ew-action=\"modal\" data-action=\"add\" data-ajax=\"" . ($this->UseAjaxActions ? "true" : "false") . "\" data-url=\"" . HtmlEncode(GetUrl($this->AddUrl)) . "\" data-btn=\"AddBtn\">" . $Language->phrase("AddLink") . "</a>";
        } else {
            $item->Body = "<a class=\"ew-add-edit ew-add\" title=\"" . $addcaption . "\" data-caption=\"" . $addcaption . "\" href=\"" . HtmlEncode(GetUrl($this->AddUrl)) . "\">" . $Language->phrase("AddLink") . "</a>";
        }
        $item->Visible = $this->AddUrl != "" && $Security->canAdd();
        $item = &$option->add("gridadd");
        if ($this->ModalGridAdd && !IsMobile()) {
            $item->Body = "<button class=\"ew-add-edit ew-grid-add\" title=\"" . $Language->phrase("GridAddLink", true) . "\" data-caption=\"" . $Language->phrase("GridAddLink", true) . "\" data-ew-action=\"modal\" data-btn=\"AddBtn\" data-url=\"" . HtmlEncode(GetUrl($this->GridAddUrl)) . "\">" . $Language->phrase("GridAddLink") . "</button>";
        } else {
            $item->Body = "<a class=\"ew-add-edit ew-grid-add\" title=\"" . $Language->phrase("GridAddLink", true) . "\" data-caption=\"" . $Language->phrase("GridAddLink", true) . "\" href=\"" . HtmlEncode(GetUrl($this->GridAddUrl)) . "\">" . $Language->phrase("GridAddLink") . "</a>";
        }
        $item->Visible = $this->GridAddUrl != "" && $Security->canAdd();

        // Add grid edit
        $option = $options["addedit"];
        $item = &$option->add("gridedit");
        if ($this->ModalGridEdit && !IsMobile()) {
            $item->Body = "<a class=\"ew-add-edit ew-grid-edit\" title=\"" . $Language->phrase("GridEditLink", true) . "\" data-caption=\"" . $Language->phrase("GridEditLink", true) . "\" data-ew-action=\"modal\" data-btn=\"GridSaveLink\" data-url=\"" . HtmlEncode(GetUrl($this->GridEditUrl)) . "\">" . $Language->phrase("GridEditLink") . "</a>";
        } else {
            $item->Body = "<a class=\"ew-add-edit ew-grid-edit\" title=\"" . $Language->phrase("GridEditLink", true) . "\" data-caption=\"" . $Language->phrase("GridEditLink", true) . "\" href=\"" . HtmlEncode(GetUrl($this->GridEditUrl)) . "\">" . $Language->phrase("GridEditLink") . "</a>";
        }
        $item->Visible = $this->GridEditUrl != "" && $Security->canEdit();
        $option = $options["action"];

        // Show column list for column visibility
        if ($this->UseColumnVisibility) {
            $option = $this->OtherOptions["column"];
            $item = &$option->addGroupOption();
            $item->Body = "";
            $item->Visible = $this->UseColumnVisibility;
            $this->createColumnOption($option, "DistrictCode");
            $this->createColumnOption($option, "DistrictName");
            $this->createColumnOption($option, "GRNCode");
            $this->createColumnOption($option, "GRN");
            $this->createColumnOption($option, "SupplierCode");
            $this->createColumnOption($option, "Supplier");
            $this->createColumnOption($option, "CampCode");
            $this->createColumnOption($option, "Camp");
            $this->createColumnOption($option, "VillageCode");
            $this->createColumnOption($option, "Village");
            $this->createColumnOption($option, "NRC");
            $this->createColumnOption($option, "FarmerName");
            $this->createColumnOption($option, "DOB");
            $this->createColumnOption($option, "Age");
            $this->createColumnOption($option, "Group");
            $this->createColumnOption($option, "Gender");
            $this->createColumnOption($option, "Seedlings");
            $this->createColumnOption($option, "Seeds");
            $this->createColumnOption($option, "Type");
            $this->createColumnOption($option, "DateAllocated");
            $this->createColumnOption($option, "ID");
        }

        // Set up custom actions
        foreach ($this->CustomActions as $name => $action) {
            $this->ListActions[$name] = $action;
        }

        // Set up options default
        foreach ($options as $name => $option) {
            if ($name != "column") { // Always use dropdown for column
                $option->UseDropDownButton = true;
                $option->UseButtonGroup = true;
            }
            //$option->ButtonClass = ""; // Class for button group
            $item = &$option->addGroupOption();
            $item->Body = "";
            $item->Visible = false;
        }
        $options["addedit"]->DropDownButtonPhrase = $Language->phrase("ButtonAddEdit");
        $options["detail"]->DropDownButtonPhrase = $Language->phrase("ButtonDetails");
        $options["action"]->DropDownButtonPhrase = $Language->phrase("ButtonActions");

        // Filter button
        $item = &$this->FilterOptions->add("savecurrentfilter");
        $item->Body = "<a class=\"ew-save-filter\" data-form=\"fcidp_datasrch\" data-ew-action=\"none\">" . $Language->phrase("SaveCurrentFilter") . "</a>";
        $item->Visible = true;
        $item = &$this->FilterOptions->add("deletefilter");
        $item->Body = "<a class=\"ew-delete-filter\" data-form=\"fcidp_datasrch\" data-ew-action=\"none\">" . $Language->phrase("DeleteFilter") . "</a>";
        $item->Visible = true;
        $this->FilterOptions->UseDropDownButton = true;
        $this->FilterOptions->UseButtonGroup = !$this->FilterOptions->UseDropDownButton;
        $this->FilterOptions->DropDownButtonPhrase = $Language->phrase("Filters");

        // Add group option item
        $item = &$this->FilterOptions->addGroupOption();
        $item->Body = "";
        $item->Visible = false;

        // Page header/footer options
        $this->HeaderOptions = new ListOptions(TagClassName: "ew-header-option", UseDropDownButton: false, UseButtonGroup: false);
        $item = &$this->HeaderOptions->addGroupOption();
        $item->Body = "";
        $item->Visible = false;
        $this->FooterOptions = new ListOptions(TagClassName: "ew-footer-option", UseDropDownButton: false, UseButtonGroup: false);
        $item = &$this->FooterOptions->addGroupOption();
        $item->Body = "";
        $item->Visible = false;

        // Show active user count from SQL
    }

    // Active user filter
    // - Get active users by SQL (SELECT COUNT(*) FROM UserTable WHERE ProfileField LIKE '%"SessionID":%')
    protected function activeUserFilter()
    {
        if (UserProfile::$FORCE_LOGOUT_USER) {
            $userProfileField = $this->Fields[Config("USER_PROFILE_FIELD_NAME")];
            return $userProfileField->Expression . " LIKE '%\"" . UserProfile::$SESSION_ID . "\":%'";
        }
        return "0=1"; // No active users
    }

    // Create new column option
    protected function createColumnOption($option, $name)
    {
        $field = $this->Fields[$name] ?? null;
        if ($field?->Visible) {
            $item = $option->add($field->Name);
            $item->Body = '<button class="dropdown-item">' .
                '<div class="form-check ew-dropdown-checkbox">' .
                '<div class="form-check-input ew-dropdown-check-input" data-field="' . $field->Param . '"></div>' .
                '<label class="form-check-label ew-dropdown-check-label">' . $field->caption() . '</label></div></button>';
        }
    }

    // Render other options
    public function renderOtherOptions()
    {
        global $Language, $Security;
        $options = &$this->OtherOptions;
        if (!$this->isGridAdd() && !$this->isGridEdit() && !$this->isMultiEdit()) { // Not grid add/grid edit/multi edit mode
            $option = $options["action"];
            // Set up list action buttons
            foreach ($this->ListActions as $listAction) {
                if ($listAction->Select == ACTION_MULTIPLE) {
                    $item = &$option->add("custom_" . $listAction->Action);
                    $caption = $listAction->Caption;
                    $icon = ($listAction->Icon != "") ? '<i class="' . HtmlEncode($listAction->Icon) . '" data-caption="' . HtmlEncode($caption) . '"></i>' . $caption : $caption;
                    $item->Body = '<button type="button" class="btn btn-default ew-action ew-list-action" title="' . HtmlEncode($caption) . '" data-caption="' . HtmlEncode($caption) . '" data-ew-action="submit" form="fcidp_datalist"' . $listAction->toDataAttributes() . '>' . $icon . '</button>';
                    $item->Visible = $listAction->Allowed;
                }
            }

            // Hide multi edit, grid edit and other options
            if ($this->TotalRecords <= 0) {
                $option = $options["addedit"];
                $item = $option["gridedit"];
                if ($item) {
                    $item->Visible = false;
                }
                $option = $options["action"];
                $option->hideAllOptions();
            }
        } else { // Grid add/grid edit/multi edit mode
            // Hide all options first
            foreach ($options as $option) {
                $option->hideAllOptions();
            }
            $pageUrl = $this->pageUrl(false);

            // Grid-Add
            if ($this->isGridAdd()) {
                    if ($this->AllowAddDeleteRow) {
                        // Add add blank row
                        $option = $options["addedit"];
                        $option->UseDropDownButton = false;
                        $item = &$option->add("addblankrow");
                        $item->Body = "<a type=\"button\" class=\"ew-add-edit ew-add-blank-row\" title=\"" . HtmlTitle($Language->phrase("AddBlankRow")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("AddBlankRow")) . "\" data-ew-action=\"add-grid-row\">" . $Language->phrase("AddBlankRow") . "</a>";
                        $item->Visible = $Security->canAdd();
                    }
                if (!($this->ModalGridAdd && !IsMobile())) {
                    $option = $options["action"];
                    $option->UseDropDownButton = false;
                    // Add grid insert
                    $item = &$option->add("gridinsert");
                    $item->Body = "<button class=\"ew-action ew-grid-insert\" title=\"" . HtmlTitle($Language->phrase("GridInsertLink")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("GridInsertLink")) . "\" form=\"fcidp_datalist\" formaction=\"" . GetUrl($this->pageName()) . "\">" . $Language->phrase("GridInsertLink") . "</button>";
                    // Add grid cancel
                    $item = &$option->add("gridcancel");
                    $cancelurl = $this->addMasterUrl($pageUrl . "action=cancel");
                    $item->Body = "<a type=\"button\" class=\"ew-action ew-grid-cancel\" title=\"" . HtmlTitle($Language->phrase("GridCancelLink")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("GridCancelLink")) . "\" href=\"" . $cancelurl . "\">" . $Language->phrase("GridCancelLink") . "</a>";
                }
            }

            // Grid-Edit
            if ($this->isGridEdit()) {
                    if ($this->AllowAddDeleteRow) {
                        // Add add blank row
                        $option = $options["addedit"];
                        $option->UseDropDownButton = false;
                        $item = &$option->add("addblankrow");
                        $item->Body = "<button class=\"ew-add-edit ew-add-blank-row\" title=\"" . HtmlTitle($Language->phrase("AddBlankRow")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("AddBlankRow")) . "\" data-ew-action=\"add-grid-row\">" . $Language->phrase("AddBlankRow") . "</button>";
                        $item->Visible = $Security->canAdd();
                    }
                if (!($this->ModalGridEdit && !IsMobile())) {
                    $option = $options["action"];
                    $option->UseDropDownButton = false;
                        $item = &$option->add("gridsave");
                        $item->Body = "<button class=\"ew-action ew-grid-save\" title=\"" . HtmlTitle($Language->phrase("GridSaveLink")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("GridSaveLink")) . "\" form=\"fcidp_datalist\" formaction=\"" . GetUrl($this->pageName()) . "\">" . $Language->phrase("GridSaveLink") . "</button>";
                        $item = &$option->add("gridcancel");
                        $cancelurl = $this->addMasterUrl($pageUrl . "action=cancel");
                        $item->Body = "<a class=\"ew-action ew-grid-cancel\" title=\"" . HtmlTitle($Language->phrase("GridCancelLink")) . "\" data-caption=\"" . HtmlTitle($Language->phrase("GridCancelLink")) . "\" href=\"" . $cancelurl . "\">" . $Language->phrase("GridCancelLink") . "</a>";
                }
            }
        }
    }

    // Process list action
    protected function processListAction()
    {
        global $Language, $Security, $Response;
        $users = [];
        $user = "";
        $filter = $this->getFilterFromRecordKeys();
        $userAction = Post("action", "");
        if ($filter != "" && $userAction != "") {
            $conn = $this->getConnection();
            // Clear current action
            $this->CurrentAction = "";
            // Check permission first
            $actionCaption = $userAction;
            $listAction = $this->ListActions[$userAction] ?? null;
            if ($listAction) {
                $this->UserAction = $userAction;
                $actionCaption = $listAction->Caption ?: $listAction->Action;
                if (!$listAction->Allowed) {
                    $errmsg = str_replace('%s', $actionCaption, $Language->phrase("CustomActionNotAllowed"));
                    if (Post("ajax") == $userAction) { // Ajax
                        echo "<p class=\"text-danger\">" . $errmsg . "</p>";
                        return true;
                    } else {
                        $this->setFailureMessage($errmsg);
                        return false;
                    }
                }
            } else {
                $errmsg = str_replace('%s', $userAction, $Language->phrase("CustomActionNotFound"));
                if (Post("ajax") == $userAction) { // Ajax
                    echo "<p class=\"text-danger\">" . $errmsg . "</p>";
                    return true;
                } else {
                    $this->setFailureMessage($errmsg);
                    return false;
                }
            }
            $rows = $this->loadRs($filter)->fetchAllAssociative();
            $this->SelectedCount = count($rows);
            $this->ActionValue = Post("actionvalue");

            // Call row action event
            if ($this->SelectedCount > 0) {
                if ($this->UseTransaction) {
                    $conn->beginTransaction();
                }
                $this->SelectedIndex = 0;
                foreach ($rows as $row) {
                    $this->SelectedIndex++;
                    $processed = $listAction->handle($row, $this);
                    if (!$processed) {
                        break;
                    }
                    $processed = $this->rowCustomAction($userAction, $row);
                    if (!$processed) {
                        break;
                    }
                }
                if ($processed) {
                    if ($this->UseTransaction) { // Commit transaction
                        if ($conn->isTransactionActive()) {
                            $conn->commit();
                        }
                    }
                    if ($this->getSuccessMessage() == "") {
                        $this->setSuccessMessage($listAction->SuccessMessage);
                    }
                    if ($this->getSuccessMessage() == "") {
                        $this->setSuccessMessage(str_replace("%s", $actionCaption, $Language->phrase("CustomActionCompleted"))); // Set up success message
                    }
                } else {
                    if ($this->UseTransaction) { // Rollback transaction
                        if ($conn->isTransactionActive()) {
                            $conn->rollback();
                        }
                    }
                    if ($this->getFailureMessage() == "") {
                        $this->setFailureMessage($listAction->FailureMessage);
                    }

                    // Set up error message
                    if ($this->getSuccessMessage() != "" || $this->getFailureMessage() != "") {
                        // Use the message, do nothing
                    } elseif ($this->CancelMessage != "") {
                        $this->setFailureMessage($this->CancelMessage);
                        $this->CancelMessage = "";
                    } else {
                        $this->setFailureMessage(str_replace('%s', $actionCaption, $Language->phrase("CustomActionFailed")));
                    }
                }
            }
            if (Post("ajax") == $userAction) { // Ajax
                if (WithJsonResponse()) { // List action returns JSON
                    $this->clearSuccessMessage(); // Clear success message
                    $this->clearFailureMessage(); // Clear failure message
                } else {
                    if ($this->getSuccessMessage() != "") {
                        echo "<p class=\"text-success\">" . $this->getSuccessMessage() . "</p>";
                        $this->clearSuccessMessage(); // Clear success message
                    }
                    if ($this->getFailureMessage() != "") {
                        echo "<p class=\"text-danger\">" . $this->getFailureMessage() . "</p>";
                        $this->clearFailureMessage(); // Clear failure message
                    }
                }
                return true;
            }
        }
        return false; // Not ajax request
    }

    // Set up Grid
    public function setupGrid()
    {
        global $CurrentForm;
        if ($this->ExportAll && $this->isExport()) {
            $this->StopRecord = $this->TotalRecords;
        } else {
            // Set the last record to display
            if ($this->TotalRecords > $this->StartRecord + $this->DisplayRecords - 1) {
                $this->StopRecord = $this->StartRecord + $this->DisplayRecords - 1;
            } else {
                $this->StopRecord = $this->TotalRecords;
            }
        }

        // Restore number of post back records
        if ($CurrentForm && ($this->isConfirm() || $this->EventCancelled)) {
            $CurrentForm->resetIndex();
            if ($CurrentForm->hasValue($this->FormKeyCountName) && ($this->isGridAdd() || $this->isGridEdit() || $this->isConfirm())) {
                $this->KeyCount = $CurrentForm->getValue($this->FormKeyCountName);
                $this->StopRecord = $this->StartRecord + $this->KeyCount - 1;
            }
        }
        $this->RecordCount = $this->StartRecord - 1;
        if ($this->CurrentRow !== false) {
            // Nothing to do
        } elseif ($this->isGridAdd() && !$this->AllowAddDeleteRow && $this->StopRecord == 0) { // Grid-Add with no records
            $this->StopRecord = $this->GridAddRowCount;
        } elseif ($this->isAdd() && $this->TotalRecords == 0) { // Inline-Add with no records
            $this->StopRecord = 1;
        }

        // Initialize aggregate
        $this->RowType = RowType::AGGREGATEINIT;
        $this->resetAttributes();
        $this->renderRow();
        if ($this->isEdit()) {
            $this->RowIndex = 1;
        }
        if (($this->isGridAdd() || $this->isGridEdit())) { // Render template row first
            $this->RowIndex = '$rowindex$';
        }
    }

    // Set up Row
    public function setupRow()
    {
        global $CurrentForm;
        if ($this->isGridAdd() || $this->isGridEdit()) {
            if ($this->RowIndex === '$rowindex$') { // Render template row first
                $this->loadRowValues();

                // Set row properties
                $this->resetAttributes();
                $this->RowAttrs->merge(["data-rowindex" => $this->RowIndex, "id" => "r0_cidp_data", "data-rowtype" => RowType::ADD]);
                $this->RowAttrs->appendClass("ew-template");
                // Render row
                $this->RowType = RowType::ADD;
                $this->renderRow();

                // Render list options
                $this->renderListOptions();

                // Reset record count for template row
                $this->RecordCount--;
                return;
            }
        }
        if ($this->isGridAdd() || $this->isGridEdit() || $this->isConfirm() || $this->isMultiEdit()) {
            $this->RowIndex++;
            $CurrentForm->Index = $this->RowIndex;
            if ($CurrentForm->hasValue($this->FormActionName) && ($this->isConfirm() || $this->EventCancelled)) {
                $this->RowAction = strval($CurrentForm->getValue($this->FormActionName));
            } elseif ($this->isGridAdd()) {
                $this->RowAction = "insert";
            } else {
                $this->RowAction = "";
            }
        }

        // Set up key count
        $this->KeyCount = $this->RowIndex;

        // Init row class and style
        $this->resetAttributes();
        $this->CssClass = "";
        if ($this->isCopy() && $this->InlineRowCount == 0 && !$this->loadRow()) { // Inline copy
            $this->CurrentAction = "add";
        }
        if ($this->isAdd() && $this->InlineRowCount == 0 || $this->isGridAdd()) {
            $this->loadRowValues(); // Load default values
            $this->OldKey = "";
            $this->setKey($this->OldKey);
        } elseif ($this->isInlineInserted() && $this->UseInfiniteScroll) {
            // Nothing to do, just use current values
        } elseif (!($this->isCopy() && $this->InlineRowCount == 0)) {
            $this->loadRowValues($this->CurrentRow); // Load row values
            if ($this->isGridEdit() || $this->isMultiEdit()) {
                $this->OldKey = $this->getKey(true); // Get from CurrentValue
                $this->setKey($this->OldKey);
            }
        }
        $this->RowType = RowType::VIEW; // Render view
        if (($this->isAdd() || $this->isCopy()) && $this->InlineRowCount == 0 || $this->isGridAdd()) { // Add
            $this->RowType = RowType::ADD; // Render add
        }
        if ($this->isGridAdd() && $this->EventCancelled && !$CurrentForm->hasValue($this->FormBlankRowName)) { // Insert failed
            $this->restoreCurrentRowFormValues($this->RowIndex); // Restore form values
        }
        if ($this->isEdit() || $this->isInlineUpdated() || $this->isInlineEditCancelled()) { // Inline edit/updated/cancelled
            if ($this->checkInlineEditKey() && $this->InlineRowCount == 0) {
                if ($this->isEdit()) { // Inline edit
                    $this->RowAction = "edit";
                    $this->RowType = RowType::EDIT; // Render edit
                } else { // Inline updated
                    $this->RowAction = "";
                    $this->RowType = RowType::VIEW; // Render view
                    $this->RowAttrs["data-oldkey"] = $this->getKey(); // Set up old key
                }
            } elseif ($this->UseInfiniteScroll) {
                $this->RowAction = "hide";
            }
        }
        if ($this->isGridEdit()) { // Grid edit
            if ($this->EventCancelled) {
                $this->restoreCurrentRowFormValues($this->RowIndex); // Restore form values
            }
            if ($this->RowAction == "insert") {
                $this->RowType = RowType::ADD; // Render add
            } else {
                $this->RowType = RowType::EDIT; // Render edit
            }
        }
        if ($this->isEdit() && $this->RowType == RowType::EDIT && $this->EventCancelled) { // Update failed
            $CurrentForm->Index = 1;
            $this->restoreFormValues(); // Restore form values
        }
        if ($this->isGridEdit() && ($this->RowType == RowType::EDIT || $this->RowType == RowType::ADD) && $this->EventCancelled) { // Update failed
            $this->restoreCurrentRowFormValues($this->RowIndex); // Restore form values
        }

        // Inline Add/Copy row (row 0)
        if ($this->RowType == RowType::ADD && ($this->isAdd() || $this->isCopy())) {
            $this->InlineRowCount++;
            $this->RecordCount--; // Reset record count for inline add/copy row
            if ($this->TotalRecords == 0) { // Reset stop record if no records
                $this->StopRecord = 0;
            }
        } else {
            // Inline Edit row
            if ($this->RowType == RowType::EDIT && $this->isEdit()) {
                $this->InlineRowCount++;
            }
            $this->RowCount++; // Increment row count
        }

        // Set up row attributes
        $this->RowAttrs->merge([
            "data-rowindex" => $this->RowCount,
            "data-key" => $this->getKey(true),
            "id" => "r" . $this->RowCount . "_cidp_data",
            "data-rowtype" => $this->RowType,
            "data-inline" => ($this->isAdd() || $this->isCopy() || $this->isEdit()) ? "true" : "false", // Inline-Add/Copy/Edit
            "class" => ($this->RowCount % 2 != 1) ? "ew-table-alt-row" : "",
        ]);
        if ($this->isAdd() && $this->RowType == RowType::ADD || $this->isEdit() && $this->RowType == RowType::EDIT) { // Inline-Add/Edit row
            $this->RowAttrs->appendClass("table-active");
        }

        // Render row
        $this->renderRow();

        // Render list options
        $this->renderListOptions();
    }

    // Load default values
    protected function loadDefaultValues()
    {
    }

    // Load basic search values
    protected function loadBasicSearchValues()
    {
        $this->BasicSearch->setKeyword(Get(Config("TABLE_BASIC_SEARCH"), ""), false);
        if ($this->BasicSearch->Keyword != "" && $this->Command == "") {
            $this->Command = "search";
        }
        $this->BasicSearch->setType(Get(Config("TABLE_BASIC_SEARCH_TYPE"), ""), false);
    }

    // Load search values for validation
    protected function loadSearchValues()
    {
        // Load search values
        $hasValue = false;

        // Load query builder rules
        $rules = Post("rules");
        if ($rules && $this->Command == "") {
            $this->QueryRules = $rules;
            $this->Command = "search";
        }

        // DistrictCode
        if ($this->DistrictCode->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->DistrictCode->AdvancedSearch->SearchValue != "" || $this->DistrictCode->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // DistrictName
        if ($this->DistrictName->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->DistrictName->AdvancedSearch->SearchValue != "" || $this->DistrictName->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // GRNCode
        if ($this->GRNCode->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->GRNCode->AdvancedSearch->SearchValue != "" || $this->GRNCode->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // GRN
        if ($this->GRN->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->GRN->AdvancedSearch->SearchValue != "" || $this->GRN->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // SupplierCode
        if ($this->SupplierCode->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->SupplierCode->AdvancedSearch->SearchValue != "" || $this->SupplierCode->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Supplier
        if ($this->Supplier->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Supplier->AdvancedSearch->SearchValue != "" || $this->Supplier->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // CampCode
        if ($this->CampCode->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->CampCode->AdvancedSearch->SearchValue != "" || $this->CampCode->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Camp
        if ($this->Camp->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Camp->AdvancedSearch->SearchValue != "" || $this->Camp->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // VillageCode
        if ($this->VillageCode->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->VillageCode->AdvancedSearch->SearchValue != "" || $this->VillageCode->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Village
        if ($this->Village->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Village->AdvancedSearch->SearchValue != "" || $this->Village->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // NRC
        if ($this->NRC->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->NRC->AdvancedSearch->SearchValue != "" || $this->NRC->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // FarmerName
        if ($this->FarmerName->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->FarmerName->AdvancedSearch->SearchValue != "" || $this->FarmerName->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // DOB
        if ($this->DOB->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->DOB->AdvancedSearch->SearchValue != "" || $this->DOB->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Age
        if ($this->Age->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Age->AdvancedSearch->SearchValue != "" || $this->Age->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Group
        if ($this->Group->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Group->AdvancedSearch->SearchValue != "" || $this->Group->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }
        if (is_array($this->Group->AdvancedSearch->SearchValue)) {
            $this->Group->AdvancedSearch->SearchValue = implode(Config("MULTIPLE_OPTION_SEPARATOR"), $this->Group->AdvancedSearch->SearchValue);
        }
        if (is_array($this->Group->AdvancedSearch->SearchValue2)) {
            $this->Group->AdvancedSearch->SearchValue2 = implode(Config("MULTIPLE_OPTION_SEPARATOR"), $this->Group->AdvancedSearch->SearchValue2);
        }

        // Gender
        if ($this->Gender->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Gender->AdvancedSearch->SearchValue != "" || $this->Gender->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Seedlings
        if ($this->Seedlings->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Seedlings->AdvancedSearch->SearchValue != "" || $this->Seedlings->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Seeds
        if ($this->Seeds->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Seeds->AdvancedSearch->SearchValue != "" || $this->Seeds->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // Type
        if ($this->Type->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->Type->AdvancedSearch->SearchValue != "" || $this->Type->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // DateAllocated
        if ($this->DateAllocated->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->DateAllocated->AdvancedSearch->SearchValue != "" || $this->DateAllocated->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }

        // ID
        if ($this->ID->AdvancedSearch->get()) {
            $hasValue = true;
            if (($this->ID->AdvancedSearch->SearchValue != "" || $this->ID->AdvancedSearch->SearchValue2 != "") && $this->Command == "") {
                $this->Command = "search";
            }
        }
        return $hasValue;
    }

    // Load form values
    protected function loadFormValues()
    {
        // Load from form
        global $CurrentForm;
        $validate = !Config("SERVER_VALIDATE");

        // Check field name 'DistrictCode' first before field var 'x_DistrictCode'
        $val = $CurrentForm->hasValue("DistrictCode") ? $CurrentForm->getValue("DistrictCode") : $CurrentForm->getValue("x_DistrictCode");
        if (!$this->DistrictCode->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->DistrictCode->Visible = false; // Disable update for API request
            } else {
                $this->DistrictCode->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_DistrictCode")) {
            $this->DistrictCode->setOldValue($CurrentForm->getValue("o_DistrictCode"));
        }

        // Check field name 'DistrictName' first before field var 'x_DistrictName'
        $val = $CurrentForm->hasValue("DistrictName") ? $CurrentForm->getValue("DistrictName") : $CurrentForm->getValue("x_DistrictName");
        if (!$this->DistrictName->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->DistrictName->Visible = false; // Disable update for API request
            } else {
                $this->DistrictName->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_DistrictName")) {
            $this->DistrictName->setOldValue($CurrentForm->getValue("o_DistrictName"));
        }

        // Check field name 'GRNCode' first before field var 'x_GRNCode'
        $val = $CurrentForm->hasValue("GRNCode") ? $CurrentForm->getValue("GRNCode") : $CurrentForm->getValue("x_GRNCode");
        if (!$this->GRNCode->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->GRNCode->Visible = false; // Disable update for API request
            } else {
                $this->GRNCode->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_GRNCode")) {
            $this->GRNCode->setOldValue($CurrentForm->getValue("o_GRNCode"));
        }

        // Check field name 'GRN' first before field var 'x_GRN'
        $val = $CurrentForm->hasValue("GRN") ? $CurrentForm->getValue("GRN") : $CurrentForm->getValue("x_GRN");
        if (!$this->GRN->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->GRN->Visible = false; // Disable update for API request
            } else {
                $this->GRN->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_GRN")) {
            $this->GRN->setOldValue($CurrentForm->getValue("o_GRN"));
        }

        // Check field name 'SupplierCode' first before field var 'x_SupplierCode'
        $val = $CurrentForm->hasValue("SupplierCode") ? $CurrentForm->getValue("SupplierCode") : $CurrentForm->getValue("x_SupplierCode");
        if (!$this->SupplierCode->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->SupplierCode->Visible = false; // Disable update for API request
            } else {
                $this->SupplierCode->setFormValue($val, true, $validate);
            }
        }
        if ($CurrentForm->hasValue("o_SupplierCode")) {
            $this->SupplierCode->setOldValue($CurrentForm->getValue("o_SupplierCode"));
        }

        // Check field name 'Supplier' first before field var 'x_Supplier'
        $val = $CurrentForm->hasValue("Supplier") ? $CurrentForm->getValue("Supplier") : $CurrentForm->getValue("x_Supplier");
        if (!$this->Supplier->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Supplier->Visible = false; // Disable update for API request
            } else {
                $this->Supplier->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Supplier")) {
            $this->Supplier->setOldValue($CurrentForm->getValue("o_Supplier"));
        }

        // Check field name 'CampCode' first before field var 'x_CampCode'
        $val = $CurrentForm->hasValue("CampCode") ? $CurrentForm->getValue("CampCode") : $CurrentForm->getValue("x_CampCode");
        if (!$this->CampCode->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->CampCode->Visible = false; // Disable update for API request
            } else {
                $this->CampCode->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_CampCode")) {
            $this->CampCode->setOldValue($CurrentForm->getValue("o_CampCode"));
        }

        // Check field name 'Camp' first before field var 'x_Camp'
        $val = $CurrentForm->hasValue("Camp") ? $CurrentForm->getValue("Camp") : $CurrentForm->getValue("x_Camp");
        if (!$this->Camp->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Camp->Visible = false; // Disable update for API request
            } else {
                $this->Camp->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Camp")) {
            $this->Camp->setOldValue($CurrentForm->getValue("o_Camp"));
        }

        // Check field name 'VillageCode' first before field var 'x_VillageCode'
        $val = $CurrentForm->hasValue("VillageCode") ? $CurrentForm->getValue("VillageCode") : $CurrentForm->getValue("x_VillageCode");
        if (!$this->VillageCode->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->VillageCode->Visible = false; // Disable update for API request
            } else {
                $this->VillageCode->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_VillageCode")) {
            $this->VillageCode->setOldValue($CurrentForm->getValue("o_VillageCode"));
        }

        // Check field name 'Village' first before field var 'x_Village'
        $val = $CurrentForm->hasValue("Village") ? $CurrentForm->getValue("Village") : $CurrentForm->getValue("x_Village");
        if (!$this->Village->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Village->Visible = false; // Disable update for API request
            } else {
                $this->Village->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Village")) {
            $this->Village->setOldValue($CurrentForm->getValue("o_Village"));
        }

        // Check field name 'NRC' first before field var 'x_NRC'
        $val = $CurrentForm->hasValue("NRC") ? $CurrentForm->getValue("NRC") : $CurrentForm->getValue("x_NRC");
        if (!$this->NRC->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->NRC->Visible = false; // Disable update for API request
            } else {
                $this->NRC->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_NRC")) {
            $this->NRC->setOldValue($CurrentForm->getValue("o_NRC"));
        }

        // Check field name 'FarmerName' first before field var 'x_FarmerName'
        $val = $CurrentForm->hasValue("FarmerName") ? $CurrentForm->getValue("FarmerName") : $CurrentForm->getValue("x_FarmerName");
        if (!$this->FarmerName->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->FarmerName->Visible = false; // Disable update for API request
            } else {
                $this->FarmerName->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_FarmerName")) {
            $this->FarmerName->setOldValue($CurrentForm->getValue("o_FarmerName"));
        }

        // Check field name 'DOB' first before field var 'x_DOB'
        $val = $CurrentForm->hasValue("DOB") ? $CurrentForm->getValue("DOB") : $CurrentForm->getValue("x_DOB");
        if (!$this->DOB->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->DOB->Visible = false; // Disable update for API request
            } else {
                $this->DOB->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_DOB")) {
            $this->DOB->setOldValue($CurrentForm->getValue("o_DOB"));
        }

        // Check field name 'Age' first before field var 'x_Age'
        $val = $CurrentForm->hasValue("Age") ? $CurrentForm->getValue("Age") : $CurrentForm->getValue("x_Age");
        if (!$this->Age->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Age->Visible = false; // Disable update for API request
            } else {
                $this->Age->setFormValue($val, true, $validate);
            }
        }
        if ($CurrentForm->hasValue("o_Age")) {
            $this->Age->setOldValue($CurrentForm->getValue("o_Age"));
        }

        // Check field name 'Group' first before field var 'x_Group'
        $val = $CurrentForm->hasValue("Group") ? $CurrentForm->getValue("Group") : $CurrentForm->getValue("x_Group");
        if (!$this->Group->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Group->Visible = false; // Disable update for API request
            } else {
                $this->Group->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Group")) {
            $this->Group->setOldValue($CurrentForm->getValue("o_Group"));
        }

        // Check field name 'Gender' first before field var 'x_Gender'
        $val = $CurrentForm->hasValue("Gender") ? $CurrentForm->getValue("Gender") : $CurrentForm->getValue("x_Gender");
        if (!$this->Gender->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Gender->Visible = false; // Disable update for API request
            } else {
                $this->Gender->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Gender")) {
            $this->Gender->setOldValue($CurrentForm->getValue("o_Gender"));
        }

        // Check field name 'Seedlings' first before field var 'x_Seedlings'
        $val = $CurrentForm->hasValue("Seedlings") ? $CurrentForm->getValue("Seedlings") : $CurrentForm->getValue("x_Seedlings");
        if (!$this->Seedlings->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Seedlings->Visible = false; // Disable update for API request
            } else {
                $this->Seedlings->setFormValue($val, true, $validate);
            }
        }
        if ($CurrentForm->hasValue("o_Seedlings")) {
            $this->Seedlings->setOldValue($CurrentForm->getValue("o_Seedlings"));
        }

        // Check field name 'Seeds' first before field var 'x_Seeds'
        $val = $CurrentForm->hasValue("Seeds") ? $CurrentForm->getValue("Seeds") : $CurrentForm->getValue("x_Seeds");
        if (!$this->Seeds->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Seeds->Visible = false; // Disable update for API request
            } else {
                $this->Seeds->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Seeds")) {
            $this->Seeds->setOldValue($CurrentForm->getValue("o_Seeds"));
        }

        // Check field name 'Type' first before field var 'x_Type'
        $val = $CurrentForm->hasValue("Type") ? $CurrentForm->getValue("Type") : $CurrentForm->getValue("x_Type");
        if (!$this->Type->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->Type->Visible = false; // Disable update for API request
            } else {
                $this->Type->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_Type")) {
            $this->Type->setOldValue($CurrentForm->getValue("o_Type"));
        }

        // Check field name 'DateAllocated' first before field var 'x_DateAllocated'
        $val = $CurrentForm->hasValue("DateAllocated") ? $CurrentForm->getValue("DateAllocated") : $CurrentForm->getValue("x_DateAllocated");
        if (!$this->DateAllocated->IsDetailKey) {
            if (IsApi() && $val === null) {
                $this->DateAllocated->Visible = false; // Disable update for API request
            } else {
                $this->DateAllocated->setFormValue($val);
            }
        }
        if ($CurrentForm->hasValue("o_DateAllocated")) {
            $this->DateAllocated->setOldValue($CurrentForm->getValue("o_DateAllocated"));
        }

        // Check field name 'ID' first before field var 'x_ID'
        $val = $CurrentForm->hasValue("ID") ? $CurrentForm->getValue("ID") : $CurrentForm->getValue("x_ID");
        if (!$this->ID->IsDetailKey && !$this->isGridAdd() && !$this->isAdd()) {
            $this->ID->setFormValue($val);
        }
    }

    // Restore form values
    public function restoreFormValues()
    {
        global $CurrentForm;
        $this->DistrictCode->CurrentValue = $this->DistrictCode->FormValue;
        $this->DistrictName->CurrentValue = $this->DistrictName->FormValue;
        $this->GRNCode->CurrentValue = $this->GRNCode->FormValue;
        $this->GRN->CurrentValue = $this->GRN->FormValue;
        $this->SupplierCode->CurrentValue = $this->SupplierCode->FormValue;
        $this->Supplier->CurrentValue = $this->Supplier->FormValue;
        $this->CampCode->CurrentValue = $this->CampCode->FormValue;
        $this->Camp->CurrentValue = $this->Camp->FormValue;
        $this->VillageCode->CurrentValue = $this->VillageCode->FormValue;
        $this->Village->CurrentValue = $this->Village->FormValue;
        $this->NRC->CurrentValue = $this->NRC->FormValue;
        $this->FarmerName->CurrentValue = $this->FarmerName->FormValue;
        $this->DOB->CurrentValue = $this->DOB->FormValue;
        $this->Age->CurrentValue = $this->Age->FormValue;
        $this->Group->CurrentValue = $this->Group->FormValue;
        $this->Gender->CurrentValue = $this->Gender->FormValue;
        $this->Seedlings->CurrentValue = $this->Seedlings->FormValue;
        $this->Seeds->CurrentValue = $this->Seeds->FormValue;
        $this->Type->CurrentValue = $this->Type->FormValue;
        $this->DateAllocated->CurrentValue = $this->DateAllocated->FormValue;
        if (!$this->isGridAdd() && !$this->isAdd()) {
            $this->ID->CurrentValue = $this->ID->FormValue;
        }
    }

    /**
     * Load result set
     *
     * @param int $offset Offset
     * @param int $rowcnt Maximum number of rows
     * @return Doctrine\DBAL\Result Result
     */
    public function loadRecordset($offset = -1, $rowcnt = -1)
    {
        // Load List page SQL (QueryBuilder)
        $sql = $this->getListSql();

        // Load result set
        if ($offset > -1) {
            $sql->setFirstResult($offset);
        }
        if ($rowcnt > 0) {
            $sql->setMaxResults($rowcnt);
        }
        $result = $sql->executeQuery();
        if (property_exists($this, "TotalRecords") && $rowcnt < 0) {
            $this->TotalRecords = $result->rowCount();
            if ($this->TotalRecords <= 0) { // Handle database drivers that does not return rowCount()
                $this->TotalRecords = $this->getRecordCount($this->getListSql());
            }
        }

        // Call Recordset Selected event
        $this->recordsetSelected($result);
        return $result;
    }

    /**
     * Load records as associative array
     *
     * @param int $offset Offset
     * @param int $rowcnt Maximum number of rows
     * @return void
     */
    public function loadRows($offset = -1, $rowcnt = -1)
    {
        // Load List page SQL (QueryBuilder)
        $sql = $this->getListSql();

        // Load result set
        if ($offset > -1) {
            $sql->setFirstResult($offset);
        }
        if ($rowcnt > 0) {
            $sql->setMaxResults($rowcnt);
        }
        $result = $sql->executeQuery();
        return $result->fetchAllAssociative();
    }

    /**
     * Load row based on key values
     *
     * @return void
     */
    public function loadRow()
    {
        global $Security, $Language;
        $filter = $this->getRecordFilter();

        // Call Row Selecting event
        $this->rowSelecting($filter);

        // Load SQL based on filter
        $this->CurrentFilter = $filter;
        $sql = $this->getCurrentSql();
        $conn = $this->getConnection();
        $res = false;
        $row = $conn->fetchAssociative($sql);
        if ($row) {
            $res = true;
            $this->loadRowValues($row); // Load row values
            if (!$this->EventCancelled) {
                $this->HashValue = $this->getRowHash($row); // Get hash value for record
            }
        }
        return $res;
    }

    /**
     * Load row values from result set or record
     *
     * @param array $row Record
     * @return void
     */
    public function loadRowValues($row = null)
    {
        $row = is_array($row) ? $row : $this->newRow();

        // Call Row Selected event
        $this->rowSelected($row);
        $this->DistrictCode->setDbValue($row['DistrictCode']);
        $this->DistrictName->setDbValue($row['DistrictName']);
        $this->GRNCode->setDbValue($row['GRNCode']);
        $this->GRN->setDbValue($row['GRN']);
        $this->SupplierCode->setDbValue($row['SupplierCode']);
        $this->Supplier->setDbValue($row['Supplier']);
        $this->CampCode->setDbValue($row['CampCode']);
        $this->Camp->setDbValue($row['Camp']);
        $this->VillageCode->setDbValue($row['VillageCode']);
        $this->Village->setDbValue($row['Village']);
        $this->NRC->setDbValue($row['NRC']);
        $this->FarmerName->setDbValue($row['FarmerName']);
        $this->DOB->setDbValue($row['DOB']);
        $this->Age->setDbValue($row['Age']);
        $this->Group->setDbValue($row['Group']);
        $this->Gender->setDbValue($row['Gender']);
        $this->Seedlings->setDbValue($row['Seedlings']);
        $this->Seeds->setDbValue($row['Seeds']);
        $this->Type->setDbValue($row['Type']);
        $this->DateAllocated->setDbValue($row['DateAllocated']);
        $this->ID->setDbValue($row['ID']);
    }

    // Return a row with default values
    protected function newRow()
    {
        $row = [];
        $row['DistrictCode'] = $this->DistrictCode->DefaultValue;
        $row['DistrictName'] = $this->DistrictName->DefaultValue;
        $row['GRNCode'] = $this->GRNCode->DefaultValue;
        $row['GRN'] = $this->GRN->DefaultValue;
        $row['SupplierCode'] = $this->SupplierCode->DefaultValue;
        $row['Supplier'] = $this->Supplier->DefaultValue;
        $row['CampCode'] = $this->CampCode->DefaultValue;
        $row['Camp'] = $this->Camp->DefaultValue;
        $row['VillageCode'] = $this->VillageCode->DefaultValue;
        $row['Village'] = $this->Village->DefaultValue;
        $row['NRC'] = $this->NRC->DefaultValue;
        $row['FarmerName'] = $this->FarmerName->DefaultValue;
        $row['DOB'] = $this->DOB->DefaultValue;
        $row['Age'] = $this->Age->DefaultValue;
        $row['Group'] = $this->Group->DefaultValue;
        $row['Gender'] = $this->Gender->DefaultValue;
        $row['Seedlings'] = $this->Seedlings->DefaultValue;
        $row['Seeds'] = $this->Seeds->DefaultValue;
        $row['Type'] = $this->Type->DefaultValue;
        $row['DateAllocated'] = $this->DateAllocated->DefaultValue;
        $row['ID'] = $this->ID->DefaultValue;
        return $row;
    }

    // Load old record
    protected function loadOldRecord()
    {
        // Load old record
        if ($this->OldKey != "") {
            $this->setKey($this->OldKey);
            $this->CurrentFilter = $this->getRecordFilter();
            $sql = $this->getCurrentSql();
            $conn = $this->getConnection();
            $rs = ExecuteQuery($sql, $conn);
            if ($row = $rs->fetch()) {
                $this->loadRowValues($row); // Load row values
                return $row;
            }
        }
        $this->loadRowValues(); // Load default row values
        return null;
    }

    // Render row values based on field settings
    public function renderRow()
    {
        global $Security, $Language, $CurrentLanguage;

        // Initialize URLs
        $this->ViewUrl = $this->getViewUrl();
        $this->EditUrl = $this->getEditUrl();
        $this->InlineEditUrl = $this->getInlineEditUrl();
        $this->CopyUrl = $this->getCopyUrl();
        $this->InlineCopyUrl = $this->getInlineCopyUrl();
        $this->DeleteUrl = $this->getDeleteUrl();

        // Call Row_Rendering event
        $this->rowRendering();

        // Common render codes for all row types

        // DistrictCode

        // DistrictName

        // GRNCode

        // GRN

        // SupplierCode

        // Supplier

        // CampCode

        // Camp

        // VillageCode

        // Village

        // NRC

        // FarmerName

        // DOB

        // Age

        // Group

        // Gender

        // Seedlings

        // Seeds

        // Type

        // DateAllocated

        // ID

        // View row
        if ($this->RowType == RowType::VIEW) {
            // DistrictCode
            $curVal = strval($this->DistrictCode->CurrentValue);
            if ($curVal != "") {
                $this->DistrictCode->ViewValue = $this->DistrictCode->lookupCacheOption($curVal);
                if ($this->DistrictCode->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->DistrictCode->Lookup->getTable()->Fields["DistrictCode"]->searchExpression(), "=", $curVal, $this->DistrictCode->Lookup->getTable()->Fields["DistrictCode"]->searchDataType(), "");
                    $sqlWrk = $this->DistrictCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->DistrictCode->Lookup->renderViewRow($rswrk[0]);
                        $this->DistrictCode->ViewValue = $this->DistrictCode->displayValue($arwrk);
                    } else {
                        $this->DistrictCode->ViewValue = $this->DistrictCode->CurrentValue;
                    }
                }
            } else {
                $this->DistrictCode->ViewValue = null;
            }

            // DistrictName
            $this->DistrictName->ViewValue = $this->DistrictName->CurrentValue;

            // GRNCode
            $this->GRNCode->ViewValue = $this->GRNCode->CurrentValue;
            $curVal = strval($this->GRNCode->CurrentValue);
            if ($curVal != "") {
                $this->GRNCode->ViewValue = $this->GRNCode->lookupCacheOption($curVal);
                if ($this->GRNCode->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->GRNCode->Lookup->getTable()->Fields["GRN"]->searchExpression(), "=", $curVal, $this->GRNCode->Lookup->getTable()->Fields["GRN"]->searchDataType(), "");
                    $sqlWrk = $this->GRNCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->GRNCode->Lookup->renderViewRow($rswrk[0]);
                        $this->GRNCode->ViewValue = $this->GRNCode->displayValue($arwrk);
                    } else {
                        $this->GRNCode->ViewValue = $this->GRNCode->CurrentValue;
                    }
                }
            } else {
                $this->GRNCode->ViewValue = null;
            }

            // GRN
            $this->GRN->ViewValue = $this->GRN->CurrentValue;

            // SupplierCode
            $this->SupplierCode->ViewValue = $this->SupplierCode->CurrentValue;
            $curVal = strval($this->SupplierCode->CurrentValue);
            if ($curVal != "") {
                $this->SupplierCode->ViewValue = $this->SupplierCode->lookupCacheOption($curVal);
                if ($this->SupplierCode->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->SupplierCode->Lookup->getTable()->Fields["NurseryCode"]->searchExpression(), "=", $curVal, $this->SupplierCode->Lookup->getTable()->Fields["NurseryCode"]->searchDataType(), "");
                    $sqlWrk = $this->SupplierCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->SupplierCode->Lookup->renderViewRow($rswrk[0]);
                        $this->SupplierCode->ViewValue = $this->SupplierCode->displayValue($arwrk);
                    } else {
                        $this->SupplierCode->ViewValue = $this->SupplierCode->CurrentValue;
                    }
                }
            } else {
                $this->SupplierCode->ViewValue = null;
            }

            // Supplier
            $this->Supplier->ViewValue = $this->Supplier->CurrentValue;

            // CampCode
            $curVal = strval($this->CampCode->CurrentValue);
            if ($curVal != "") {
                $this->CampCode->ViewValue = $this->CampCode->lookupCacheOption($curVal);
                if ($this->CampCode->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->CampCode->Lookup->getTable()->Fields["CampCode"]->searchExpression(), "=", $curVal, $this->CampCode->Lookup->getTable()->Fields["CampCode"]->searchDataType(), "");
                    $sqlWrk = $this->CampCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->CampCode->Lookup->renderViewRow($rswrk[0]);
                        $this->CampCode->ViewValue = $this->CampCode->displayValue($arwrk);
                    } else {
                        $this->CampCode->ViewValue = $this->CampCode->CurrentValue;
                    }
                }
            } else {
                $this->CampCode->ViewValue = null;
            }

            // Camp
            $this->Camp->ViewValue = $this->Camp->CurrentValue;

            // VillageCode
            $curVal = strval($this->VillageCode->CurrentValue);
            if ($curVal != "") {
                $this->VillageCode->ViewValue = $this->VillageCode->lookupCacheOption($curVal);
                if ($this->VillageCode->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->VillageCode->Lookup->getTable()->Fields["VillageCode"]->searchExpression(), "=", $curVal, $this->VillageCode->Lookup->getTable()->Fields["VillageCode"]->searchDataType(), "");
                    $sqlWrk = $this->VillageCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->VillageCode->Lookup->renderViewRow($rswrk[0]);
                        $this->VillageCode->ViewValue = $this->VillageCode->displayValue($arwrk);
                    } else {
                        $this->VillageCode->ViewValue = $this->VillageCode->CurrentValue;
                    }
                }
            } else {
                $this->VillageCode->ViewValue = null;
            }

            // Village
            $this->Village->ViewValue = $this->Village->CurrentValue;

            // NRC
            $this->NRC->ViewValue = $this->NRC->CurrentValue;

            // FarmerName
            $this->FarmerName->ViewValue = $this->FarmerName->CurrentValue;

            // DOB
            $this->DOB->ViewValue = $this->DOB->CurrentValue;

            // Age
            $this->Age->ViewValue = $this->Age->CurrentValue;
            $this->Age->ViewValue = FormatNumber($this->Age->ViewValue, $this->Age->formatPattern());

            // Group
            if (strval($this->Group->CurrentValue) != "") {
                $this->Group->ViewValue = new OptionValues();
                $arwrk = explode(Config("MULTIPLE_OPTION_SEPARATOR"), strval($this->Group->CurrentValue));
                $cnt = count($arwrk);
                for ($ari = 0; $ari < $cnt; $ari++)
                    $this->Group->ViewValue->add($this->Group->optionCaption(trim($arwrk[$ari])));
            } else {
                $this->Group->ViewValue = null;
            }

            // Gender
            $curVal = strval($this->Gender->CurrentValue);
            if ($curVal != "") {
                $this->Gender->ViewValue = $this->Gender->lookupCacheOption($curVal);
                if ($this->Gender->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->Gender->Lookup->getTable()->Fields["Gender"]->searchExpression(), "=", $curVal, $this->Gender->Lookup->getTable()->Fields["Gender"]->searchDataType(), "");
                    $sqlWrk = $this->Gender->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->Gender->Lookup->renderViewRow($rswrk[0]);
                        $this->Gender->ViewValue = $this->Gender->displayValue($arwrk);
                    } else {
                        $this->Gender->ViewValue = $this->Gender->CurrentValue;
                    }
                }
            } else {
                $this->Gender->ViewValue = null;
            }

            // Seedlings
            $this->Seedlings->ViewValue = $this->Seedlings->CurrentValue;
            $this->Seedlings->ViewValue = FormatNumber($this->Seedlings->ViewValue, $this->Seedlings->formatPattern());
            $this->Seedlings->CellCssStyle .= "text-align: right;";

            // Seeds
            $this->Seeds->ViewValue = $this->Seeds->CurrentValue;
            $this->Seeds->ViewValue = FormatNumber($this->Seeds->ViewValue, $this->Seeds->formatPattern());
            $this->Seeds->CellCssStyle .= "text-align: right;";

            // Type
            $curVal = strval($this->Type->CurrentValue);
            if ($curVal != "") {
                $this->Type->ViewValue = $this->Type->lookupCacheOption($curVal);
                if ($this->Type->ViewValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->Type->Lookup->getTable()->Fields["SeedVariety"]->searchExpression(), "=", $curVal, $this->Type->Lookup->getTable()->Fields["SeedVariety"]->searchDataType(), "");
                    $sqlWrk = $this->Type->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->Type->Lookup->renderViewRow($rswrk[0]);
                        $this->Type->ViewValue = $this->Type->displayValue($arwrk);
                    } else {
                        $this->Type->ViewValue = $this->Type->CurrentValue;
                    }
                }
            } else {
                $this->Type->ViewValue = null;
            }

            // DateAllocated
            $this->DateAllocated->ViewValue = $this->DateAllocated->CurrentValue;

            // ID
            $this->ID->ViewValue = $this->ID->CurrentValue;

            // DistrictCode
            $this->DistrictCode->HrefValue = "";
            $this->DistrictCode->TooltipValue = "";

            // DistrictName
            $this->DistrictName->HrefValue = "";
            $this->DistrictName->TooltipValue = "";
            if (!$this->isExport()) {
                $this->DistrictName->ViewValue = $this->highlightValue($this->DistrictName);
            }

            // GRNCode
            $this->GRNCode->HrefValue = "";
            $this->GRNCode->TooltipValue = "";
            if (!$this->isExport()) {
                $this->GRNCode->ViewValue = $this->highlightValue($this->GRNCode);
            }

            // GRN
            $this->GRN->HrefValue = "";
            $this->GRN->TooltipValue = "";
            if (!$this->isExport()) {
                $this->GRN->ViewValue = $this->highlightValue($this->GRN);
            }

            // SupplierCode
            $this->SupplierCode->HrefValue = "";
            $this->SupplierCode->TooltipValue = "";
            if (!$this->isExport()) {
                $this->SupplierCode->ViewValue = $this->highlightValue($this->SupplierCode);
            }

            // Supplier
            $this->Supplier->HrefValue = "";
            $this->Supplier->TooltipValue = "";
            if (!$this->isExport()) {
                $this->Supplier->ViewValue = $this->highlightValue($this->Supplier);
            }

            // CampCode
            $this->CampCode->HrefValue = "";
            $this->CampCode->TooltipValue = "";

            // Camp
            $this->Camp->HrefValue = "";
            $this->Camp->TooltipValue = "";
            if (!$this->isExport()) {
                $this->Camp->ViewValue = $this->highlightValue($this->Camp);
            }

            // VillageCode
            $this->VillageCode->HrefValue = "";
            $this->VillageCode->TooltipValue = "";

            // Village
            $this->Village->HrefValue = "";
            $this->Village->TooltipValue = "";
            if (!$this->isExport()) {
                $this->Village->ViewValue = $this->highlightValue($this->Village);
            }

            // NRC
            $this->NRC->HrefValue = "";
            $this->NRC->TooltipValue = "";
            if (!$this->isExport()) {
                $this->NRC->ViewValue = $this->highlightValue($this->NRC);
            }

            // FarmerName
            $this->FarmerName->HrefValue = "";
            $this->FarmerName->TooltipValue = "";
            if (!$this->isExport()) {
                $this->FarmerName->ViewValue = $this->highlightValue($this->FarmerName);
            }

            // DOB
            $this->DOB->HrefValue = "";
            $this->DOB->TooltipValue = "";
            if (!$this->isExport()) {
                $this->DOB->ViewValue = $this->highlightValue($this->DOB);
            }

            // Age
            $this->Age->HrefValue = "";
            $this->Age->TooltipValue = "";
            if (!$this->isExport()) {
                $this->Age->ViewValue = $this->highlightValue($this->Age);
            }

            // Group
            $this->Group->HrefValue = "";
            $this->Group->TooltipValue = "";

            // Gender
            $this->Gender->HrefValue = "";
            $this->Gender->TooltipValue = "";

            // Seedlings
            $this->Seedlings->HrefValue = "";
            $this->Seedlings->TooltipValue = "";

            // Seeds
            $this->Seeds->HrefValue = "";
            $this->Seeds->TooltipValue = "";

            // Type
            $this->Type->HrefValue = "";
            $this->Type->TooltipValue = "";

            // DateAllocated
            $this->DateAllocated->HrefValue = "";
            $this->DateAllocated->TooltipValue = "";
            if (!$this->isExport()) {
                $this->DateAllocated->ViewValue = $this->highlightValue($this->DateAllocated);
            }

            // ID
            $this->ID->HrefValue = "";
            $this->ID->TooltipValue = "";
            if (!$this->isExport()) {
                $this->ID->ViewValue = $this->highlightValue($this->ID);
            }
        } elseif ($this->RowType == RowType::ADD) {
            // DistrictCode
            $curVal = trim(strval($this->DistrictCode->CurrentValue));
            if ($curVal != "") {
                $this->DistrictCode->ViewValue = $this->DistrictCode->lookupCacheOption($curVal);
            } else {
                $this->DistrictCode->ViewValue = $this->DistrictCode->Lookup !== null && is_array($this->DistrictCode->lookupOptions()) && count($this->DistrictCode->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->DistrictCode->ViewValue !== null) { // Load from cache
                $this->DistrictCode->EditValue = array_values($this->DistrictCode->lookupOptions());
                if ($this->DistrictCode->ViewValue == "") {
                    $this->DistrictCode->ViewValue = $Language->phrase("PleaseSelect");
                }
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->DistrictCode->Lookup->getTable()->Fields["DistrictCode"]->searchExpression(), "=", $this->DistrictCode->CurrentValue, $this->DistrictCode->Lookup->getTable()->Fields["DistrictCode"]->searchDataType(), "");
                }
                $sqlWrk = $this->DistrictCode->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                if ($ari > 0) { // Lookup values found
                    $arwrk = $this->DistrictCode->Lookup->renderViewRow($rswrk[0]);
                    $this->DistrictCode->ViewValue = $this->DistrictCode->displayValue($arwrk);
                } else {
                    $this->DistrictCode->ViewValue = $Language->phrase("PleaseSelect");
                }
                $arwrk = $rswrk;
                $this->DistrictCode->EditValue = $arwrk;
            }
            $this->DistrictCode->PlaceHolder = RemoveHtml($this->DistrictCode->caption());

            // DistrictName
            $this->DistrictName->setupEditAttributes();
            if (!$this->DistrictName->Raw) {
                $this->DistrictName->CurrentValue = HtmlDecode($this->DistrictName->CurrentValue);
            }
            $this->DistrictName->EditValue = HtmlEncode($this->DistrictName->CurrentValue);
            $this->DistrictName->PlaceHolder = RemoveHtml($this->DistrictName->caption());

            // GRNCode
            $this->GRNCode->setupEditAttributes();
            if (!$this->GRNCode->Raw) {
                $this->GRNCode->CurrentValue = HtmlDecode($this->GRNCode->CurrentValue);
            }
            $this->GRNCode->EditValue = HtmlEncode($this->GRNCode->CurrentValue);
            $curVal = strval($this->GRNCode->CurrentValue);
            if ($curVal != "") {
                $this->GRNCode->EditValue = $this->GRNCode->lookupCacheOption($curVal);
                if ($this->GRNCode->EditValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->GRNCode->Lookup->getTable()->Fields["GRN"]->searchExpression(), "=", $curVal, $this->GRNCode->Lookup->getTable()->Fields["GRN"]->searchDataType(), "");
                    $sqlWrk = $this->GRNCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->GRNCode->Lookup->renderViewRow($rswrk[0]);
                        $this->GRNCode->EditValue = $this->GRNCode->displayValue($arwrk);
                    } else {
                        $this->GRNCode->EditValue = HtmlEncode($this->GRNCode->CurrentValue);
                    }
                }
            } else {
                $this->GRNCode->EditValue = null;
            }
            $this->GRNCode->PlaceHolder = RemoveHtml($this->GRNCode->caption());

            // GRN
            $this->GRN->setupEditAttributes();
            if (!$this->GRN->Raw) {
                $this->GRN->CurrentValue = HtmlDecode($this->GRN->CurrentValue);
            }
            $this->GRN->EditValue = HtmlEncode($this->GRN->CurrentValue);
            $this->GRN->PlaceHolder = RemoveHtml($this->GRN->caption());

            // SupplierCode
            $this->SupplierCode->setupEditAttributes();
            $this->SupplierCode->EditValue = $this->SupplierCode->CurrentValue;
            $curVal = strval($this->SupplierCode->CurrentValue);
            if ($curVal != "") {
                $this->SupplierCode->EditValue = $this->SupplierCode->lookupCacheOption($curVal);
                if ($this->SupplierCode->EditValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->SupplierCode->Lookup->getTable()->Fields["NurseryCode"]->searchExpression(), "=", $curVal, $this->SupplierCode->Lookup->getTable()->Fields["NurseryCode"]->searchDataType(), "");
                    $sqlWrk = $this->SupplierCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->SupplierCode->Lookup->renderViewRow($rswrk[0]);
                        $this->SupplierCode->EditValue = $this->SupplierCode->displayValue($arwrk);
                    } else {
                        $this->SupplierCode->EditValue = HtmlEncode($this->SupplierCode->CurrentValue);
                    }
                }
            } else {
                $this->SupplierCode->EditValue = null;
            }
            $this->SupplierCode->PlaceHolder = RemoveHtml($this->SupplierCode->caption());

            // Supplier
            $this->Supplier->setupEditAttributes();
            if (!$this->Supplier->Raw) {
                $this->Supplier->CurrentValue = HtmlDecode($this->Supplier->CurrentValue);
            }
            $this->Supplier->EditValue = HtmlEncode($this->Supplier->CurrentValue);
            $this->Supplier->PlaceHolder = RemoveHtml($this->Supplier->caption());

            // CampCode
            $curVal = trim(strval($this->CampCode->CurrentValue));
            if ($curVal != "") {
                $this->CampCode->ViewValue = $this->CampCode->lookupCacheOption($curVal);
            } else {
                $this->CampCode->ViewValue = $this->CampCode->Lookup !== null && is_array($this->CampCode->lookupOptions()) && count($this->CampCode->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->CampCode->ViewValue !== null) { // Load from cache
                $this->CampCode->EditValue = array_values($this->CampCode->lookupOptions());
                if ($this->CampCode->ViewValue == "") {
                    $this->CampCode->ViewValue = $Language->phrase("PleaseSelect");
                }
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->CampCode->Lookup->getTable()->Fields["CampCode"]->searchExpression(), "=", $this->CampCode->CurrentValue, $this->CampCode->Lookup->getTable()->Fields["CampCode"]->searchDataType(), "");
                }
                $sqlWrk = $this->CampCode->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                if ($ari > 0) { // Lookup values found
                    $arwrk = $this->CampCode->Lookup->renderViewRow($rswrk[0]);
                    $this->CampCode->ViewValue = $this->CampCode->displayValue($arwrk);
                } else {
                    $this->CampCode->ViewValue = $Language->phrase("PleaseSelect");
                }
                $arwrk = $rswrk;
                $this->CampCode->EditValue = $arwrk;
            }
            $this->CampCode->PlaceHolder = RemoveHtml($this->CampCode->caption());

            // Camp
            $this->Camp->setupEditAttributes();
            if (!$this->Camp->Raw) {
                $this->Camp->CurrentValue = HtmlDecode($this->Camp->CurrentValue);
            }
            $this->Camp->EditValue = HtmlEncode($this->Camp->CurrentValue);
            $this->Camp->PlaceHolder = RemoveHtml($this->Camp->caption());

            // VillageCode
            $curVal = trim(strval($this->VillageCode->CurrentValue));
            if ($curVal != "") {
                $this->VillageCode->ViewValue = $this->VillageCode->lookupCacheOption($curVal);
            } else {
                $this->VillageCode->ViewValue = $this->VillageCode->Lookup !== null && is_array($this->VillageCode->lookupOptions()) && count($this->VillageCode->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->VillageCode->ViewValue !== null) { // Load from cache
                $this->VillageCode->EditValue = array_values($this->VillageCode->lookupOptions());
                if ($this->VillageCode->ViewValue == "") {
                    $this->VillageCode->ViewValue = $Language->phrase("PleaseSelect");
                }
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->VillageCode->Lookup->getTable()->Fields["VillageCode"]->searchExpression(), "=", $this->VillageCode->CurrentValue, $this->VillageCode->Lookup->getTable()->Fields["VillageCode"]->searchDataType(), "");
                }
                $sqlWrk = $this->VillageCode->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                if ($ari > 0) { // Lookup values found
                    $arwrk = $this->VillageCode->Lookup->renderViewRow($rswrk[0]);
                    $this->VillageCode->ViewValue = $this->VillageCode->displayValue($arwrk);
                } else {
                    $this->VillageCode->ViewValue = $Language->phrase("PleaseSelect");
                }
                $arwrk = $rswrk;
                $this->VillageCode->EditValue = $arwrk;
            }
            $this->VillageCode->PlaceHolder = RemoveHtml($this->VillageCode->caption());

            // Village
            $this->Village->setupEditAttributes();
            if (!$this->Village->Raw) {
                $this->Village->CurrentValue = HtmlDecode($this->Village->CurrentValue);
            }
            $this->Village->EditValue = HtmlEncode($this->Village->CurrentValue);
            $this->Village->PlaceHolder = RemoveHtml($this->Village->caption());

            // NRC
            $this->NRC->setupEditAttributes();
            if (!$this->NRC->Raw) {
                $this->NRC->CurrentValue = HtmlDecode($this->NRC->CurrentValue);
            }
            $this->NRC->EditValue = HtmlEncode($this->NRC->CurrentValue);
            $this->NRC->PlaceHolder = RemoveHtml($this->NRC->caption());

            // FarmerName
            $this->FarmerName->setupEditAttributes();
            if (!$this->FarmerName->Raw) {
                $this->FarmerName->CurrentValue = HtmlDecode($this->FarmerName->CurrentValue);
            }
            $this->FarmerName->EditValue = HtmlEncode($this->FarmerName->CurrentValue);
            $this->FarmerName->PlaceHolder = RemoveHtml($this->FarmerName->caption());

            // DOB
            $this->DOB->setupEditAttributes();
            if (!$this->DOB->Raw) {
                $this->DOB->CurrentValue = HtmlDecode($this->DOB->CurrentValue);
            }
            $this->DOB->EditValue = HtmlEncode($this->DOB->CurrentValue);
            $this->DOB->PlaceHolder = RemoveHtml($this->DOB->caption());

            // Age
            $this->Age->setupEditAttributes();
            $this->Age->EditValue = $this->Age->CurrentValue;
            $this->Age->PlaceHolder = RemoveHtml($this->Age->caption());
            if (strval($this->Age->EditValue) != "" && is_numeric($this->Age->EditValue)) {
                $this->Age->EditValue = FormatNumber($this->Age->EditValue, null);
            }

            // Group
            $this->Group->setupEditAttributes();
            $this->Group->EditValue = $this->Group->options(false);
            $this->Group->PlaceHolder = RemoveHtml($this->Group->caption());

            // Gender
            $this->Gender->setupEditAttributes();
            $curVal = trim(strval($this->Gender->CurrentValue));
            if ($curVal != "") {
                $this->Gender->ViewValue = $this->Gender->lookupCacheOption($curVal);
            } else {
                $this->Gender->ViewValue = $this->Gender->Lookup !== null && is_array($this->Gender->lookupOptions()) && count($this->Gender->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->Gender->ViewValue !== null) { // Load from cache
                $this->Gender->EditValue = array_values($this->Gender->lookupOptions());
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->Gender->Lookup->getTable()->Fields["Gender"]->searchExpression(), "=", $this->Gender->CurrentValue, $this->Gender->Lookup->getTable()->Fields["Gender"]->searchDataType(), "");
                }
                $sqlWrk = $this->Gender->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                $arwrk = $rswrk;
                $this->Gender->EditValue = $arwrk;
            }
            $this->Gender->PlaceHolder = RemoveHtml($this->Gender->caption());

            // Seedlings
            $this->Seedlings->setupEditAttributes();
            $this->Seedlings->EditValue = $this->Seedlings->CurrentValue;
            $this->Seedlings->PlaceHolder = RemoveHtml($this->Seedlings->caption());
            if (strval($this->Seedlings->EditValue) != "" && is_numeric($this->Seedlings->EditValue)) {
                $this->Seedlings->EditValue = FormatNumber($this->Seedlings->EditValue, null);
            }

            // Seeds
            $this->Seeds->setupEditAttributes();
            $this->Seeds->EditValue = $this->Seeds->CurrentValue;
            $this->Seeds->PlaceHolder = RemoveHtml($this->Seeds->caption());
            if (strval($this->Seeds->EditValue) != "" && is_numeric($this->Seeds->EditValue)) {
                $this->Seeds->EditValue = FormatNumber($this->Seeds->EditValue, null);
            }

            // Type
            $this->Type->setupEditAttributes();
            $curVal = trim(strval($this->Type->CurrentValue));
            if ($curVal != "") {
                $this->Type->ViewValue = $this->Type->lookupCacheOption($curVal);
            } else {
                $this->Type->ViewValue = $this->Type->Lookup !== null && is_array($this->Type->lookupOptions()) && count($this->Type->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->Type->ViewValue !== null) { // Load from cache
                $this->Type->EditValue = array_values($this->Type->lookupOptions());
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->Type->Lookup->getTable()->Fields["SeedVariety"]->searchExpression(), "=", $this->Type->CurrentValue, $this->Type->Lookup->getTable()->Fields["SeedVariety"]->searchDataType(), "");
                }
                $sqlWrk = $this->Type->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                $arwrk = $rswrk;
                $this->Type->EditValue = $arwrk;
            }
            $this->Type->PlaceHolder = RemoveHtml($this->Type->caption());

            // DateAllocated
            $this->DateAllocated->setupEditAttributes();
            if (!$this->DateAllocated->Raw) {
                $this->DateAllocated->CurrentValue = HtmlDecode($this->DateAllocated->CurrentValue);
            }
            $this->DateAllocated->EditValue = HtmlEncode($this->DateAllocated->CurrentValue);
            $this->DateAllocated->PlaceHolder = RemoveHtml($this->DateAllocated->caption());

            // ID

            // Add refer script

            // DistrictCode
            $this->DistrictCode->HrefValue = "";

            // DistrictName
            $this->DistrictName->HrefValue = "";

            // GRNCode
            $this->GRNCode->HrefValue = "";

            // GRN
            $this->GRN->HrefValue = "";

            // SupplierCode
            $this->SupplierCode->HrefValue = "";

            // Supplier
            $this->Supplier->HrefValue = "";

            // CampCode
            $this->CampCode->HrefValue = "";

            // Camp
            $this->Camp->HrefValue = "";

            // VillageCode
            $this->VillageCode->HrefValue = "";

            // Village
            $this->Village->HrefValue = "";

            // NRC
            $this->NRC->HrefValue = "";

            // FarmerName
            $this->FarmerName->HrefValue = "";

            // DOB
            $this->DOB->HrefValue = "";

            // Age
            $this->Age->HrefValue = "";

            // Group
            $this->Group->HrefValue = "";

            // Gender
            $this->Gender->HrefValue = "";

            // Seedlings
            $this->Seedlings->HrefValue = "";

            // Seeds
            $this->Seeds->HrefValue = "";

            // Type
            $this->Type->HrefValue = "";

            // DateAllocated
            $this->DateAllocated->HrefValue = "";

            // ID
            $this->ID->HrefValue = "";
        } elseif ($this->RowType == RowType::EDIT) {
            // DistrictCode
            $curVal = trim(strval($this->DistrictCode->CurrentValue));
            if ($curVal != "") {
                $this->DistrictCode->ViewValue = $this->DistrictCode->lookupCacheOption($curVal);
            } else {
                $this->DistrictCode->ViewValue = $this->DistrictCode->Lookup !== null && is_array($this->DistrictCode->lookupOptions()) && count($this->DistrictCode->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->DistrictCode->ViewValue !== null) { // Load from cache
                $this->DistrictCode->EditValue = array_values($this->DistrictCode->lookupOptions());
                if ($this->DistrictCode->ViewValue == "") {
                    $this->DistrictCode->ViewValue = $Language->phrase("PleaseSelect");
                }
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->DistrictCode->Lookup->getTable()->Fields["DistrictCode"]->searchExpression(), "=", $this->DistrictCode->CurrentValue, $this->DistrictCode->Lookup->getTable()->Fields["DistrictCode"]->searchDataType(), "");
                }
                $sqlWrk = $this->DistrictCode->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                if ($ari > 0) { // Lookup values found
                    $arwrk = $this->DistrictCode->Lookup->renderViewRow($rswrk[0]);
                    $this->DistrictCode->ViewValue = $this->DistrictCode->displayValue($arwrk);
                } else {
                    $this->DistrictCode->ViewValue = $Language->phrase("PleaseSelect");
                }
                $arwrk = $rswrk;
                $this->DistrictCode->EditValue = $arwrk;
            }
            $this->DistrictCode->PlaceHolder = RemoveHtml($this->DistrictCode->caption());

            // DistrictName
            $this->DistrictName->setupEditAttributes();
            if (!$this->DistrictName->Raw) {
                $this->DistrictName->CurrentValue = HtmlDecode($this->DistrictName->CurrentValue);
            }
            $this->DistrictName->EditValue = HtmlEncode($this->DistrictName->CurrentValue);
            $this->DistrictName->PlaceHolder = RemoveHtml($this->DistrictName->caption());

            // GRNCode
            $this->GRNCode->setupEditAttributes();
            if (!$this->GRNCode->Raw) {
                $this->GRNCode->CurrentValue = HtmlDecode($this->GRNCode->CurrentValue);
            }
            $this->GRNCode->EditValue = HtmlEncode($this->GRNCode->CurrentValue);
            $curVal = strval($this->GRNCode->CurrentValue);
            if ($curVal != "") {
                $this->GRNCode->EditValue = $this->GRNCode->lookupCacheOption($curVal);
                if ($this->GRNCode->EditValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->GRNCode->Lookup->getTable()->Fields["GRN"]->searchExpression(), "=", $curVal, $this->GRNCode->Lookup->getTable()->Fields["GRN"]->searchDataType(), "");
                    $sqlWrk = $this->GRNCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->GRNCode->Lookup->renderViewRow($rswrk[0]);
                        $this->GRNCode->EditValue = $this->GRNCode->displayValue($arwrk);
                    } else {
                        $this->GRNCode->EditValue = HtmlEncode($this->GRNCode->CurrentValue);
                    }
                }
            } else {
                $this->GRNCode->EditValue = null;
            }
            $this->GRNCode->PlaceHolder = RemoveHtml($this->GRNCode->caption());

            // GRN
            $this->GRN->setupEditAttributes();
            if (!$this->GRN->Raw) {
                $this->GRN->CurrentValue = HtmlDecode($this->GRN->CurrentValue);
            }
            $this->GRN->EditValue = HtmlEncode($this->GRN->CurrentValue);
            $this->GRN->PlaceHolder = RemoveHtml($this->GRN->caption());

            // SupplierCode
            $this->SupplierCode->setupEditAttributes();
            $this->SupplierCode->EditValue = $this->SupplierCode->CurrentValue;
            $curVal = strval($this->SupplierCode->CurrentValue);
            if ($curVal != "") {
                $this->SupplierCode->EditValue = $this->SupplierCode->lookupCacheOption($curVal);
                if ($this->SupplierCode->EditValue === null) { // Lookup from database
                    $filterWrk = SearchFilter($this->SupplierCode->Lookup->getTable()->Fields["NurseryCode"]->searchExpression(), "=", $curVal, $this->SupplierCode->Lookup->getTable()->Fields["NurseryCode"]->searchDataType(), "");
                    $sqlWrk = $this->SupplierCode->Lookup->getSql(false, $filterWrk, '', $this, true, true);
                    $conn = Conn();
                    $config = $conn->getConfiguration();
                    $config->setResultCache($this->Cache);
                    $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                    $ari = count($rswrk);
                    if ($ari > 0) { // Lookup values found
                        $arwrk = $this->SupplierCode->Lookup->renderViewRow($rswrk[0]);
                        $this->SupplierCode->EditValue = $this->SupplierCode->displayValue($arwrk);
                    } else {
                        $this->SupplierCode->EditValue = HtmlEncode($this->SupplierCode->CurrentValue);
                    }
                }
            } else {
                $this->SupplierCode->EditValue = null;
            }
            $this->SupplierCode->PlaceHolder = RemoveHtml($this->SupplierCode->caption());

            // Supplier
            $this->Supplier->setupEditAttributes();
            if (!$this->Supplier->Raw) {
                $this->Supplier->CurrentValue = HtmlDecode($this->Supplier->CurrentValue);
            }
            $this->Supplier->EditValue = HtmlEncode($this->Supplier->CurrentValue);
            $this->Supplier->PlaceHolder = RemoveHtml($this->Supplier->caption());

            // CampCode
            $curVal = trim(strval($this->CampCode->CurrentValue));
            if ($curVal != "") {
                $this->CampCode->ViewValue = $this->CampCode->lookupCacheOption($curVal);
            } else {
                $this->CampCode->ViewValue = $this->CampCode->Lookup !== null && is_array($this->CampCode->lookupOptions()) && count($this->CampCode->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->CampCode->ViewValue !== null) { // Load from cache
                $this->CampCode->EditValue = array_values($this->CampCode->lookupOptions());
                if ($this->CampCode->ViewValue == "") {
                    $this->CampCode->ViewValue = $Language->phrase("PleaseSelect");
                }
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->CampCode->Lookup->getTable()->Fields["CampCode"]->searchExpression(), "=", $this->CampCode->CurrentValue, $this->CampCode->Lookup->getTable()->Fields["CampCode"]->searchDataType(), "");
                }
                $sqlWrk = $this->CampCode->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                if ($ari > 0) { // Lookup values found
                    $arwrk = $this->CampCode->Lookup->renderViewRow($rswrk[0]);
                    $this->CampCode->ViewValue = $this->CampCode->displayValue($arwrk);
                } else {
                    $this->CampCode->ViewValue = $Language->phrase("PleaseSelect");
                }
                $arwrk = $rswrk;
                $this->CampCode->EditValue = $arwrk;
            }
            $this->CampCode->PlaceHolder = RemoveHtml($this->CampCode->caption());

            // Camp
            $this->Camp->setupEditAttributes();
            if (!$this->Camp->Raw) {
                $this->Camp->CurrentValue = HtmlDecode($this->Camp->CurrentValue);
            }
            $this->Camp->EditValue = HtmlEncode($this->Camp->CurrentValue);
            $this->Camp->PlaceHolder = RemoveHtml($this->Camp->caption());

            // VillageCode
            $curVal = trim(strval($this->VillageCode->CurrentValue));
            if ($curVal != "") {
                $this->VillageCode->ViewValue = $this->VillageCode->lookupCacheOption($curVal);
            } else {
                $this->VillageCode->ViewValue = $this->VillageCode->Lookup !== null && is_array($this->VillageCode->lookupOptions()) && count($this->VillageCode->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->VillageCode->ViewValue !== null) { // Load from cache
                $this->VillageCode->EditValue = array_values($this->VillageCode->lookupOptions());
                if ($this->VillageCode->ViewValue == "") {
                    $this->VillageCode->ViewValue = $Language->phrase("PleaseSelect");
                }
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->VillageCode->Lookup->getTable()->Fields["VillageCode"]->searchExpression(), "=", $this->VillageCode->CurrentValue, $this->VillageCode->Lookup->getTable()->Fields["VillageCode"]->searchDataType(), "");
                }
                $sqlWrk = $this->VillageCode->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                if ($ari > 0) { // Lookup values found
                    $arwrk = $this->VillageCode->Lookup->renderViewRow($rswrk[0]);
                    $this->VillageCode->ViewValue = $this->VillageCode->displayValue($arwrk);
                } else {
                    $this->VillageCode->ViewValue = $Language->phrase("PleaseSelect");
                }
                $arwrk = $rswrk;
                $this->VillageCode->EditValue = $arwrk;
            }
            $this->VillageCode->PlaceHolder = RemoveHtml($this->VillageCode->caption());

            // Village
            $this->Village->setupEditAttributes();
            if (!$this->Village->Raw) {
                $this->Village->CurrentValue = HtmlDecode($this->Village->CurrentValue);
            }
            $this->Village->EditValue = HtmlEncode($this->Village->CurrentValue);
            $this->Village->PlaceHolder = RemoveHtml($this->Village->caption());

            // NRC
            $this->NRC->setupEditAttributes();
            if (!$this->NRC->Raw) {
                $this->NRC->CurrentValue = HtmlDecode($this->NRC->CurrentValue);
            }
            $this->NRC->EditValue = HtmlEncode($this->NRC->CurrentValue);
            $this->NRC->PlaceHolder = RemoveHtml($this->NRC->caption());

            // FarmerName
            $this->FarmerName->setupEditAttributes();
            if (!$this->FarmerName->Raw) {
                $this->FarmerName->CurrentValue = HtmlDecode($this->FarmerName->CurrentValue);
            }
            $this->FarmerName->EditValue = HtmlEncode($this->FarmerName->CurrentValue);
            $this->FarmerName->PlaceHolder = RemoveHtml($this->FarmerName->caption());

            // DOB
            $this->DOB->setupEditAttributes();
            if (!$this->DOB->Raw) {
                $this->DOB->CurrentValue = HtmlDecode($this->DOB->CurrentValue);
            }
            $this->DOB->EditValue = HtmlEncode($this->DOB->CurrentValue);
            $this->DOB->PlaceHolder = RemoveHtml($this->DOB->caption());

            // Age
            $this->Age->setupEditAttributes();
            $this->Age->EditValue = $this->Age->CurrentValue;
            $this->Age->PlaceHolder = RemoveHtml($this->Age->caption());
            if (strval($this->Age->EditValue) != "" && is_numeric($this->Age->EditValue)) {
                $this->Age->EditValue = FormatNumber($this->Age->EditValue, null);
            }

            // Group
            $this->Group->setupEditAttributes();
            $this->Group->EditValue = $this->Group->options(false);
            $this->Group->PlaceHolder = RemoveHtml($this->Group->caption());

            // Gender
            $this->Gender->setupEditAttributes();
            $curVal = trim(strval($this->Gender->CurrentValue));
            if ($curVal != "") {
                $this->Gender->ViewValue = $this->Gender->lookupCacheOption($curVal);
            } else {
                $this->Gender->ViewValue = $this->Gender->Lookup !== null && is_array($this->Gender->lookupOptions()) && count($this->Gender->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->Gender->ViewValue !== null) { // Load from cache
                $this->Gender->EditValue = array_values($this->Gender->lookupOptions());
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->Gender->Lookup->getTable()->Fields["Gender"]->searchExpression(), "=", $this->Gender->CurrentValue, $this->Gender->Lookup->getTable()->Fields["Gender"]->searchDataType(), "");
                }
                $sqlWrk = $this->Gender->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                $arwrk = $rswrk;
                $this->Gender->EditValue = $arwrk;
            }
            $this->Gender->PlaceHolder = RemoveHtml($this->Gender->caption());

            // Seedlings
            $this->Seedlings->setupEditAttributes();
            $this->Seedlings->EditValue = $this->Seedlings->CurrentValue;
            $this->Seedlings->PlaceHolder = RemoveHtml($this->Seedlings->caption());
            if (strval($this->Seedlings->EditValue) != "" && is_numeric($this->Seedlings->EditValue)) {
                $this->Seedlings->EditValue = FormatNumber($this->Seedlings->EditValue, null);
            }

            // Seeds
            $this->Seeds->setupEditAttributes();
            $this->Seeds->EditValue = $this->Seeds->CurrentValue;
            $this->Seeds->PlaceHolder = RemoveHtml($this->Seeds->caption());
            if (strval($this->Seeds->EditValue) != "" && is_numeric($this->Seeds->EditValue)) {
                $this->Seeds->EditValue = FormatNumber($this->Seeds->EditValue, null);
            }

            // Type
            $this->Type->setupEditAttributes();
            $curVal = trim(strval($this->Type->CurrentValue));
            if ($curVal != "") {
                $this->Type->ViewValue = $this->Type->lookupCacheOption($curVal);
            } else {
                $this->Type->ViewValue = $this->Type->Lookup !== null && is_array($this->Type->lookupOptions()) && count($this->Type->lookupOptions()) > 0 ? $curVal : null;
            }
            if ($this->Type->ViewValue !== null) { // Load from cache
                $this->Type->EditValue = array_values($this->Type->lookupOptions());
            } else { // Lookup from database
                if ($curVal == "") {
                    $filterWrk = "0=1";
                } else {
                    $filterWrk = SearchFilter($this->Type->Lookup->getTable()->Fields["SeedVariety"]->searchExpression(), "=", $this->Type->CurrentValue, $this->Type->Lookup->getTable()->Fields["SeedVariety"]->searchDataType(), "");
                }
                $sqlWrk = $this->Type->Lookup->getSql(true, $filterWrk, '', $this, false, true);
                $conn = Conn();
                $config = $conn->getConfiguration();
                $config->setResultCache($this->Cache);
                $rswrk = $conn->executeCacheQuery($sqlWrk, [], [], $this->CacheProfile)->fetchAll();
                $ari = count($rswrk);
                $arwrk = $rswrk;
                $this->Type->EditValue = $arwrk;
            }
            $this->Type->PlaceHolder = RemoveHtml($this->Type->caption());

            // DateAllocated
            $this->DateAllocated->setupEditAttributes();
            if (!$this->DateAllocated->Raw) {
                $this->DateAllocated->CurrentValue = HtmlDecode($this->DateAllocated->CurrentValue);
            }
            $this->DateAllocated->EditValue = HtmlEncode($this->DateAllocated->CurrentValue);
            $this->DateAllocated->PlaceHolder = RemoveHtml($this->DateAllocated->caption());

            // ID
            $this->ID->setupEditAttributes();
            $this->ID->EditValue = $this->ID->CurrentValue;

            // Edit refer script

            // DistrictCode
            $this->DistrictCode->HrefValue = "";

            // DistrictName
            $this->DistrictName->HrefValue = "";

            // GRNCode
            $this->GRNCode->HrefValue = "";

            // GRN
            $this->GRN->HrefValue = "";

            // SupplierCode
            $this->SupplierCode->HrefValue = "";

            // Supplier
            $this->Supplier->HrefValue = "";

            // CampCode
            $this->CampCode->HrefValue = "";

            // Camp
            $this->Camp->HrefValue = "";

            // VillageCode
            $this->VillageCode->HrefValue = "";

            // Village
            $this->Village->HrefValue = "";

            // NRC
            $this->NRC->HrefValue = "";

            // FarmerName
            $this->FarmerName->HrefValue = "";

            // DOB
            $this->DOB->HrefValue = "";

            // Age
            $this->Age->HrefValue = "";

            // Group
            $this->Group->HrefValue = "";

            // Gender
            $this->Gender->HrefValue = "";

            // Seedlings
            $this->Seedlings->HrefValue = "";

            // Seeds
            $this->Seeds->HrefValue = "";

            // Type
            $this->Type->HrefValue = "";

            // DateAllocated
            $this->DateAllocated->HrefValue = "";

            // ID
            $this->ID->HrefValue = "";
        }
        if ($this->RowType == RowType::ADD || $this->RowType == RowType::EDIT || $this->RowType == RowType::SEARCH) { // Add/Edit/Search row
            $this->setupFieldTitles();
        }

        // Call Row Rendered event
        if ($this->RowType != RowType::AGGREGATEINIT) {
            $this->rowRendered();
        }
    }

    // Validate search
    protected function validateSearch()
    {
        // Check if validation required
        if (!Config("SERVER_VALIDATE")) {
            return true;
        }

        // Return validate result
        $validateSearch = !$this->hasInvalidFields();

        // Call Form_CustomValidate event
        $formCustomError = "";
        $validateSearch = $validateSearch && $this->formCustomValidate($formCustomError);
        if ($formCustomError != "") {
            $this->setFailureMessage($formCustomError);
        }
        return $validateSearch;
    }

    // Validate form
    protected function validateForm()
    {
        global $Language, $Security;

        // Check if validation required
        if (!Config("SERVER_VALIDATE")) {
            return true;
        }
        $validateForm = true;
            if ($this->DistrictCode->Visible && $this->DistrictCode->Required) {
                if (!$this->DistrictCode->IsDetailKey && EmptyValue($this->DistrictCode->FormValue)) {
                    $this->DistrictCode->addErrorMessage(str_replace("%s", $this->DistrictCode->caption(), $this->DistrictCode->RequiredErrorMessage));
                }
            }
            if ($this->DistrictName->Visible && $this->DistrictName->Required) {
                if (!$this->DistrictName->IsDetailKey && EmptyValue($this->DistrictName->FormValue)) {
                    $this->DistrictName->addErrorMessage(str_replace("%s", $this->DistrictName->caption(), $this->DistrictName->RequiredErrorMessage));
                }
            }
            if ($this->GRNCode->Visible && $this->GRNCode->Required) {
                if (!$this->GRNCode->IsDetailKey && EmptyValue($this->GRNCode->FormValue)) {
                    $this->GRNCode->addErrorMessage(str_replace("%s", $this->GRNCode->caption(), $this->GRNCode->RequiredErrorMessage));
                }
            }
            if ($this->GRN->Visible && $this->GRN->Required) {
                if (!$this->GRN->IsDetailKey && EmptyValue($this->GRN->FormValue)) {
                    $this->GRN->addErrorMessage(str_replace("%s", $this->GRN->caption(), $this->GRN->RequiredErrorMessage));
                }
            }
            if ($this->SupplierCode->Visible && $this->SupplierCode->Required) {
                if (!$this->SupplierCode->IsDetailKey && EmptyValue($this->SupplierCode->FormValue)) {
                    $this->SupplierCode->addErrorMessage(str_replace("%s", $this->SupplierCode->caption(), $this->SupplierCode->RequiredErrorMessage));
                }
            }
            if (!CheckInteger($this->SupplierCode->FormValue)) {
                $this->SupplierCode->addErrorMessage($this->SupplierCode->getErrorMessage(false));
            }
            if ($this->Supplier->Visible && $this->Supplier->Required) {
                if (!$this->Supplier->IsDetailKey && EmptyValue($this->Supplier->FormValue)) {
                    $this->Supplier->addErrorMessage(str_replace("%s", $this->Supplier->caption(), $this->Supplier->RequiredErrorMessage));
                }
            }
            if ($this->CampCode->Visible && $this->CampCode->Required) {
                if (!$this->CampCode->IsDetailKey && EmptyValue($this->CampCode->FormValue)) {
                    $this->CampCode->addErrorMessage(str_replace("%s", $this->CampCode->caption(), $this->CampCode->RequiredErrorMessage));
                }
            }
            if ($this->Camp->Visible && $this->Camp->Required) {
                if (!$this->Camp->IsDetailKey && EmptyValue($this->Camp->FormValue)) {
                    $this->Camp->addErrorMessage(str_replace("%s", $this->Camp->caption(), $this->Camp->RequiredErrorMessage));
                }
            }
            if ($this->VillageCode->Visible && $this->VillageCode->Required) {
                if (!$this->VillageCode->IsDetailKey && EmptyValue($this->VillageCode->FormValue)) {
                    $this->VillageCode->addErrorMessage(str_replace("%s", $this->VillageCode->caption(), $this->VillageCode->RequiredErrorMessage));
                }
            }
            if ($this->Village->Visible && $this->Village->Required) {
                if (!$this->Village->IsDetailKey && EmptyValue($this->Village->FormValue)) {
                    $this->Village->addErrorMessage(str_replace("%s", $this->Village->caption(), $this->Village->RequiredErrorMessage));
                }
            }
            if ($this->NRC->Visible && $this->NRC->Required) {
                if (!$this->NRC->IsDetailKey && EmptyValue($this->NRC->FormValue)) {
                    $this->NRC->addErrorMessage(str_replace("%s", $this->NRC->caption(), $this->NRC->RequiredErrorMessage));
                }
            }
            if ($this->FarmerName->Visible && $this->FarmerName->Required) {
                if (!$this->FarmerName->IsDetailKey && EmptyValue($this->FarmerName->FormValue)) {
                    $this->FarmerName->addErrorMessage(str_replace("%s", $this->FarmerName->caption(), $this->FarmerName->RequiredErrorMessage));
                }
            }
            if ($this->DOB->Visible && $this->DOB->Required) {
                if (!$this->DOB->IsDetailKey && EmptyValue($this->DOB->FormValue)) {
                    $this->DOB->addErrorMessage(str_replace("%s", $this->DOB->caption(), $this->DOB->RequiredErrorMessage));
                }
            }
            if ($this->Age->Visible && $this->Age->Required) {
                if (!$this->Age->IsDetailKey && EmptyValue($this->Age->FormValue)) {
                    $this->Age->addErrorMessage(str_replace("%s", $this->Age->caption(), $this->Age->RequiredErrorMessage));
                }
            }
            if (!CheckNumber($this->Age->FormValue)) {
                $this->Age->addErrorMessage($this->Age->getErrorMessage(false));
            }
            if ($this->Group->Visible && $this->Group->Required) {
                if ($this->Group->FormValue == "") {
                    $this->Group->addErrorMessage(str_replace("%s", $this->Group->caption(), $this->Group->RequiredErrorMessage));
                }
            }
            if ($this->Gender->Visible && $this->Gender->Required) {
                if (!$this->Gender->IsDetailKey && EmptyValue($this->Gender->FormValue)) {
                    $this->Gender->addErrorMessage(str_replace("%s", $this->Gender->caption(), $this->Gender->RequiredErrorMessage));
                }
            }
            if ($this->Seedlings->Visible && $this->Seedlings->Required) {
                if (!$this->Seedlings->IsDetailKey && EmptyValue($this->Seedlings->FormValue)) {
                    $this->Seedlings->addErrorMessage(str_replace("%s", $this->Seedlings->caption(), $this->Seedlings->RequiredErrorMessage));
                }
            }
            if (!CheckNumber($this->Seedlings->FormValue)) {
                $this->Seedlings->addErrorMessage($this->Seedlings->getErrorMessage(false));
            }
            if ($this->Seeds->Visible && $this->Seeds->Required) {
                if (!$this->Seeds->IsDetailKey && EmptyValue($this->Seeds->FormValue)) {
                    $this->Seeds->addErrorMessage(str_replace("%s", $this->Seeds->caption(), $this->Seeds->RequiredErrorMessage));
                }
            }
            if ($this->Type->Visible && $this->Type->Required) {
                if (!$this->Type->IsDetailKey && EmptyValue($this->Type->FormValue)) {
                    $this->Type->addErrorMessage(str_replace("%s", $this->Type->caption(), $this->Type->RequiredErrorMessage));
                }
            }
            if ($this->DateAllocated->Visible && $this->DateAllocated->Required) {
                if (!$this->DateAllocated->IsDetailKey && EmptyValue($this->DateAllocated->FormValue)) {
                    $this->DateAllocated->addErrorMessage(str_replace("%s", $this->DateAllocated->caption(), $this->DateAllocated->RequiredErrorMessage));
                }
            }
            if ($this->ID->Visible && $this->ID->Required) {
                if (!$this->ID->IsDetailKey && EmptyValue($this->ID->FormValue)) {
                    $this->ID->addErrorMessage(str_replace("%s", $this->ID->caption(), $this->ID->RequiredErrorMessage));
                }
            }

        // Return validate result
        $validateForm = $validateForm && !$this->hasInvalidFields();

        // Call Form_CustomValidate event
        $formCustomError = "";
        $validateForm = $validateForm && $this->formCustomValidate($formCustomError);
        if ($formCustomError != "") {
            $this->setFailureMessage($formCustomError);
        }
        return $validateForm;
    }

    // Delete records based on current filter
    protected function deleteRows()
    {
        global $Language, $Security;
        if (!$Security->canDelete()) {
            $this->setFailureMessage($Language->phrase("NoDeletePermission")); // No delete permission
            return false;
        }
        $sql = $this->getCurrentSql();
        $conn = $this->getConnection();
        $rows = $conn->fetchAllAssociative($sql);
        if (count($rows) == 0) {
            $this->setFailureMessage($Language->phrase("NoRecord")); // No record found
            return false;
        }

        // Clone old rows
        $rsold = $rows;
        $successKeys = [];
        $failKeys = [];
        foreach ($rsold as $row) {
            $thisKey = "";
            if ($thisKey != "") {
                $thisKey .= Config("COMPOSITE_KEY_SEPARATOR");
            }
            $thisKey .= $row['ID'];

            // Call row deleting event
            $deleteRow = $this->rowDeleting($row);
            if ($deleteRow) { // Delete
                $deleteRow = $this->delete($row);
                if (!$deleteRow && !EmptyValue($this->DbErrorMessage)) { // Show database error
                    $this->setFailureMessage($this->DbErrorMessage);
                }
            }
            if ($deleteRow === false) {
                if ($this->UseTransaction) {
                    $successKeys = []; // Reset success keys
                    break;
                }
                $failKeys[] = $thisKey;
            } else {
                if (Config("DELETE_UPLOADED_FILES")) { // Delete old files
                    $this->deleteUploadedFiles($row);
                }

                // Call Row Deleted event
                $this->rowDeleted($row);
                $successKeys[] = $thisKey;
            }
        }

        // Any records deleted
        $deleteRows = count($successKeys) > 0;
        if (!$deleteRows) {
            // Set up error message
            if ($this->getSuccessMessage() != "" || $this->getFailureMessage() != "") {
                // Use the message, do nothing
            } elseif ($this->CancelMessage != "") {
                $this->setFailureMessage($this->CancelMessage);
                $this->CancelMessage = "";
            } else {
                $this->setFailureMessage($Language->phrase("DeleteCancelled"));
            }
        }
        return $deleteRows;
    }

    // Update record based on key values
    protected function editRow()
    {
        global $Security, $Language;
        $oldKeyFilter = $this->getRecordFilter();
        $filter = $this->applyUserIDFilters($oldKeyFilter);
        $conn = $this->getConnection();

        // Load old row
        $this->CurrentFilter = $filter;
        $sql = $this->getCurrentSql();
        $rsold = $conn->fetchAssociative($sql);
        if (!$rsold) {
            $this->setFailureMessage($Language->phrase("NoRecord")); // Set no record message
            return false; // Update Failed
        } else {
            // Load old values
            $this->loadDbValues($rsold);
        }

        // Get new row
        $rsnew = $this->getEditRow($rsold);

        // Update current values
        $this->setCurrentValues($rsnew);

        // Call Row Updating event
        $updateRow = $this->rowUpdating($rsold, $rsnew);
        if ($updateRow) {
            if (count($rsnew) > 0) {
                $this->CurrentFilter = $filter; // Set up current filter
                $editRow = $this->update($rsnew, "", $rsold);
                if (!$editRow && !EmptyValue($this->DbErrorMessage)) { // Show database error
                    $this->setFailureMessage($this->DbErrorMessage);
                }
            } else {
                $editRow = true; // No field to update
            }
            if ($editRow) {
            }
        } else {
            if ($this->getSuccessMessage() != "" || $this->getFailureMessage() != "") {
                // Use the message, do nothing
            } elseif ($this->CancelMessage != "") {
                $this->setFailureMessage($this->CancelMessage);
                $this->CancelMessage = "";
            } else {
                $this->setFailureMessage($Language->phrase("UpdateCancelled"));
            }
            $editRow = false;
        }

        // Call Row_Updated event
        if ($editRow) {
            $this->rowUpdated($rsold, $rsnew);
        }
        return $editRow;
    }

    /**
     * Get edit row
     *
     * @return array
     */
    protected function getEditRow($rsold)
    {
        global $Security;
        $rsnew = [];

        // DistrictCode
        $this->DistrictCode->setDbValueDef($rsnew, $this->DistrictCode->CurrentValue, $this->DistrictCode->ReadOnly);

        // DistrictName
        $this->DistrictName->setDbValueDef($rsnew, $this->DistrictName->CurrentValue, $this->DistrictName->ReadOnly);

        // GRNCode
        $this->GRNCode->setDbValueDef($rsnew, $this->GRNCode->CurrentValue, $this->GRNCode->ReadOnly);

        // GRN
        $this->GRN->setDbValueDef($rsnew, $this->GRN->CurrentValue, $this->GRN->ReadOnly);

        // SupplierCode
        $this->SupplierCode->setDbValueDef($rsnew, $this->SupplierCode->CurrentValue, $this->SupplierCode->ReadOnly);

        // Supplier
        $this->Supplier->setDbValueDef($rsnew, $this->Supplier->CurrentValue, $this->Supplier->ReadOnly);

        // CampCode
        $this->CampCode->setDbValueDef($rsnew, $this->CampCode->CurrentValue, $this->CampCode->ReadOnly);

        // Camp
        $this->Camp->setDbValueDef($rsnew, $this->Camp->CurrentValue, $this->Camp->ReadOnly);

        // VillageCode
        $this->VillageCode->setDbValueDef($rsnew, $this->VillageCode->CurrentValue, $this->VillageCode->ReadOnly);

        // Village
        $this->Village->setDbValueDef($rsnew, $this->Village->CurrentValue, $this->Village->ReadOnly);

        // NRC
        $this->NRC->setDbValueDef($rsnew, $this->NRC->CurrentValue, $this->NRC->ReadOnly);

        // FarmerName
        $this->FarmerName->setDbValueDef($rsnew, $this->FarmerName->CurrentValue, $this->FarmerName->ReadOnly);

        // DOB
        $this->DOB->setDbValueDef($rsnew, $this->DOB->CurrentValue, $this->DOB->ReadOnly);

        // Age
        $this->Age->setDbValueDef($rsnew, $this->Age->CurrentValue, $this->Age->ReadOnly);

        // Group
        $this->Group->setDbValueDef($rsnew, $this->Group->CurrentValue, $this->Group->ReadOnly);

        // Gender
        $this->Gender->setDbValueDef($rsnew, $this->Gender->CurrentValue, $this->Gender->ReadOnly);

        // Seedlings
        $this->Seedlings->setDbValueDef($rsnew, $this->Seedlings->CurrentValue, $this->Seedlings->ReadOnly);

        // Seeds
        $this->Seeds->setDbValueDef($rsnew, $this->Seeds->CurrentValue, $this->Seeds->ReadOnly);

        // Type
        $this->Type->setDbValueDef($rsnew, $this->Type->CurrentValue, $this->Type->ReadOnly);

        // DateAllocated
        $this->DateAllocated->setDbValueDef($rsnew, $this->DateAllocated->CurrentValue, $this->DateAllocated->ReadOnly);
        return $rsnew;
    }

    /**
     * Restore edit form from row
     * @param array $row Row
     */
    protected function restoreEditFormFromRow($row)
    {
        if (isset($row['DistrictCode'])) { // DistrictCode
            $this->DistrictCode->CurrentValue = $row['DistrictCode'];
        }
        if (isset($row['DistrictName'])) { // DistrictName
            $this->DistrictName->CurrentValue = $row['DistrictName'];
        }
        if (isset($row['GRNCode'])) { // GRNCode
            $this->GRNCode->CurrentValue = $row['GRNCode'];
        }
        if (isset($row['GRN'])) { // GRN
            $this->GRN->CurrentValue = $row['GRN'];
        }
        if (isset($row['SupplierCode'])) { // SupplierCode
            $this->SupplierCode->CurrentValue = $row['SupplierCode'];
        }
        if (isset($row['Supplier'])) { // Supplier
            $this->Supplier->CurrentValue = $row['Supplier'];
        }
        if (isset($row['CampCode'])) { // CampCode
            $this->CampCode->CurrentValue = $row['CampCode'];
        }
        if (isset($row['Camp'])) { // Camp
            $this->Camp->CurrentValue = $row['Camp'];
        }
        if (isset($row['VillageCode'])) { // VillageCode
            $this->VillageCode->CurrentValue = $row['VillageCode'];
        }
        if (isset($row['Village'])) { // Village
            $this->Village->CurrentValue = $row['Village'];
        }
        if (isset($row['NRC'])) { // NRC
            $this->NRC->CurrentValue = $row['NRC'];
        }
        if (isset($row['FarmerName'])) { // FarmerName
            $this->FarmerName->CurrentValue = $row['FarmerName'];
        }
        if (isset($row['DOB'])) { // DOB
            $this->DOB->CurrentValue = $row['DOB'];
        }
        if (isset($row['Age'])) { // Age
            $this->Age->CurrentValue = $row['Age'];
        }
        if (isset($row['Group'])) { // Group
            $this->Group->CurrentValue = $row['Group'];
        }
        if (isset($row['Gender'])) { // Gender
            $this->Gender->CurrentValue = $row['Gender'];
        }
        if (isset($row['Seedlings'])) { // Seedlings
            $this->Seedlings->CurrentValue = $row['Seedlings'];
        }
        if (isset($row['Seeds'])) { // Seeds
            $this->Seeds->CurrentValue = $row['Seeds'];
        }
        if (isset($row['Type'])) { // Type
            $this->Type->CurrentValue = $row['Type'];
        }
        if (isset($row['DateAllocated'])) { // DateAllocated
            $this->DateAllocated->CurrentValue = $row['DateAllocated'];
        }
    }

    // Load row hash
    protected function loadRowHash()
    {
        $filter = $this->getRecordFilter();

        // Load SQL based on filter
        $this->CurrentFilter = $filter;
        $sql = $this->getCurrentSql();
        $conn = $this->getConnection();
        $row = $conn->fetchAssociative($sql);
        $this->HashValue = $row ? $this->getRowHash($row) : ""; // Get hash value for record
    }

    // Get Row Hash
    public function getRowHash($row)
    {
        if (!$row) {
            return "";
        }
        $hash = "";
        $hash .= GetFieldHash($row['DistrictCode']); // DistrictCode
        $hash .= GetFieldHash($row['DistrictName']); // DistrictName
        $hash .= GetFieldHash($row['GRNCode']); // GRNCode
        $hash .= GetFieldHash($row['GRN']); // GRN
        $hash .= GetFieldHash($row['SupplierCode']); // SupplierCode
        $hash .= GetFieldHash($row['Supplier']); // Supplier
        $hash .= GetFieldHash($row['CampCode']); // CampCode
        $hash .= GetFieldHash($row['Camp']); // Camp
        $hash .= GetFieldHash($row['VillageCode']); // VillageCode
        $hash .= GetFieldHash($row['Village']); // Village
        $hash .= GetFieldHash($row['NRC']); // NRC
        $hash .= GetFieldHash($row['FarmerName']); // FarmerName
        $hash .= GetFieldHash($row['DOB']); // DOB
        $hash .= GetFieldHash($row['Age']); // Age
        $hash .= GetFieldHash($row['Group']); // Group
        $hash .= GetFieldHash($row['Gender']); // Gender
        $hash .= GetFieldHash($row['Seedlings']); // Seedlings
        $hash .= GetFieldHash($row['Seeds']); // Seeds
        $hash .= GetFieldHash($row['Type']); // Type
        $hash .= GetFieldHash($row['DateAllocated']); // DateAllocated
        return md5($hash);
    }

    // Add record
    protected function addRow($rsold = null)
    {
        global $Language, $Security;

        // Get new row
        $rsnew = $this->getAddRow();

        // Update current values
        $this->setCurrentValues($rsnew);
        $conn = $this->getConnection();

        // Load db values from old row
        $this->loadDbValues($rsold);

        // Call Row Inserting event
        $insertRow = $this->rowInserting($rsold, $rsnew);
        if ($insertRow) {
            $addRow = $this->insert($rsnew);
            if ($addRow) {
            } elseif (!EmptyValue($this->DbErrorMessage)) { // Show database error
                $this->setFailureMessage($this->DbErrorMessage);
            }
        } else {
            if ($this->getSuccessMessage() != "" || $this->getFailureMessage() != "") {
                // Use the message, do nothing
            } elseif ($this->CancelMessage != "") {
                $this->setFailureMessage($this->CancelMessage);
                $this->CancelMessage = "";
            } else {
                $this->setFailureMessage($Language->phrase("InsertCancelled"));
            }
            $addRow = false;
        }
        if ($addRow) {
            // Call Row Inserted event
            $this->rowInserted($rsold, $rsnew);
        }
        return $addRow;
    }

    /**
     * Get add row
     *
     * @return array
     */
    protected function getAddRow()
    {
        global $Security;
        $rsnew = [];

        // DistrictCode
        $this->DistrictCode->setDbValueDef($rsnew, $this->DistrictCode->CurrentValue, false);

        // DistrictName
        $this->DistrictName->setDbValueDef($rsnew, $this->DistrictName->CurrentValue, false);

        // GRNCode
        $this->GRNCode->setDbValueDef($rsnew, $this->GRNCode->CurrentValue, false);

        // GRN
        $this->GRN->setDbValueDef($rsnew, $this->GRN->CurrentValue, false);

        // SupplierCode
        $this->SupplierCode->setDbValueDef($rsnew, $this->SupplierCode->CurrentValue, false);

        // Supplier
        $this->Supplier->setDbValueDef($rsnew, $this->Supplier->CurrentValue, false);

        // CampCode
        $this->CampCode->setDbValueDef($rsnew, $this->CampCode->CurrentValue, false);

        // Camp
        $this->Camp->setDbValueDef($rsnew, $this->Camp->CurrentValue, false);

        // VillageCode
        $this->VillageCode->setDbValueDef($rsnew, $this->VillageCode->CurrentValue, false);

        // Village
        $this->Village->setDbValueDef($rsnew, $this->Village->CurrentValue, false);

        // NRC
        $this->NRC->setDbValueDef($rsnew, $this->NRC->CurrentValue, false);

        // FarmerName
        $this->FarmerName->setDbValueDef($rsnew, $this->FarmerName->CurrentValue, false);

        // DOB
        $this->DOB->setDbValueDef($rsnew, $this->DOB->CurrentValue, false);

        // Age
        $this->Age->setDbValueDef($rsnew, $this->Age->CurrentValue, false);

        // Group
        $this->Group->setDbValueDef($rsnew, $this->Group->CurrentValue, false);

        // Gender
        $this->Gender->setDbValueDef($rsnew, $this->Gender->CurrentValue, false);

        // Seedlings
        $this->Seedlings->setDbValueDef($rsnew, $this->Seedlings->CurrentValue, false);

        // Seeds
        $this->Seeds->setDbValueDef($rsnew, $this->Seeds->CurrentValue, false);

        // Type
        $this->Type->setDbValueDef($rsnew, $this->Type->CurrentValue, false);

        // DateAllocated
        $this->DateAllocated->setDbValueDef($rsnew, $this->DateAllocated->CurrentValue, false);
        return $rsnew;
    }

    /**
     * Restore add form from row
     * @param array $row Row
     */
    protected function restoreAddFormFromRow($row)
    {
        if (isset($row['DistrictCode'])) { // DistrictCode
            $this->DistrictCode->setFormValue($row['DistrictCode']);
        }
        if (isset($row['DistrictName'])) { // DistrictName
            $this->DistrictName->setFormValue($row['DistrictName']);
        }
        if (isset($row['GRNCode'])) { // GRNCode
            $this->GRNCode->setFormValue($row['GRNCode']);
        }
        if (isset($row['GRN'])) { // GRN
            $this->GRN->setFormValue($row['GRN']);
        }
        if (isset($row['SupplierCode'])) { // SupplierCode
            $this->SupplierCode->setFormValue($row['SupplierCode']);
        }
        if (isset($row['Supplier'])) { // Supplier
            $this->Supplier->setFormValue($row['Supplier']);
        }
        if (isset($row['CampCode'])) { // CampCode
            $this->CampCode->setFormValue($row['CampCode']);
        }
        if (isset($row['Camp'])) { // Camp
            $this->Camp->setFormValue($row['Camp']);
        }
        if (isset($row['VillageCode'])) { // VillageCode
            $this->VillageCode->setFormValue($row['VillageCode']);
        }
        if (isset($row['Village'])) { // Village
            $this->Village->setFormValue($row['Village']);
        }
        if (isset($row['NRC'])) { // NRC
            $this->NRC->setFormValue($row['NRC']);
        }
        if (isset($row['FarmerName'])) { // FarmerName
            $this->FarmerName->setFormValue($row['FarmerName']);
        }
        if (isset($row['DOB'])) { // DOB
            $this->DOB->setFormValue($row['DOB']);
        }
        if (isset($row['Age'])) { // Age
            $this->Age->setFormValue($row['Age']);
        }
        if (isset($row['Group'])) { // Group
            $this->Group->setFormValue($row['Group']);
        }
        if (isset($row['Gender'])) { // Gender
            $this->Gender->setFormValue($row['Gender']);
        }
        if (isset($row['Seedlings'])) { // Seedlings
            $this->Seedlings->setFormValue($row['Seedlings']);
        }
        if (isset($row['Seeds'])) { // Seeds
            $this->Seeds->setFormValue($row['Seeds']);
        }
        if (isset($row['Type'])) { // Type
            $this->Type->setFormValue($row['Type']);
        }
        if (isset($row['DateAllocated'])) { // DateAllocated
            $this->DateAllocated->setFormValue($row['DateAllocated']);
        }
    }

    // Load advanced search
    public function loadAdvancedSearch()
    {
        $this->DistrictCode->AdvancedSearch->load();
        $this->DistrictName->AdvancedSearch->load();
        $this->GRNCode->AdvancedSearch->load();
        $this->GRN->AdvancedSearch->load();
        $this->SupplierCode->AdvancedSearch->load();
        $this->Supplier->AdvancedSearch->load();
        $this->CampCode->AdvancedSearch->load();
        $this->Camp->AdvancedSearch->load();
        $this->VillageCode->AdvancedSearch->load();
        $this->Village->AdvancedSearch->load();
        $this->NRC->AdvancedSearch->load();
        $this->FarmerName->AdvancedSearch->load();
        $this->DOB->AdvancedSearch->load();
        $this->Age->AdvancedSearch->load();
        $this->Group->AdvancedSearch->load();
        $this->Gender->AdvancedSearch->load();
        $this->Seedlings->AdvancedSearch->load();
        $this->Seeds->AdvancedSearch->load();
        $this->Type->AdvancedSearch->load();
        $this->DateAllocated->AdvancedSearch->load();
        $this->ID->AdvancedSearch->load();
    }

    // Get export HTML tag
    protected function getExportTag($type, $custom = false)
    {
        global $Language;
        if ($type == "print" || $custom) { // Printer friendly / custom export
            $pageUrl = $this->pageUrl(false);
            $exportUrl = GetUrl($pageUrl . "export=" . $type . ($custom ? "&amp;custom=1" : ""));
        } else { // Export API URL
            $exportUrl = GetApiUrl(Config("API_EXPORT_ACTION") . "/" . $type . "/" . $this->TableVar);
        }
        if (SameText($type, "excel")) {
            if ($custom) {
                return "<button type=\"button\" class=\"btn btn-default ew-export-link ew-excel\" title=\"" . HtmlEncode($Language->phrase("ExportToExcel", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToExcel", true)) . "\" form=\"fcidp_datalist\" data-url=\"$exportUrl\" data-ew-action=\"export\" data-export=\"excel\" data-custom=\"true\" data-export-selected=\"false\">" . $Language->phrase("ExportToExcel") . "</button>";
            } else {
                return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-excel\" title=\"" . HtmlEncode($Language->phrase("ExportToExcel", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToExcel", true)) . "\">" . $Language->phrase("ExportToExcel") . "</a>";
            }
        } elseif (SameText($type, "word")) {
            if ($custom) {
                return "<button type=\"button\" class=\"btn btn-default ew-export-link ew-word\" title=\"" . HtmlEncode($Language->phrase("ExportToWord", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToWord", true)) . "\" form=\"fcidp_datalist\" data-url=\"$exportUrl\" data-ew-action=\"export\" data-export=\"word\" data-custom=\"true\" data-export-selected=\"false\">" . $Language->phrase("ExportToWord") . "</button>";
            } else {
                return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-word\" title=\"" . HtmlEncode($Language->phrase("ExportToWord", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToWord", true)) . "\">" . $Language->phrase("ExportToWord") . "</a>";
            }
        } elseif (SameText($type, "pdf")) {
            if ($custom) {
                return "<button type=\"button\" class=\"btn btn-default ew-export-link ew-pdf\" title=\"" . HtmlEncode($Language->phrase("ExportToPdf", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToPdf", true)) . "\" form=\"fcidp_datalist\" data-url=\"$exportUrl\" data-ew-action=\"export\" data-export=\"pdf\" data-custom=\"true\" data-export-selected=\"false\">" . $Language->phrase("ExportToPdf") . "</button>";
            } else {
                return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-pdf\" title=\"" . HtmlEncode($Language->phrase("ExportToPdf", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToPdf", true)) . "\">" . $Language->phrase("ExportToPdf") . "</a>";
            }
        } elseif (SameText($type, "html")) {
            return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-html\" title=\"" . HtmlEncode($Language->phrase("ExportToHtml", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToHtml", true)) . "\">" . $Language->phrase("ExportToHtml") . "</a>";
        } elseif (SameText($type, "xml")) {
            return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-xml\" title=\"" . HtmlEncode($Language->phrase("ExportToXml", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToXml", true)) . "\">" . $Language->phrase("ExportToXml") . "</a>";
        } elseif (SameText($type, "csv")) {
            return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-csv\" title=\"" . HtmlEncode($Language->phrase("ExportToCsv", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("ExportToCsv", true)) . "\">" . $Language->phrase("ExportToCsv") . "</a>";
        } elseif (SameText($type, "email")) {
            $url = $custom ? ' data-url="' . $exportUrl . '"' : '';
            return '<button type="button" class="btn btn-default ew-export-link ew-email" title="' . $Language->phrase("ExportToEmail", true) . '" data-caption="' . $Language->phrase("ExportToEmail", true) . '" form="fcidp_datalist" data-ew-action="email" data-custom="false" data-hdr="' . $Language->phrase("ExportToEmail", true) . '" data-exported-selected="false"' . $url . '>' . $Language->phrase("ExportToEmail") . '</button>';
        } elseif (SameText($type, "print")) {
            return "<a href=\"$exportUrl\" class=\"btn btn-default ew-export-link ew-print\" title=\"" . HtmlEncode($Language->phrase("PrinterFriendly", true)) . "\" data-caption=\"" . HtmlEncode($Language->phrase("PrinterFriendly", true)) . "\">" . $Language->phrase("PrinterFriendly") . "</a>";
        }
    }

    // Set up export options
    protected function setupExportOptions()
    {
        global $Language, $Security;

        // Printer friendly
        $item = &$this->ExportOptions->add("print");
        $item->Body = $this->getExportTag("print");
        $item->Visible = true;

        // Export to Excel
        $item = &$this->ExportOptions->add("excel");
        $item->Body = $this->getExportTag("excel");
        $item->Visible = true;

        // Export to Word
        $item = &$this->ExportOptions->add("word");
        $item->Body = $this->getExportTag("word");
        $item->Visible = true;

        // Export to HTML
        $item = &$this->ExportOptions->add("html");
        $item->Body = $this->getExportTag("html");
        $item->Visible = false;

        // Export to XML
        $item = &$this->ExportOptions->add("xml");
        $item->Body = $this->getExportTag("xml");
        $item->Visible = false;

        // Export to CSV
        $item = &$this->ExportOptions->add("csv");
        $item->Body = $this->getExportTag("csv");
        $item->Visible = true;

        // Export to PDF
        $item = &$this->ExportOptions->add("pdf");
        $item->Body = $this->getExportTag("pdf");
        $item->Visible = false;

        // Export to Email
        $item = &$this->ExportOptions->add("email");
        $item->Body = $this->getExportTag("email");
        $item->Visible = false;

        // Drop down button for export
        $this->ExportOptions->UseButtonGroup = true;
        $this->ExportOptions->UseDropDownButton = false;
        if ($this->ExportOptions->UseButtonGroup && IsMobile()) {
            $this->ExportOptions->UseDropDownButton = true;
        }
        $this->ExportOptions->DropDownButtonPhrase = $Language->phrase("ButtonExport");

        // Add group option item
        $item = &$this->ExportOptions->addGroupOption();
        $item->Body = "";
        $item->Visible = false;
        if (!$Security->canExport()) { // Export not allowed
            $this->ExportOptions->hideAllOptions();
        }
    }

    // Set up search options
    protected function setupSearchOptions()
    {
        global $Language, $Security;
        $pageUrl = $this->pageUrl(false);
        $this->SearchOptions = new ListOptions(TagClassName: "ew-search-option");

        // Search button
        $item = &$this->SearchOptions->add("searchtoggle");
        $searchToggleClass = ($this->SearchWhere != "") ? " active" : "";
        $item->Body = "<a class=\"btn btn-default ew-search-toggle" . $searchToggleClass . "\" role=\"button\" title=\"" . $Language->phrase("SearchPanel") . "\" data-caption=\"" . $Language->phrase("SearchPanel") . "\" data-ew-action=\"search-toggle\" data-form=\"fcidp_datasrch\" aria-pressed=\"" . ($searchToggleClass == " active" ? "true" : "false") . "\">" . $Language->phrase("SearchLink") . "</a>";
        $item->Visible = true;

        // Show all button
        $item = &$this->SearchOptions->add("showall");
        if ($this->UseCustomTemplate || !$this->UseAjaxActions) {
            $item->Body = "<a class=\"btn btn-default ew-show-all\" role=\"button\" title=\"" . $Language->phrase("ShowAll") . "\" data-caption=\"" . $Language->phrase("ShowAll") . "\" href=\"" . $pageUrl . "cmd=reset\">" . $Language->phrase("ShowAllBtn") . "</a>";
        } else {
            $item->Body = "<a class=\"btn btn-default ew-show-all\" role=\"button\" title=\"" . $Language->phrase("ShowAll") . "\" data-caption=\"" . $Language->phrase("ShowAll") . "\" data-ew-action=\"refresh\" data-url=\"" . $pageUrl . "cmd=reset\">" . $Language->phrase("ShowAllBtn") . "</a>";
        }
        $item->Visible = ($this->SearchWhere != $this->DefaultSearchWhere && $this->SearchWhere != "0=101");

        // Advanced search button
        $item = &$this->SearchOptions->add("advancedsearch");
        if ($this->ModalSearch && !IsMobile()) {
            $item->Body = "<a class=\"btn btn-default ew-advanced-search\" title=\"" . $Language->phrase("AdvancedSearch", true) . "\" data-table=\"cidp_data\" data-caption=\"" . $Language->phrase("AdvancedSearch", true) . "\" data-ew-action=\"modal\" data-url=\"CidpDataSearch\" data-btn=\"SearchBtn\">" . $Language->phrase("AdvancedSearch", false) . "</a>";
        } else {
            $item->Body = "<a class=\"btn btn-default ew-advanced-search\" title=\"" . $Language->phrase("AdvancedSearch", true) . "\" data-caption=\"" . $Language->phrase("AdvancedSearch", true) . "\" href=\"CidpDataSearch\">" . $Language->phrase("AdvancedSearch", false) . "</a>";
        }
        $item->Visible = true;

        // Search highlight button
        $item = &$this->SearchOptions->add("searchhighlight");
        $item->Body = "<a class=\"btn btn-default ew-highlight active\" role=\"button\" title=\"" . $Language->phrase("Highlight") . "\" data-caption=\"" . $Language->phrase("Highlight") . "\" data-ew-action=\"highlight\" data-form=\"fcidp_datasrch\" data-name=\"" . $this->highlightName() . "\">" . $Language->phrase("HighlightBtn") . "</a>";
        $item->Visible = ($this->SearchWhere != "" && $this->TotalRecords > 0);

        // Button group for search
        $this->SearchOptions->UseDropDownButton = false;
        $this->SearchOptions->UseButtonGroup = true;
        $this->SearchOptions->DropDownButtonPhrase = $Language->phrase("ButtonSearch");

        // Add group option item
        $item = &$this->SearchOptions->addGroupOption();
        $item->Body = "";
        $item->Visible = false;

        // Hide search options
        if ($this->isExport() || $this->CurrentAction && $this->CurrentAction != "search") {
            $this->SearchOptions->hideAllOptions();
        }
        if (!$Security->canSearch()) {
            $this->SearchOptions->hideAllOptions();
            $this->FilterOptions->hideAllOptions();
        }
    }

    // Check if any search fields
    public function hasSearchFields()
    {
        return true;
    }

    // Render search options
    protected function renderSearchOptions()
    {
        if (!$this->hasSearchFields() && $this->SearchOptions["searchtoggle"]) {
            $this->SearchOptions["searchtoggle"]->Visible = false;
        }
    }

    /**
    * Export data in HTML/CSV/Word/Excel/XML/Email/PDF format
    *
    * @param bool $return Return the data rather than output it
    * @return mixed
    */
    public function exportData($doc)
    {
        global $Language;
        $rs = null;
        $this->TotalRecords = $this->listRecordCount();

        // Export all
        if ($this->ExportAll) {
            if (Config("EXPORT_ALL_TIME_LIMIT") >= 0) {
                @set_time_limit(Config("EXPORT_ALL_TIME_LIMIT"));
            }
            $this->DisplayRecords = $this->TotalRecords;
            $this->StopRecord = $this->TotalRecords;
        } else { // Export one page only
            $this->setupStartRecord(); // Set up start record position
            // Set the last record to display
            if ($this->DisplayRecords <= 0) {
                $this->StopRecord = $this->TotalRecords;
            } else {
                $this->StopRecord = $this->StartRecord + $this->DisplayRecords - 1;
            }
        }
        $rs = $this->loadRecordset($this->StartRecord - 1, $this->DisplayRecords <= 0 ? $this->TotalRecords : $this->DisplayRecords);
        if (!$rs || !$doc) {
            RemoveHeader("Content-Type"); // Remove header
            RemoveHeader("Content-Disposition");
            $this->showMessage();
            return;
        }
        $this->StartRecord = 1;
        $this->StopRecord = $this->DisplayRecords <= 0 ? $this->TotalRecords : $this->DisplayRecords;

        // Call Page Exporting server event
        $doc->ExportCustom = !$this->pageExporting($doc);

        // Page header
        $header = $this->PageHeader;
        $this->pageDataRendering($header);
        $doc->Text .= $header;
        $this->exportDocument($doc, $rs, $this->StartRecord, $this->StopRecord, "");
        $rs->free();

        // Page footer
        $footer = $this->PageFooter;
        $this->pageDataRendered($footer);
        $doc->Text .= $footer;

        // Export header and footer
        $doc->exportHeaderAndFooter();

        // Call Page Exported server event
        $this->pageExported($doc);
    }

    // Set up Breadcrumb
    protected function setupBreadcrumb()
    {
        global $Breadcrumb, $Language;
        $Breadcrumb = new Breadcrumb("index");
        $url = CurrentUrl();
        $url = preg_replace('/\?cmd=reset(all){0,1}$/i', '', $url); // Remove cmd=reset(all)
        $Breadcrumb->add("list", $this->TableVar, $url, "", $this->TableVar, true);
    }

    // Setup lookup options
    public function setupLookupOptions($fld)
    {
        if ($fld->Lookup && $fld->Lookup->Options === null) {
            // Get default connection and filter
            $conn = $this->getConnection();
            $lookupFilter = "";

            // No need to check any more
            $fld->Lookup->Options = [];

            // Set up lookup SQL and connection
            switch ($fld->FieldVar) {
                case "x_DistrictCode":
                    break;
                case "x_GRNCode":
                    break;
                case "x_SupplierCode":
                    break;
                case "x_CampCode":
                    break;
                case "x_VillageCode":
                    break;
                case "x_Group":
                    break;
                case "x_Gender":
                    break;
                case "x_Type":
                    break;
                default:
                    $lookupFilter = "";
                    break;
            }

            // Always call to Lookup->getSql so that user can setup Lookup->Options in Lookup_Selecting server event
            $sql = $fld->Lookup->getSql(false, "", $lookupFilter, $this);

            // Set up lookup cache
            if (!$fld->hasLookupOptions() && $fld->UseLookupCache && $sql != "" && count($fld->Lookup->Options) == 0 && count($fld->Lookup->FilterFields) == 0) {
                $totalCnt = $this->getRecordCount($sql, $conn);
                if ($totalCnt > $fld->LookupCacheCount) { // Total count > cache count, do not cache
                    return;
                }
                $rows = $conn->executeQuery($sql)->fetchAll();
                $ar = [];
                foreach ($rows as $row) {
                    $row = $fld->Lookup->renderViewRow($row, Container($fld->Lookup->LinkTable));
                    $key = $row["lf"];
                    if (IsFloatType($fld->Type)) { // Handle float field
                        $key = (float)$key;
                    }
                    $ar[strval($key)] = $row;
                }
                $fld->Lookup->Options = $ar;
            }
        }
    }

    // Set up starting record parameters
    public function setupStartRecord()
    {
        if ($this->DisplayRecords == 0) {
            return;
        }
        $pageNo = Get(Config("TABLE_PAGE_NUMBER"));
        $startRec = Get(Config("TABLE_START_REC"));
        $infiniteScroll = ConvertToBool(Param("infinitescroll"));
        if ($pageNo !== null) { // Check for "pageno" parameter first
            $pageNo = ParseInteger($pageNo);
            if (is_numeric($pageNo)) {
                $this->StartRecord = ($pageNo - 1) * $this->DisplayRecords + 1;
                if ($this->StartRecord <= 0) {
                    $this->StartRecord = 1;
                } elseif ($this->StartRecord >= (int)(($this->TotalRecords - 1) / $this->DisplayRecords) * $this->DisplayRecords + 1) {
                    $this->StartRecord = (int)(($this->TotalRecords - 1) / $this->DisplayRecords) * $this->DisplayRecords + 1;
                }
            }
        } elseif ($startRec !== null && is_numeric($startRec)) { // Check for "start" parameter
            $this->StartRecord = $startRec;
        } elseif (!$infiniteScroll) {
            $this->StartRecord = $this->getStartRecordNumber();
        }

        // Check if correct start record counter
        if (!is_numeric($this->StartRecord) || intval($this->StartRecord) <= 0) { // Avoid invalid start record counter
            $this->StartRecord = 1; // Reset start record counter
        } elseif ($this->StartRecord > $this->TotalRecords) { // Avoid starting record > total records
            $this->StartRecord = (int)(($this->TotalRecords - 1) / $this->DisplayRecords) * $this->DisplayRecords + 1; // Point to last page first record
        } elseif (($this->StartRecord - 1) % $this->DisplayRecords != 0) {
            $this->StartRecord = (int)(($this->StartRecord - 1) / $this->DisplayRecords) * $this->DisplayRecords + 1; // Point to page boundary
        }
        if (!$infiniteScroll) {
            $this->setStartRecordNumber($this->StartRecord);
        }
    }

    // Get page count
    public function pageCount() {
        return ceil($this->TotalRecords / $this->DisplayRecords);
    }

    // Parse query builder rule
    protected function parseRules($group, $fieldName = "", $itemName = "") {
        $group["condition"] ??= "AND";
        if (!in_array($group["condition"], ["AND", "OR"])) {
            throw new \Exception("Unable to build SQL query with condition '" . $group["condition"] . "'");
        }
        if (!is_array($group["rules"] ?? null)) {
            return "";
        }
        $parts = [];
        foreach ($group["rules"] as $rule) {
            if (is_array($rule["rules"] ?? null) && count($rule["rules"]) > 0) {
                $part = $this->parseRules($rule, $fieldName, $itemName);
                if ($part) {
                    $parts[] = "(" . " " . $part . " " . ")" . " ";
                }
            } else {
                $field = $rule["field"];
                $fld = $this->fieldByParam($field);
                $dbid = $this->Dbid;
                if ($fld instanceof ReportField && is_array($fld->DashboardSearchSourceFields)) {
                    $item = $fld->DashboardSearchSourceFields[$itemName] ?? null;
                    if ($item) {
                        $tbl = Container($item["table"]);
                        $dbid = $tbl->Dbid;
                        $fld = $tbl->Fields[$item["field"]];
                    } else {
                        $fld = null;
                    }
                }
                if ($fld && ($fieldName == "" || $fld->Name == $fieldName)) { // Field name not specified or matched field name
                    $fldOpr = array_search($rule["operator"], Config("CLIENT_SEARCH_OPERATORS"));
                    $ope = Config("QUERY_BUILDER_OPERATORS")[$rule["operator"]] ?? null;
                    if (!$ope || !$fldOpr) {
                        throw new \Exception("Unknown SQL operation for operator '" . $rule["operator"] . "'");
                    }
                    if ($ope["nb_inputs"] > 0 && isset($rule["value"]) && !EmptyValue($rule["value"]) || IsNullOrEmptyOperator($fldOpr)) {
                        $fldVal = $rule["value"];
                        if (is_array($fldVal)) {
                            $fldVal = $fld->isMultiSelect() ? implode(Config("MULTIPLE_OPTION_SEPARATOR"), $fldVal) : $fldVal[0];
                        }
                        $useFilter = $fld->UseFilter; // Query builder does not use filter
                        try {
                            if ($fld instanceof ReportField) { // Search report fields
                                if ($fld->SearchType == "dropdown") {
                                    if (is_array($fldVal)) {
                                        $sql = "";
                                        foreach ($fldVal as $val) {
                                            AddFilter($sql, DropDownFilter($fld, $val, $fldOpr, $dbid), "OR");
                                        }
                                        $parts[] = $sql;
                                    } else {
                                        $parts[] = DropDownFilter($fld, $fldVal, $fldOpr, $dbid);
                                    }
                                } else {
                                    $fld->AdvancedSearch->SearchOperator = $fldOpr;
                                    $fld->AdvancedSearch->SearchValue = $fldVal;
                                    $parts[] = GetReportFilter($fld, false, $dbid);
                                }
                            } else { // Search normal fields
                                if ($fld->isMultiSelect()) {
                                    $parts[] = $fldVal != "" ? GetMultiSearchSql($fld, $fldOpr, ConvertSearchValue($fldVal, $fldOpr, $fld), $this->Dbid) : "";
                                } else {
                                    $fldVal2 = ContainsString($fldOpr, "BETWEEN") ? $rule["value"][1] : ""; // BETWEEN
                                    if (is_array($fldVal2)) {
                                        $fldVal2 = implode(Config("MULTIPLE_OPTION_SEPARATOR"), $fldVal2);
                                    }
                                    $fld->AdvancedSearch->SearchValue = ConvertSearchValue($fldVal, $fldOpr, $fld);
                                    $fld->AdvancedSearch->SearchValue2 = ConvertSearchValue($fldVal2, $fldOpr, $fld);
                                    $parts[] = GetSearchSql(
                                        $fld,
                                        $fld->AdvancedSearch->SearchValue, // SearchValue
                                        $fldOpr,
                                        "", // $fldCond not used
                                        $fld->AdvancedSearch->SearchValue2, // SearchValue2
                                        "", // $fldOpr2 not used
                                        $this->Dbid
                                    );
                                }
                            }
                        } finally {
                            $fld->UseFilter = $useFilter;
                        }
                    }
                }
            }
        }
        $where = "";
        foreach ($parts as $part) {
            AddFilter($where, $part, $group["condition"]);
        }
        if ($where && ($group["not"] ?? false)) {
            $where = "NOT (" . $where . ")";
        }
        return $where;
    }

    // Page Load event
    public function pageLoad()
    {
        //Log("Page Load");
    }

    // Page Unload event
    public function pageUnload()
    {
        //Log("Page Unload");
    }

    // Page Redirecting event
    public function pageRedirecting(&$url)
    {
        // Example:
        //$url = "your URL";
    }

    // Message Showing event
    // $type = ''|'success'|'failure'|'warning'
    public function messageShowing(&$msg, $type)
    {
        if ($type == "success") {
            //$msg = "your success message";
        } elseif ($type == "failure") {
            //$msg = "your failure message";
        } elseif ($type == "warning") {
            //$msg = "your warning message";
        } else {
            //$msg = "your message";
        }
    }

    // Page Render event
    public function pageRender()
    {
        //Log("Page Render");
    }

    // Page Data Rendering event
    public function pageDataRendering(&$header)
    {
        // Example:
        //$header = "your header";
    }

    // Page Data Rendered event
    public function pageDataRendered(&$footer)
    {
        // Example:
        //$footer = "your footer";
    }

    // Page Breaking event
    public function pageBreaking(&$break, &$content)
    {
        // Example:
        //$break = false; // Skip page break, or
        //$content = "<div style=\"break-after:page;\"></div>"; // Modify page break content
    }

    // Form Custom Validate event
    public function formCustomValidate(&$customError)
    {
        // Return error message in $customError
        return true;
    }

    // ListOptions Load event
    public function listOptionsLoad()
    {
        // Example:
        //$opt = &$this->ListOptions->add("new");
        //$opt->Header = "xxx";
        //$opt->OnLeft = true; // Link on left
        //$opt->moveTo(0); // Move to first column
    }

    // ListOptions Rendering event
    public function listOptionsRendering()
    {
        //Container("DetailTableGrid")->DetailAdd = (...condition...); // Set to true or false conditionally
        //Container("DetailTableGrid")->DetailEdit = (...condition...); // Set to true or false conditionally
        //Container("DetailTableGrid")->DetailView = (...condition...); // Set to true or false conditionally
    }

    // ListOptions Rendered event
    public function listOptionsRendered()
    {
        // Example:
        //$this->ListOptions["new"]->Body = "xxx";
    }

    // Row Custom Action event
    public function rowCustomAction($action, $row)
    {
        // Return false to abort
        return true;
    }

    // Page Exporting event
    // $doc = export object
    public function pageExporting(&$doc)
    {
        //$doc->Text = "my header"; // Export header
        //return false; // Return false to skip default export and use Row_Export event
        return true; // Return true to use default export and skip Row_Export event
    }

    // Row Export event
    // $doc = export document object
    public function rowExport($doc, $rs)
    {
        //$doc->Text .= "my content"; // Build HTML with field value: $rs["MyField"] or $this->MyField->ViewValue
    }

    // Page Exported event
    // $doc = export document object
    public function pageExported($doc)
    {
        //$doc->Text .= "my footer"; // Export footer
        //Log($doc->Text);
    }

    // Page Importing event
    public function pageImporting(&$builder, &$options)
    {
        //var_dump($options); // Show all options for importing
        //$builder = fn($workflow) => $workflow->addStep($myStep);
        //return false; // Return false to skip import
        return true;
    }

    // Row Import event
    public function rowImport(&$row, $cnt)
    {
        //Log($cnt); // Import record count
        //var_dump($row); // Import row
        //return false; // Return false to skip import
        return true;
    }

    // Page Imported event
    public function pageImported($obj, $results)
    {
        //var_dump($obj); // Workflow result object
        //var_dump($results); // Import results
    }
}
